#ifndef MVIMPACT_BARCODE_IMPL_H_
#define MVIMPACT_BARCODE_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace barcode {


inline void BarcodeConfiguration::setSpacing(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_SPACING, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getSpacing() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_SPACING, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline void BarcodeConfiguration::setContrast(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_CONTRAST, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getContrast() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_CONTRAST, 0, 0, &value))
			throwImpactError();
		return value;
	}


inline void BarcodeConfiguration::setGapSize(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_GAP_SIZE, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getGapSize() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_GAP_SIZE, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline void BarcodeConfiguration::setMinGaps(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_MIN_GAPS, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getMinGaps() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_MIN_GAPS, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline void BarcodeConfiguration::setNoiseThreshold(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_NOISE_THRESHOLD, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getNoiseThreshold() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_NOISE_THRESHOLD, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline void BarcodeConfiguration::setDirections(
		int value) 
	{
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_DIRECTIONS, value))
			throwImpactError();
	}

inline int BarcodeConfiguration::getDirections() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_DIRECTIONS, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline void BarcodeConfiguration::setFilterOrphanedRays(
		bool value) 
	{
		mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_FILTER_ORPHANED_RAYS, value_))
			throwImpactError();
	}

inline bool BarcodeConfiguration::getFilterOrphanedRays() const
	{
		mvIMPACT_C::IPL_BOOL value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_FILTER_ORPHANED_RAYS, 0, 0, &value))
			throwImpactError();
		return value ? true : false;
	}

inline void BarcodeConfiguration::setFilterOrphanedDecodes(
		bool value) 
	{
		mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_FILTER_ORPHANED_DECODES, value_))
			throwImpactError();
	}

inline bool BarcodeConfiguration::getFilterOrphanedDecodes() const
	{
		mvIMPACT_C::IPL_BOOL value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_FILTER_ORPHANED_DECODES, 0, 0, &value))
			throwImpactError();
		return value ? true : false;
	}

inline void BarcodeConfiguration::setCheckPrintQuality(
		bool value) 
	{
		mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
		if (!mvIMPACT_C::CR1D_Control(getHandle(), mvIMPACT_C::CR1D_CFG_CHECK_PRINT_QUALITY, value_))
			throwImpactError();
	}

inline bool BarcodeConfiguration::getCheckPrintQuality() const
	{
		mvIMPACT_C::IPL_BOOL value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_CFG_CHECK_PRINT_QUALITY, 0, 0, &value))
			throwImpactError();
		return value ? true : false;
	}

inline BarcodeConfiguration::BarcodeConfiguration() 
		: handle_(unassigned), counter_(new int(1))
	{ 
		assert(unassigned == getHandle());
		if (!mvIMPACT_C::CR1D_BufAlloc(&handle_))
			throwImpactError();
		assert(unassigned != getHandle());
	}

inline BarcodeConfiguration::BarcodeConfiguration(const BarcodeConfiguration& r) 
	{
		copy(r);
	}

inline BarcodeConfiguration& BarcodeConfiguration::operator=(
		const BarcodeConfiguration& r)
	{ 
		if (&r != this)	{
			dispose();
			copy(r);
		}
		return *this;
	}

inline BarcodeConfiguration::~BarcodeConfiguration()
	{
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}


inline mvIMPACT_C::CR1D_BUFHANDLE BarcodeConfiguration::getHandle  () const 
	{ return handle_; }

inline int BarcodeConfiguration::getUseCount() const 
	{ return *counter_; }

inline bool BarcodeConfiguration::getIsUnique   () const { return 1 == *counter_; }

inline BarcodeConfiguration::BarcodeConfiguration(
		mvIMPACT_C::CR1D_BUFHANDLE bufhandle) 
		: handle_(unassigned), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		handle_ = bufhandle;
		assert(unassigned != getHandle());
	}

inline void BarcodeConfiguration::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			if (unassigned != getHandle())
			{
				assert(unassigned != getHandle());
				if (!mvIMPACT_C::CR1D_BufFree(getHandle())) throwImpactError();
				handle_ = unassigned;
				assert(unassigned == getHandle());
			}
		}
	}

inline void BarcodeConfiguration::copy(const BarcodeConfiguration& src)
{
	handle_ = src.handle_;
	counter_ = src.counter_;
	
	(*counter_)++;	
}

inline int BarcodeResults::getNumberOfCandidates() const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_NUMBER_OF_CANDIDATES, 0, 0, &value))
			throwImpactError();
		return value;
	}

inline int BarcodeResults::getNumberOfCodes(int candidate) const
	{
		int value = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_NUMBER_OF_CODES, candidate, 0, &value))
			throwImpactError();
		return value;
	}

inline std::string BarcodeResults::getCodeType(int candidate, int code) const
	{
		mvIMPACT_C::IPL_ULONG length = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CODETYPE_LENGTH, candidate, code, &length))
			throwImpactError();
		DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CODETYPE, candidate, code, value.getPointer()))
			throwImpactError();
		return std::string(value.getPointer());
	}

inline std::string BarcodeResults::getCode(int candidate, int code) const
	{
		mvIMPACT_C::IPL_ULONG length = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CODEDATA_LENGTH, candidate, code, &length))
			throwImpactError();
		DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CODEDATA, candidate, code, value.getPointer()))
			throwImpactError();
		return std::string(value.getPointer());
	}

inline std::string BarcodeResults::getChecksum(int candidate, int code) const
	{
		mvIMPACT_C::IPL_ULONG length = 0;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CHECKSUM_LENGTH, candidate, code, &length))
			throwImpactError();
		DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_CHECKSUM, candidate, code, value.getPointer()))
			throwImpactError();
		return std::string(value.getPointer());
	}

inline std::vector<Point> BarcodeResults::getLocation(int candidate) const
	{
		DynArray<mvIMPACT_C::IPL_POINT, Point> iplResult(4);
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_LOCATION, candidate, 0, iplResult.getPointer()))
			throwImpactError();
		return iplResult.asVector();
	}

inline bool BarcodeResults::getDirection(int candidate) const
	{
		mvIMPACT_C::IPL_BOOL value;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_DIRECTION, candidate, 0, &value))
			throwImpactError();
		return value ? true : false;
	}

inline double BarcodeResults::getOverprint(int candidate) const
	{
		mvIMPACT_C::IPL_DOUBLE value;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_OVERPRINT, candidate, 0, &value))
			throwImpactError();
		return value;
	}

inline double BarcodeResults::getTotalDecodingTime () const
	{
		mvIMPACT_C::IPL_DOUBLE t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_TOTAL_DECODING_TIME_AS_DOUBLE, 0, 0, &t))
			throwImpactError();
		return t;
	}

inline double BarcodeResults::getDecodingTime (int candidate) const
	{
		mvIMPACT_C::IPL_DOUBLE t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_DECODING_TIME_AS_DOUBLE, candidate, 0, &t))
			throwImpactError();
		return t;
	}

inline int BarcodeResults::getReflectanceMinimum (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_REFLECTANCE_MINIMUM, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getReflectanceMaximum (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_REFLECTANCE_MAXIMUM, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getSymbolContrast (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_SYMBOL_CONTRAST, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getGlobalThreshold (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_GLOBAL_THRESHOLD, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getMinimumEdgeContrast (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MINIMUM_EDGE_CONTRAST, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline double BarcodeResults::getModulation (int candidate) const
	{
		mvIMPACT_C::IPL_DOUBLE t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MODULATION, candidate, 0, &t))
			throwImpactError();
		return (double)t;
	}

inline int BarcodeResults::getMaximumNonUniformity (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MAXIMUM_NON_UNIFORMITY, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline bool BarcodeResults::getEdgeDeterminationTest (int candidate) const
	{
		mvIMPACT_C::IPL_BOOL t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_EDGE_DETERMINATION_TEST, candidate, 0, &t))
			throwImpactError();
		return t ? true : false;
	}

inline bool BarcodeResults::getMinimumReflectanceTest (int candidate) const
	{
		mvIMPACT_C::IPL_BOOL t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MINIMUM_REFLECTANCE_TEST, candidate, 0, &t))
			throwImpactError();
		return t ? true : false;
	}

inline bool BarcodeResults::getMinimumEdgeContrastTest (int candidate) const
	{
		mvIMPACT_C::IPL_BOOL t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MINIMUM_EDGE_CONTRAST_TEST, candidate, 0, &t))
			throwImpactError();
		return t ? true : false;
	}

inline int BarcodeResults::getSymbolContrastGrade (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_SYMBOL_CONTRAST_GRADE, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getModulationGrade (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_MODULATION_GRADE, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getDefectsGrade (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_DEFECTS_GRADE, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getDecodabilityGrade (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_DECODABILITY_GRADE, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline int BarcodeResults::getScanGrade (int candidate) const
	{
		mvIMPACT_C::IPL_ULONG t;
		if (!mvIMPACT_C::CR1D_Inquire(getHandle(), mvIMPACT_C::CR1D_GET_SCAN_GRADE, candidate, 0, &t))
			throwImpactError();
		return (int)t;
	}

inline BarcodeResults::BarcodeResults() 
		: handle_(unassigned), counter_(new int(1))
	{ 
	}

inline BarcodeResults::BarcodeResults(const BarcodeResults& r) 
	{
		copy(r);
	}

inline BarcodeResults& BarcodeResults::operator=(const BarcodeResults& r)
	{ 
		if (&r != this)	
		{
			dispose();
			copy(r);
		}
		return *this;
	}

inline BarcodeResults::~BarcodeResults()
	{
		// a destructor must never throw
		try
		{
			dispose();
		}
		catch(...)
		{
			mvIMPACT_C::IPL_Trace(" caught exception in destructor");
		}
	}

inline mvIMPACT_C::CR1D_BUFHANDLE BarcodeResults::getHandle  () const 
	{ return handle_; }

inline int BarcodeResults::getUseCount() const 
	{ return *counter_; }

inline bool BarcodeResults::getIsUnique   () const { return 1 == *counter_; }

inline BarcodeResults::BarcodeResults(
		mvIMPACT_C::CR1D_BUFHANDLE bufhandle) 
		: handle_(unassigned), counter_(new int(1))
	{
		assert(unassigned == getHandle());
		handle_ = bufhandle;
		assert(unassigned != getHandle());
	}

inline void BarcodeResults::dispose()
	{
		if (0 == --*counter_)
		{
			delete counter_;
			if (unassigned != getHandle())
			{
				assert(unassigned != getHandle());
				if (!mvIMPACT_C::CR1D_BufFree(getHandle()))
					throwImpactError();
				handle_ = unassigned;
				assert(unassigned == getHandle());
			}
		}
	}


inline void BarcodeResults::copy(const BarcodeResults& src)
{
	counter_ = src.counter_;
	handle_ = src.handle_;
	
	(*counter_)++;
}


inline BarcodeResults BarcodeResults::barcodeResultsFactory(
	mvIMPACT_C::CR1D_BUFHANDLE buffer_handle) 
//-----------------------------------------------------------------------------
{
	if (IPL_DONT_CARE == buffer_handle)
		return BarcodeResults();

	return BarcodeResults(buffer_handle);
}

inline BarcodeResults decodeBC(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_Decode(S.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

#ifndef I386_LINUX
inline BarcodeResults decodeBC(
	const Image& S, 
	Window& W,
	LineArrayOfInterest& L)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeAndVisualizeRuns(
		S.getHandle(), &C, &D, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	return BarcodeResults::barcodeResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void decodeBC(
	BarcodeResults& D,
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_Decode(S.getHandle(), &C, &hD))
		throwImpactError();
	
	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}

#ifndef I386_LINUX
inline void decodeBC(
	BarcodeResults& D,
	const Image& S,
	Window& W,
	LineArrayOfInterest& L) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeAndVisualizeRuns(
		S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }
}
#endif // !I386_LINUX^M

inline BarcodeResults decodeBC(
	const Image& S,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_Decode(S.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

#ifndef I386_LINUX
inline BarcodeResults decodeBC(
	const Image& S,
	const BarcodeConfiguration& config, 
	Window& W,
	LineArrayOfInterest& L) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeAndVisualizeRuns(
		S.getHandle(), &C, &D, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	return BarcodeResults::barcodeResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void decodeBC(
	BarcodeResults& D, 
	const Image& S,
	const BarcodeConfiguration& config) 
	//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_Decode(
		S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }
}

#ifndef I386_LINUX
inline void decodeBC(
	BarcodeResults& D, 
	const Image& S,
	const BarcodeConfiguration& config,
	Window& W,
	LineArrayOfInterest& L) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeAndVisualizeRuns(
		S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }
}
#endif // !I386_LINUX^M


inline BarcodeResults decodeBC(
	const Profile& P) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeProfile(P.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

inline void decodeBC(
	BarcodeResults& D,
	const Profile& P) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_DecodeProfile(P.getHandle(), &C, &hD))
		throwImpactError();
	
	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}

inline BarcodeResults decodeBC(
	const Profile& P,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeProfile(P.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

inline void decodeBC(
	BarcodeResults& D, 
	const Profile& P,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_DecodeProfile(P.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}


inline BarcodeResults decodeBCAt(
	const Image& S,
	const Line& line) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::IPL_LINE L;
	L.x0 = line.getP0().getX();
	L.y0 = line.getP0().getY();
	L.x1 = line.getP1().getX();
	L.y1 = line.getP1().getY();

	if (!mvIMPACT_C::CR1D_DecodeAtLocation(S.getHandle(), &L, &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}


inline void decodeBCAt(
	BarcodeResults& D,
	const Image& S,
	const Line& line) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::IPL_LINE L;
	L.x0 = line.getP0().getX();
	L.y0 = line.getP0().getY();
	L.x1 = line.getP1().getX();
	L.y1 = line.getP1().getY();

	if (!mvIMPACT_C::CR1D_DecodeAtLocation(S.getHandle(), &L, &C, &hD))
		throwImpactError();
	
	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}


inline BarcodeResults decodeBCAt(
	const Image& S,
	const BarcodeConfiguration& config,
	const Line& line)  
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::IPL_LINE L;
	L.x0 = line.getP0().getX();
	L.y0 = line.getP0().getY();
	L.x1 = line.getP1().getX();
	L.y1 = line.getP1().getY();

	if (!mvIMPACT_C::CR1D_DecodeAtLocation(S.getHandle(), &L, &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}


inline void decodeBCAt(
	BarcodeResults& D, 
	const Image& S,
	const BarcodeConfiguration& config,
	const Line& line)  
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::IPL_LINE L;
	L.x0 = line.getP0().getX();
	L.y0 = line.getP0().getY();
	L.x1 = line.getP1().getX();
	L.y1 = line.getP1().getY();

	if (!mvIMPACT_C::CR1D_DecodeAtLocation(S.getHandle(), &L, &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}


inline BarcodeResults decodeBCFromDistances(
	const Profile& P) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeDistances(P.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

inline void decodeBCFromDistances(
	BarcodeResults& D,
	const Profile& P) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_DecodeDistances(P.getHandle(), &C, &hD))
		throwImpactError();
	
	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}

inline BarcodeResults decodeBCFromDistances(
	const Profile& P,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_DecodeDistances(P.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

inline void decodeBCFromDistances(
	BarcodeResults& D, 
	const Profile& P,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_DecodeDistances(P.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) { D = BarcodeResults::barcodeResultsFactory(hD); }
}


inline BarcodeResults locateBC(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_Locate(S.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

#ifndef I386_LINUX
inline BarcodeResults locateBC(
	const Image& S, 
	Window& W,
	LineArrayOfInterest& L) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	return BarcodeResults::barcodeResultsFactory(D);
}
#endif // !I386_LINUX^M

inline void locateBC(
	BarcodeResults& D,
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_Locate(S.getHandle(), &C, &hD))
		throwImpactError();
	
	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }

}

#ifndef I386_LINUX
inline void locateBC(
	BarcodeResults& D,
	const Image& S, 
	Window& W,
	LineArrayOfInterest& L)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }

}
#endif // !I386_LINUX^M

inline void locateBC(
	BarcodeResults& D, 
	const Image& S,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::CR1D_Locate(S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }
}

#ifndef I386_LINUX
inline void locateBC(
	BarcodeResults& D, 
	const Image& S,
	const BarcodeConfiguration& config, 
	Window& W,
	LineArrayOfInterest& L) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &hD, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	if(hD != D.getHandle()) 
		{ D = BarcodeResults::barcodeResultsFactory(hD); }
}
#endif // !I386_LINUX^M

inline 	BarcodeResults locateBC(
	const Image& S,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_Locate(S.getHandle(), &C, &D))
		throwImpactError();
	return BarcodeResults::barcodeResultsFactory(D);
}

#ifndef I386_LINUX
inline 	BarcodeResults locateBC(
	const Image& S,
	const BarcodeConfiguration& config, 
	Window& W,
	LineArrayOfInterest& L)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::DISP_OOIHANDLE hOoi = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_LocateAndVisualizeRuns(
			S.getHandle(), &C, &D, W.getOoiSetHandle(), &hOoi, &W))
		throwImpactError();

	LineArrayOfInterest la(W, hOoi);
	L = la;

	return BarcodeResults::barcodeResultsFactory(D);
}
#endif // !I386_LINUX^M



inline Image edgeMapBC(
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_EdgeMap(S.getHandle(), &C, &D))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void edgeMapBC(
	Image& D, 
	const Image& S) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE C = IPL_DONT_CARE;
	if (!mvIMPACT_C::CR1D_EdgeMap(S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = Image::imageFactory(hD); }
}

inline 	Image edgeMapBC(
	const Image& S,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE D = IPL_DONT_CARE;
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	if (!mvIMPACT_C::CR1D_EdgeMap(S.getHandle(), &C, &D))
		throwImpactError();
	return Image::imageFactory(D);
}

inline void edgeMapBC(
	Image& D, 
	const Image& S,
	const BarcodeConfiguration& config) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	mvIMPACT_C::CR1D_BUFHANDLE C = config.getHandle();
	if (!mvIMPACT_C::CR1D_EdgeMap(S.getHandle(), &C, &hD))
		throwImpactError();

	if(hD != D.getHandle()) 
		{ D = Image::imageFactory(hD); }
}


inline void addBCDecoder(
	const std::string& decoder,
	int index)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::CR1D_AddDecoder(decoder.c_str(), index))
		throwImpactError();
}





inline void removeBCDecoder(
	const std::string& decoder)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::CR1D_RemoveDecoder(decoder.c_str()))
		throwImpactError();
}





inline int getBCDecoderCount()
//-----------------------------------------------------------------------------
{
	int value;
	if (!mvIMPACT_C::CR1D_InquireDecoder(0, mvIMPACT_C::CR1D_DECODER_COUNT, &value))
		throwImpactError();
	return value;
}





inline std::string getBCDecoderName(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG length = 0;
	if (!mvIMPACT_C::CR1D_InquireDecoder(index, mvIMPACT_C::CR1D_DECODER_NAME_LENGTH, &length))
		throwImpactError();
	DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
	if (!mvIMPACT_C::CR1D_InquireDecoder(
			index, 
			mvIMPACT_C::CR1D_DECODER_NAME, 
			value.getPointer()))
	{
		throwImpactError();
	}
	return std::string(value.getPointer());
}


inline std::string getBCDecoderFileName(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_ULONG length = 0;
	if (!mvIMPACT_C::CR1D_InquireDecoder(index, mvIMPACT_C::CR1D_DECODER_FILENAME_LENGTH, &length))
		throwImpactError();
	DynArray<mvIMPACT_C::IPL_CHAR> value((int)(length+1), (mvIMPACT_C::IPL_CHAR)0);
	if (!mvIMPACT_C::CR1D_InquireDecoder(
			index, 
			mvIMPACT_C::CR1D_DECODER_FILENAME, 
			value.getPointer()))
	{
		throwImpactError();
	}
	return std::string(value.getPointer());
}


inline bool getBCTryNextOnSuccess(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BOOL value;
	if (!mvIMPACT_C::CR1D_InquireDecoder(index, mvIMPACT_C::CR1D_DECODER_TRY_NEXT_ON_SUCCESS, &value))
		throwImpactError();
	return value ? true : false;
}





inline void setBCTryNextOnSuccess(
	int index,
	bool value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BOOL value_ = value ? IPL_TRUE : IPL_FALSE;
	if (!mvIMPACT_C::CR1D_ControlDecoder(index, mvIMPACT_C::CR1D_DECODER_TRY_NEXT_ON_SUCCESS, &value_))
		throwImpactError();
}





inline ChecksummingMode getBCChecksumMode(
	int index)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_CHECKSUMFLAGS value;
	if (!mvIMPACT_C::CR1D_InquireDecoder(index, mvIMPACT_C::CR1D_DECODER_CHECKSUM, &value))
		throwImpactError();
	return ChecksummingMode(value);
}





inline void setBCChecksumMode(
	int index,
	ChecksummingMode value)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::CR1D_CHECKSUMFLAGS value_= static_cast<mvIMPACT_C::CR1D_CHECKSUMFLAGS>(value);
	if (!mvIMPACT_C::CR1D_ControlDecoder(index, mvIMPACT_C::CR1D_DECODER_CHECKSUM, &value_))
		throwImpactError();
}


inline bool checkLicenseBC()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::CR1D_CheckLic());
}

inline std::string getVersionBC()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::CR1D_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline std::string getVersionBCDecoder(const std::string& decoder)
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::CR1D_GetVersionDecoder(decoder.c_str(), sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}


	} // namespace barcode
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_BARCODE_IMPL_H_
