#ifndef MVIMPACT_COLOR_H_
#define MVIMPACT_COLOR_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON



#ifndef  DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_color.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS


namespace mvIMPACT {

        /// \namespace mvIMPACT::color This namespace contains all classes and functions from mvIMPACT Color.
	namespace color {
	


	  /**
	   * \brief The color space conversion specifier.
	   *
	   * Supported color space conversion codes.
	   * Combines one of the source color spaces with one of the destination color 
	   * spaces. Includes only the combinations, where the source and destination 
	   * color spaces are different.
	   */
	  enum ColorSpaceConversion
	    {
	      /// Convert from CMY to GRAY colorspace.
	      cscCmy2gray=mvIMPACT_C::COL_CMY2GRAY,
	      /// Convert from CMY to HLS colorspace.
	      cscCmy2hls=mvIMPACT_C::COL_CMY2HLS,
	      /// Convert from CMY to HSV colorspace.
	      cscCmy2hsv=mvIMPACT_C::COL_CMY2HSV,
	      /// Convert from CMY to RGB colorspace.
	      cscCmy2rgb=mvIMPACT_C::COL_CMY2RGB,
	      /// Convert from CMY to XYZ601 colorspace.
	      cscCmy2xyz601=mvIMPACT_C::COL_CMY2XYZ601,
	      /// Convert from CMY to XYZ709 colorspace.
	      cscCmy2xyz709=mvIMPACT_C::COL_CMY2XYZ709,
	      /// Convert from CMY to XYZITU colorspace.
	      cscCmy2xyzitu=mvIMPACT_C::COL_CMY2XYZITU,
	      /// Convert from CMY to YCC601 colorspace.
	      cscCmy2ycc601=mvIMPACT_C::COL_CMY2YCC601,
	      /// Convert from CMY to YIQ colorspace.
	      cscCmy2yiq=mvIMPACT_C::COL_CMY2YIQ,
	      /// Convert from CMY to YUV colorspace.
	      cscCmy2yuv=mvIMPACT_C::COL_CMY2YUV,
	      /// Convert from GRAY to CMY colorspace.
	      cscGray2cmy=mvIMPACT_C::COL_GRAY2CMY,
	      /// Convert from GRAY to HLS colorspace.
	      cscGray2hls=mvIMPACT_C::COL_GRAY2HLS,
	      /// Convert from GRAY to HSV colorspace.
	      cscGray2hsv=mvIMPACT_C::COL_GRAY2HSV,
	      /// Convert from GRAY to RGB colorspace.
	      cscGray2rgb=mvIMPACT_C::COL_GRAY2RGB,
	      /// Convert from GRAY to XYZ601 colorspace.
	      cscGray2xyz601=mvIMPACT_C::COL_GRAY2XYZ601,
	      /// Convert from GRAY to XYZ709 colorspace.
	      cscGray2xyz709=mvIMPACT_C::COL_GRAY2XYZ709,
	      /// Convert from GRAY to XYZITU colorspace.
	      cscGray2xyzitu=mvIMPACT_C::COL_GRAY2XYZITU,
	      /// Convert from GRAY to YCC601 colorspace.
	      cscGray2ycc601=mvIMPACT_C::COL_GRAY2YCC601,
	      /// Convert from GRAY to YIQ colorspace.
	      cscGray2yiq=mvIMPACT_C::COL_GRAY2YIQ,
	      /// Convert from GRAY to YUV colorspace.
	      cscGray2yuv=mvIMPACT_C::COL_GRAY2YUV,
	      /// Convert from HLS to CMY colorspace.
	      cscHls2cmy=mvIMPACT_C::COL_HLS2CMY,
	      /// Convert from HLS to GRAY colorspace.
	      cscHls2gray=mvIMPACT_C::COL_HLS2GRAY,
	      /// Convert from HLS to HSV colorspace.
	      cscHls2hsv=mvIMPACT_C::COL_HLS2HSV,
	      /// Convert from HLS to RGB colorspace.
	      cscHls2rgb=mvIMPACT_C::COL_HLS2RGB,
	      /// Convert from HLS to XYZ601 colorspace.
	      cscHls2xyz601=mvIMPACT_C::COL_HLS2XYZ601,
	      /// Convert from HLS to XYZ709 colorspace.
	      cscHls2xyz709=mvIMPACT_C::COL_HLS2XYZ709,
	      /// Convert from HLS to XYZITU colorspace.
	      cscHls2xyzitu=mvIMPACT_C::COL_HLS2XYZITU,
	      /// Convert from HLS to YCC601 colorspace.
	      cscHls2ycc601=mvIMPACT_C::COL_HLS2YCC601,
	      /// Convert from HLS to YIQ colorspace.
	      cscHls2yiq=mvIMPACT_C::COL_HLS2YIQ,
	      /// Convert from HLS to YUV colorspace.
	      cscHls2yuv=mvIMPACT_C::COL_HLS2YUV,
	      /// Convert from HSV to CMY colorspace.
	      cscHsv2cmy=mvIMPACT_C::COL_HSV2CMY,
	      /// Convert from HSV to GRAY colorspace.
	      cscHsv2gray=mvIMPACT_C::COL_HSV2GRAY,
	      /// Convert from HSV to HLS colorspace.
	      cscHsv2hls=mvIMPACT_C::COL_HSV2HLS,
	      /// Convert from HSV to RGB colorspace.
	      cscHsv2rgb=mvIMPACT_C::COL_HSV2RGB,
	      /// Convert from HSV to XYZ601 colorspace.
	      cscHsv2xyz601=mvIMPACT_C::COL_HSV2XYZ601,
	      /// Convert from HSV to XYZ709 colorspace.
	      cscHsv2xyz709=mvIMPACT_C::COL_HSV2XYZ709,
	      /// Convert from HSV to XYZITU colorspace.
	      cscHsv2xyzitu=mvIMPACT_C::COL_HSV2XYZITU,
	      /// Convert from HSV to YCC601 colorspace.
	      cscHsv2ycc601=mvIMPACT_C::COL_HSV2YCC601,
	      /// Convert from HSV to YIQ colorspace.
	      cscHsv2yiq=mvIMPACT_C::COL_HSV2YIQ,
	      /// Convert from HSV to YUV colorspace.
	      cscHsv2yuv=mvIMPACT_C::COL_HSV2YUV,
	      /// Convert from RGB to CMY colorspace.
	      cscRgb2cmy=mvIMPACT_C::COL_RGB2CMY,
	      /// Convert from RGB to GRAY colorspace.
	      cscRgb2gray=mvIMPACT_C::COL_RGB2GRAY,
	      /// Convert from RGB to HLS colorspace.
	      cscRgb2hls=mvIMPACT_C::COL_RGB2HLS,
	      /// Convert from RGB to HSV colorspace.
	      cscRgb2hsv=mvIMPACT_C::COL_RGB2HSV,
	      /// Convert from RGB to XYZ601 colorspace.
	      cscRgb2xyz601=mvIMPACT_C::COL_RGB2XYZ601,
	      /// Convert from RGB to XYZ709 colorspace.
	      cscRgb2xyz709=mvIMPACT_C::COL_RGB2XYZ709,
	      /// Convert from RGB to XYZITU colorspace.
	      cscRgb2xyzitu=mvIMPACT_C::COL_RGB2XYZITU,
	      /// Convert from RGB to YCC601 colorspace.
	      cscRgb2ycc601=mvIMPACT_C::COL_RGB2YCC601,
	      /// Convert from RGB to YIQ colorspace.
	      cscRgb2yiq=mvIMPACT_C::COL_RGB2YIQ,
	      /// Convert from RGB to YUV colorspace.
	      cscRgb2yuv=mvIMPACT_C::COL_RGB2YUV,
	      /// Convert from XYZ601 to CMY colorspace.
	      cscXyz6012cmy=mvIMPACT_C::COL_XYZ6012CMY,
	      /// Convert from XYZ601 to GRAY colorspace.
	      cscXyz6012gray=mvIMPACT_C::COL_XYZ6012GRAY,
	      /// Convert from XYZ601 to HLS colorspace.
	      cscXyz6012hls=mvIMPACT_C::COL_XYZ6012HLS,
	      /// Convert from XYZ601 to HSV colorspace.
	      cscXyz6012hsv=mvIMPACT_C::COL_XYZ6012HSV,
	      /// Convert from XYZ601 to RGB colorspace.
	      cscXyz6012rgb=mvIMPACT_C::COL_XYZ6012RGB,
	      /// Convert from XYZ601 to XYZ709 colorspace.
	      cscXyz6012xyz709=mvIMPACT_C::COL_XYZ6012XYZ709,
	      /// Convert from XYZ601 to XYZITU colorspace.
	      cscXyz6012xyzitu=mvIMPACT_C::COL_XYZ6012XYZITU,
	      /// Convert from XYZ601 to YCC601 colorspace.
	      cscXyz6012ycc601=mvIMPACT_C::COL_XYZ6012YCC601,
	      /// Convert from XYZ601 to YIQ colorspace.
	      cscXyz6012yiq=mvIMPACT_C::COL_XYZ6012YIQ,
	      /// Convert from XYZ601 to YUV colorspace.
	      cscXyz6012yuv=mvIMPACT_C::COL_XYZ6012YUV,
	      /// Convert from XYZ709 to CMY colorspace.
	      cscXyz7092cmy=mvIMPACT_C::COL_XYZ7092CMY,
	      /// Convert from XYZ709 to GRAY colorspace.
	      cscXyz7092gray=mvIMPACT_C::COL_XYZ7092GRAY,
	      /// Convert from XYZ709 to HLS colorspace.
	      cscXyz7092hls=mvIMPACT_C::COL_XYZ7092HLS,
	      /// Convert from XYZ709 to HSV colorspace.
	      cscXyz7092hsv=mvIMPACT_C::COL_XYZ7092HSV,
	      /// Convert from XYZ709 to RGB colorspace.
	      cscXyz7092rgb=mvIMPACT_C::COL_XYZ7092RGB,
	      /// Convert from XYZ709 to XYZ601 colorspace.
	      cscXyz7092xyz601=mvIMPACT_C::COL_XYZ7092XYZ601,
	      /// Convert from XYZ709 to XYZITU colorspace.
	      cscXyz7092xyzitu=mvIMPACT_C::COL_XYZ7092XYZITU,
	      /// Convert from XYZ709 to YCC601 colorspace.
	      cscXyz7092ycc601=mvIMPACT_C::COL_XYZ7092YCC601,
	      /// Convert from XYZ709 to YIQ colorspace.
	      cscXyz7092yiq=mvIMPACT_C::COL_XYZ7092YIQ,
	      /// Convert from XYZ709 to YUV colorspace.
	      cscXyz7092yuv=mvIMPACT_C::COL_XYZ7092YUV,
	      /// Convert from XYZITU to CMY colorspace.
	      cscXyzitu2cmy=mvIMPACT_C::COL_XYZITU2CMY,
	      /// Convert from XYZITU to GRAY colorspace.
	      cscXyzitu2gray=mvIMPACT_C::COL_XYZITU2GRAY,
	      /// Convert from XYZITU to HLS colorspace.s
	      cscXyzitu2hls=mvIMPACT_C::COL_XYZITU2HLS,
	      /// Convert from XYZITU to HSV colorspace.
	      cscXyzitu2hsv=mvIMPACT_C::COL_XYZITU2HSV,
	      /// Convert from XYZITU to RGB colorspace.
	      cscXyzitu2rgb=mvIMPACT_C::COL_XYZITU2RGB,
	      /// Convert from XYZITU to XYZ601 colorspace.
	      cscXyzitu2xyz601=mvIMPACT_C::COL_XYZITU2XYZ601,
	      /// Convert from XYZITU to XYZ709 colorspace.
	      cscXyzitu2xyz709=mvIMPACT_C::COL_XYZITU2XYZ709,
	      /// Convert from XYZITU to YCC601 colorspace.
	      cscXyzitu2ycc601=mvIMPACT_C::COL_XYZITU2YCC601,
	      /// Convert from XYZITU to YIQ colorspace.
	      cscXyzitu2yiq=mvIMPACT_C::COL_XYZITU2YIQ,
	      /// Convert from XYZITU to YUV colorspace.
	      cscXyzitu2yuv=mvIMPACT_C::COL_XYZITU2YUV,
	      /// Convert from YCC601 to CMY colorspace.
	      cscYcc6012cmy=mvIMPACT_C::COL_YCC6012CMY,
	      /// Convert from YCC601 to GRAY colorspace.
	      cscYcc6012gray=mvIMPACT_C::COL_YCC6012GRAY,
	      /// Convert from YCC601 to HLS colorspace.
	      cscYcc6012hls=mvIMPACT_C::COL_YCC6012HLS,
	      /// Convert from YCC601 to HSV colorspace.
	      cscYcc6012hsv=mvIMPACT_C::COL_YCC6012HSV,
	      /// Convert from YCC601 to RGB colorspace.
	      cscYcc6012rgb=mvIMPACT_C::COL_YCC6012RGB,
	      /// Convert from YCC601 to XYZ601 colorspace.
	      cscYcc6012xyz601=mvIMPACT_C::COL_YCC6012XYZ601,
	      /// Convert from YCC601 to XYZ709 colorspace.
	      cscYcc6012xyz709=mvIMPACT_C::COL_YCC6012XYZ709,
	      /// Convert from YCC601 to XYZITU colorspace.
	      cscYcc6012xyzitu=mvIMPACT_C::COL_YCC6012XYZITU,
	      /// Convert from YCC601 to YIQ colorspace.
	      cscYcc6012yiq=mvIMPACT_C::COL_YCC6012YIQ,
	      /// Convert from YCC601 to YUV colorspace.
	      cscYcc6012yuv=mvIMPACT_C::COL_YCC6012YUV,
	      /// Convert from YIQ to CMY colorspace.
	      cscYiq2cmy=mvIMPACT_C::COL_YIQ2CMY,
	      /// Convert from YIQ to GRAY colorspace.
	      cscYiq2gray=mvIMPACT_C::COL_YIQ2GRAY,
	      /// Convert from YIQ to HLS colorspace.
	      cscYiq2hls=mvIMPACT_C::COL_YIQ2HLS,
	      /// Convert from YIQ to HSV colorspace.
	      cscYiq2hsv=mvIMPACT_C::COL_YIQ2HSV,
	      /// Convert from YIQ to RGB colorspace.
	      cscYiq2rgb=mvIMPACT_C::COL_YIQ2RGB,
	      /// Convert from YIQ to XYZ601 colorspace.
	      cscYiq2xyz601=mvIMPACT_C::COL_YIQ2XYZ601,
	      /// Convert from YIQ to XYZ709 colorspace.
	      cscYiq2xyz709=mvIMPACT_C::COL_YIQ2XYZ709,
	      /// Convert from YIQ to XYZITU colorspace.
	      cscYiq2xyzitu=mvIMPACT_C::COL_YIQ2XYZITU,
	      /// Convert from YIQ to YCC601 colorspace.
	      cscYiq2ycc601=mvIMPACT_C::COL_YIQ2YCC601,
	      /// Convert from YIQ to YUV colorspace.
	      cscYiq2yuv=mvIMPACT_C::COL_YIQ2YUV,
	      /// Convert from YUV to CMY colorspace.
	      cscYuv2cmy=mvIMPACT_C::COL_YUV2CMY,
	      /// Convert from YUV to GRAY colorspace.
	      cscYuv2gray=mvIMPACT_C::COL_YUV2GRAY,
	      /// Convert from YUV to HLS colorspace.
	      cscYuv2hls=mvIMPACT_C::COL_YUV2HLS,
	      /// Convert from YUV to HSV colorspace.
	      cscYuv2hsv=mvIMPACT_C::COL_YUV2HSV,
	      /// Convert from YUV to RGB colorspace.
	      cscYuv2rgb=mvIMPACT_C::COL_YUV2RGB,
	      /// Convert from YUV to XYZ601 colorspace.
	      cscYuv2xyz601=mvIMPACT_C::COL_YUV2XYZ601,
	      /// Convert from YUV to XYZ709 colorspace.
	      cscYuv2xyz709=mvIMPACT_C::COL_YUV2XYZ709,
	      /// Convert from YUV to XYZITU colorspace.
	      cscYuv2xyzitu=mvIMPACT_C::COL_YUV2XYZITU,
	      /// Convert from YUV to YCC601 colorspace.
	      cscYuv2ycc601=mvIMPACT_C::COL_YUV2YCC601,
	      /// Convert from YUV to YIQ colorspace.
	      cscYuv2yiq=mvIMPACT_C::COL_YUV2YIQ,
		  /// Convert from Bayer Mosaic to RGB colorspace.
		  cscBm2rgb=mvIMPACT_C::COL_BM2RGB,
		  /// Convert from Bayer Mosaic to YUV colorspace.
		  cscBm2yuv=mvIMPACT_C::COL_BM2YUV,
		  /// Convert from Bayer Mosaic to GRAY colorspace.
		  cscBm2gray=mvIMPACT_C::COL_BM2GRAY,
	    };

	  /**
	   * \brief The color threshold mode specifier.
	   *
	   * Supported color thresholding modes.
	   */
	  enum ColorThresholdMode
	    {
	      /// Logically or channels when thresholding.
	      ctmOrChannels=mvIMPACT_C::COL_OR_CHANNELS,
	      /// Logically and channels when thresholding.
	      ctmAndChannels=mvIMPACT_C::COL_AND_CHANNELS,
	      /// Logically nor channels when thresholding.
	      ctmNorChannels=mvIMPACT_C::COL_OR_CHANNELS | mvIMPACT_C::COL_INVERT,
	      /// Logically nand channels when thresholding.
	      ctmNandChannels=mvIMPACT_C::COL_AND_CHANNELS | mvIMPACT_C::COL_INVERT,
	    };
	  
	  
	  
	  


	  /**
	   * \brief  Threshold a color image.
	   *
	   * Performs a color threshold, thresholding each band separately and combining results logically.
	   * First the image is internally converted from the source colorspace to the thresholding colorspace.
	   * Then each band is thresholded against the specified lower and upper thresholds.
	   * Finally, the results of the three bands are logically ORed or ANDed, and optionally this result is inverted.
	   *
	   * The source image must be a color image with three bands, the destination image must be a monochrome image with one band.
	   * This function returns the resulting image.
	   * 
	   * \see mvIMPACT::Image
	   * \see #ColorSpace
	   * \see #ColorThresholdMode
	   */
	  //-----------------------------------------------------------------------------
	  Image colorThreshold(
					/// The color source image (must have three bands).
					const Image& source,
					/// The colorspace of the source.
					ColorSpace sourceColorSpace,
					/// The colorspace the thresholding is performed in.
					ColorSpace thresholdColorSpace,
					/// The lower threshold for the first channel. 
					/// Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel0Upper)s-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel0Lower,
					/// The upper threshold for the first channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamComplex Python="%(channel0Lower)s+1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel0Upper,
					/// The lower threshold for the second channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel1Upper)s-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel1Lower,
					/// The upper threshold for the second channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamComplex Python="%(channel1Lower)s+1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel1Upper,
					/// The lower threshold for the third channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel2Upper)s-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel2Lower,
					/// The upper threshold for the third channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamComplex Python="%(channel2Lower)s+1"/></mvParamMin>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel2Upper,
					/// The binarization mode.
					ColorThresholdMode mode);

	  /**
	   * \brief  Threshold a color image.
	   *
	   * Performs a color threshold, thresholding each band separately and combining results logically.
	   * First the image is internally converted from the source colorspace to the thresholding colorspace.
	   * Then each band is thresholded against the specified lower and upper thresholds.
	   * Finally, the results of the three bands are logically ORed or ANDed, and optionally this result is inverted.
	   *
	   * The source image must be a color image with three bands, the destination image must be a monochrome image with one band.
	   * \see mvIMPACT::Image
	   * \see #ColorSpace
	   * \see #ColorThresholdMode
	   */
	  //-----------------------------------------------------------------------------
	  void colorThreshold(
					/// The monochrome destination image (must have one band).
					Image& dest,
					/// The color source image (must have three bands).
					const Image& source,
					/// The colorspace of the source.
					ColorSpace sourcecolorSpace,
					/// The colorspace the thresholding is performed in.
					ColorSpace thresholdColorSpace,
					/// The lower threshold for the first channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel0Upper)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel0Lower,
					/// The upper threshold for the first channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMax><mvParamComplex Python="%(channel0Lower)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel0Upper,
					/// The lower threshold for the second channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel1Upper)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel1Lower,
					/// The upper threshold for the second channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMax><mvParamComplex Python="%(channel1Lower)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel1Upper,
					/// The lower threshold for the third channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(channel2Upper)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel2Lower,
					/// The upper threshold for the third channel. 
					///Each channel is separately binarized and the result is logically combined according to the mode parameter.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMax><mvParamComplex Python="%(channel2Lower)s"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int channel2Upper,
					/// The binarization mode.
					ColorThresholdMode mode);

	  /**
	   * \brief  Apply a color twist matrix.
	   *
	   * This function converts the source image by multiplying pixel values with the color twist matrix.
	   * You can perform various color space conversions with this function, as well as other color operations, 
	   * such as color balancing, swapping, changing contrast, etc.
	   *
	   * The color-twist matrix is a generalized 4 by 4 matrix [i, j] that converts the three channels (a, b, c) into (d, e, f) using matrix multiplication.
	   * This function returns the converted result color image.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::MatrixDouble
	   */
	  //-----------------------------------------------------------------------------
	  Image colorTwist(
			   /// The source image. Must be a color image with 3 bands.
			   const Image& source,
			   /// 4x4 matrix
			   const MatrixDouble& matrix);


	  /**
	   * \brief  Apply a color twist matrix.
	   *
	   * This function converts the source image by multiplying pixel values with the color twist matrix.
	   * You can perform various color space conversions with this function, as well as other color operations, 
	   * such as color balancing, swapping, changing contrast, etc.
	   *
	   * The color-twist matrix is a generalized 4 by 4 matrix [i, j] that converts the three channels (a, b, c) into (d, e, f) using matrix multiplication.
	   * This function stores the converted result color image in variable \a dest.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::MatrixDouble
	   */
	  //-----------------------------------------------------------------------------
	  void colorTwist(
			  /// The destination image that receives the converted result color image. Must be of the same size as the source image.
			  Image& dest,
			  /// The source image. Must be a color image with 3 bands.
			  const Image& source,
			  /// 4x4 matrix
			  const MatrixDouble& matrix);

	  /**
	   * \brief Convert a colorspace id to a colormodel string.
	   *
	   * This function converts a colorspace id to a colormodel string.
	   *
	   * \see #ColorSpace
	   * \see mvIMPACT::color::colorspaceID
	   */
	  //-----------------------------------------------------------------------------
	  std::string colorModelFromID(
				       /// The colorspace id.
				       ColorSpace colorSpace);



	  /**
	   * \brief Convert a colormodel string to a colorspace id.
	   *
	   * This function converts a color model string to a colorspace id.
	   * \see #ColorSpace
	   * \see mvIMPACT::color::colorModelFromID
	   */
	  //-----------------------------------------------------------------------------
	  ColorSpace colorspaceID(
				  /// The color model string.
				  const std::string& model);


	  /**
	   * \brief Convert a colorspace.
	   *
	   * This function converts the source image, changing from the source color space to the destination color space. It supports the following color spaces:
	   * - RGB: This is the standard subtractive color space, where each pixel is coded with a red, green and blue value.
	   * - Gray: Not a color space in the strict sense, but a monochrome representation, where each pixel is coded with a single luminance value.
	   * - CMY: This is the standard additive color space, where each pixel is coded with a cyan, magenta and yellow value.
	   * - HLS: This is an intuitive color space, where each pixel is coded with a hue, luminance and saturation value.
	   * - HSV: This is an intuitive color space, where each pixel is coded with a hue, saturation and brightness value.
	   * - XYZ601: This is a device independent CIE color space (Illuminant 601).
	   * - XYZ709: This is a device independent CIE color space (Illuminant 709).
	   * - XYZITU: This is a device independent CIE color space (Illuminant ITU).
	   * - YCC: Color space coding luminance and two chromaticity values.
	   * - YIQ: This is the color space used in american NTSC television.
	   * - YUV: This is the color space used in european PAL television.
	   * - BM: This is the Bayer Mosaic.
	   *
	   * If the conversion is to or from the Gray color space, the default weights of 0.212671, 0.715160 and 0.072169 are used for red, 
	   * green and blue, respectively.
	   * This function returns the converted result color image.
	   *
	   * \see mvIMPACT::Image
	   * \see #ColorSpaceConversion
	   */
	  //-----------------------------------------------------------------------------
	  Image convertColorSpace(
				  /// The source image. Must be a color image with 3 bands.
				  const Image& source,
				  /// The colorspace conversion (specifying a conversion from a source to a destination space).
				  ColorSpaceConversion colorSpaceConversion);


	  /**
	   * \brief Convert a colorspace.
	   *
	   * This function converts the source image, changing from the source color space to the destination color space. It supports the following color spaces:
	   * - RGB: This is the standard subtractive color space, where each pixel is coded with a red, green and blue value.
	   * - Gray: Not a color space in the strict sense, but a monochrome representation, where each pixel is coded with a single luminance value.
	   * - CMY: This is the standard additive color space, where each pixel is coded with a cyan, magenta and yellow value.
	   * - HLS: This is an intuitive color space, where each pixel is coded with a hue, luminance and saturation value.
	   * - HSV: This is an intuitive color space, where each pixel is coded with a hue, saturation and brightness value.
	   * - XYZ601: This is a device independent CIE color space (Illuminant 601).
	   * - XYZ709: This is a device independent CIE color space (Illuminant 709).
	   * - XYZITU: This is a device independent CIE color space (Illuminant ITU).
	   * - YCC: Color space coding luminance and two chromaticity values.
	   * - YIQ: This is the color space used in american NTSC television.
	   * - YUV: This is the color space used in european PAL television.
	   * - BM: This is the Bayer Mosaic.
	   *
	   * If the conversion is to or from the Gray color space, the default weights of 0.212671, 0.715160 and 0.072169 are used for red, 
	   * green and blue, respectively.
	   * This function stores the converted result color image in variable \a dest.
	   *
	   * \see mvIMPACT::Image
	   * \see #ColorSpaceConversion
	   */
	  //-----------------------------------------------------------------------------
	  void convertColorSpace(
				 /// The destination image that receives the converted result color image. 
				 /// Must be of the same size as the source image.
				 Image& dest,
				 /// The source image. Must be a color image with 3 bands.
				 const Image& source,
				 /// The colorspace conversion (specifying a conversion from a source to a destination space).
				 ColorSpaceConversion colorSpaceConversion);


	  /**
	   * \brief  Convert a grayscale image to color.
	   *
	   * This function converts the grayscale source image to the color destination image,
	   * forming color by multiplying pixel values with color factors for red, green and blue between 0.0 and 1.0.
	   * This function returns the converted color image.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::color::convertToGrayscale
	   */
	  //-----------------------------------------------------------------------------
	  Image convertToColor(
					/// The source image. Must be a grayscale image with 1 band.
					const Image& source,
					/// The red color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double redFactor,
					/// The green color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double greenFactor,
					/// The blue color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double blueFactor);



	  /**
	   * \brief  Convert a grayscale image to color.
	   *
	   * This function converts the grayscale source image to the color destination image,
	   * forming color by multiplying pixel values with color factors for red, green and blue between 0.0 and 1.0.
	   * This function stores the converted color image in variable \a dest.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::color::convertToGrayscale
	   */
	  //-----------------------------------------------------------------------------
	  void convertToColor(
					/// The destination image. Must be a color image with 3 bands and the same size as the source image.
					Image& dest,
					/// The source image. Must be a grayscale image with 1 band.
					const Image& source,
					/// The red color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double redFactor,
					/// The green color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double greenFactor,
					/// The blue color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double blueFactor);


	  /**
	   * \brief  Convert a color image to grayscale.
	   *
	   * This function converts the color source image to the grayscale destination image,
	   * by multiplying the color pixel values with color factors for red, green and blue between 0.0 and 1.0.
	   * This function returns the converted grayscale image.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::color::convertToColor
	   */
	  //-----------------------------------------------------------------------------
	  Image convertToGrayscale(
					/// The source image. Must be a color image with 3 bands.
					const Image& source,
					/// The red color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double redFactor = 0.212671,
					/// The green color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double greenFactor = 0.715160,
					/// The blue color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double blueFactor = 0.072169);



	  /**
	   * \brief  Convert a color image to grayscale.
	   *
	   * This function converts the color source image to the grayscale destination image,
	   * by multiplying the color pixel values with color factors for red, green and blue between 0.0 and 1.0.
	   * This function stores the converted grayscale image in variable \a dest.
	   *
	   * \see mvIMPACT::Image
	   * \see mvIMPACT::color::convertToColor
	   */
	  //-----------------------------------------------------------------------------
	  void convertToGrayscale(
					/// The destination image. Must be a grayscale image with 1 band and the same size as the source image.
					Image& dest,
					/// The source image. Must be a color image with 3 bands.
					const Image& source,
					/// The red color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double redFactor = 0.212671,
					/// The green color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double greenFactor = 0.715160,
					/// The blue color factor.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
					/// <mvParamStep>0.01</mvParamStep>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					double blueFactor = 0.072169);



	  /**
	   * \brief  Extract the color model string of a specific band.
	   *
	   * This function extracts a color model string for a specific band.
	   * For example, extracting bands 0, 1, 2 for the RGB model would reveal
	   * R, G, and B, respectively.
	   * This function retruns a string specifying the extracted color model.
	   * 
	   */
	  //-----------------------------------------------------------------------------
	  std::string extractModel(
					/// The input color model string (i.e. RGB or HSI).
					const std::string& model,
					/// The band to extract (ranging from 0 to 2).
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="2"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly    			  						
					int band);



	  /**
	   * \brief  Convert a color model string to it's long form.
	   *
	   * This function converts a color model string to it's long form, i.e. RGB is 
	   * converted to R/G/B, 640 to 640 nm, etc.
	   * This function returns the long form of the color model string.
	   * For example "RGB" is converted to "R/G/B".
	   */
	  //-----------------------------------------------------------------------------
	  std::string modelToLong(
				  /// A color model string (any form).
				  const std::string& model);



	  /**
	   * \brief  Convert a color model string to it's short form.
	   *
	   * This function converts a color model string to it's short form, i.e. "R/G/B" is 
	   * converted to "RGB", 640 nm to 640, etc.
	   * This function returns the short form of the color model string.
	   */
	  //-----------------------------------------------------------------------------
	  std::string modelToShort(
				   /// A color model string (any form).
				   const std::string& model);


	  /**
	   * \brief  Convert a wavelength to a tristimulus color value.
	   *
	   * Converts a wavelength between 380 and 780 nm to a color value with respect
	   * to a colorspace. The function will first calculate the RGB tristimulus values
	   * for the wavelength and then convert to the specified colorspace.
	   * This function returns a vector of the three tristimulus values corresponding to the wavelength with respect to the colorspace.
	   * \see #ColorSpace
	   */

	  //-----------------------------------------------------------------------------
	  std::vector<double> spectralToColor(
					    /// A spectral wavelength between 380 and 780 nm.
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="380"/></mvParamMin>
						/// <mvParamMax><mvParamConst value="780"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly    			  						
						int wavelength,
					    /// A colorspace specifier. The tristiumulus values A, B, C will be calculated with respect to the colorspace.
					    ColorSpace colorSpace);


	  /**
	   * \brief  Calculate a color model tinting palette.
	   *
	   * This function calculates a color tinting palette according to the color model passed in. 
	   * For example, if you pass "R" for the color model, the tinting palette will contain reddish tones. 
	   * Likewise, if you pass "G" for the color model, the tinting palette will contain greenish tones, etc.
	   * This function returns a palette corresponding to the color model.
	   * \see mvIMPACT::Palette
	   */
	  //-----------------------------------------------------------------------------
	  Palette tintingPalette(
				 /// The color model string.
				 const std::string& model);


	  /**
	   * \brief  Calculate a color model tinting palette.
	   *
	   * This function calculates a color tinting palette according to the color model passed in. 
	   * For example, if you pass "R" for the color model, the tinting palette will contain reddish tones. 
	   * Likewise, if you pass "G" for the color model, the tinting palette will contain greenish tones, etc.
	   * This function stores the resulting palette in variable \a palette.
	   * \see mvIMPACT::Palette
	   */
	  //-----------------------------------------------------------------------------
	  void tintingPalette(
			      /// The color model string.
			      const std::string& model,
			      /// The destination palette that is to receive the calculated values.
			      Palette& palette);


	  /// \brief Checks if the color license is valid.
	  bool checkLicenseColor();

	/// \brief Return version number from vl_color module.

	/// This function returns a version string. The following format is used: 
	/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
	/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
	/// Example: 6.3.1.4529
	//-----------------------------------------------------------------------------
	std::string getVersionColor();


	/**
	* \brief  Calculates a color distance image suitable for edge detection with mvIMPACT Measure.
	*
	* This function calculates a grayscale image, that emphasizes the transitions between the two RGB colors
	* given with \c color1 and \c color2. The source image given with \c source must be a unsigned 8-bit RGB image. 
	* The resulting image can be used to detect edges using mvIMPACT Measure. 
	* The function works by calculating two color distance images between the source image and the two given RGB colors.  
	* Then the first color distance image is divided by the second color distance image and the result is scaled such
	* that it fits into the resulting unsigned 8-bit grayscale buffer.
	*
	* \note \c source must be a unsigned 8-bit RGB image (3 bands)
	*
	* \see mvIMPACT::Image
	* \see mvIMPACT::RGBColor
	*/
	//-----------------------------------------------------------------------------
	Image calcColorDistanceImage(
					/// The source image. Must be a unsigned 8 bit color image with 3 bands.
					const Image& source,
					/// The first RGB color 
					const RGBColor& color1,
					/// The second RGB color 
					const RGBColor& color2
					);




	/**
	* \brief  Calculates a color distance image suitable for edge detection with mvIMPACT Measure.
	*
	* This function calculates a grayscale image, that emphasizes the transitions between the two RGB colors
	* given with \c color1 and \c color2. The source image given with \c source must be a unsigned 8-bit RGB image. The
	* destination image, if given with \c dest must be of format unsigned 8-bit grayscale. The resulting image can be used to
	* detect edges using mvIMPACT Measure. 
	* The function works by calculating two color distance images between the source image and the two given RGB colors.  
	* Then the first color distance image is divided by the second color distance image and the result is scaled such
	* that it fits into the resulting unsigned 8-bit grayscale buffer.
	*
	* \note \c source must be a unsigned 8-bit RGB image (3 bands)
	* \note \c dest must be a unsigend 8-bit grayscale image (1 band)
	*
	* \see mvIMPACT::Image
	* \see mvIMPACT::RGBColor
	*/
	//-----------------------------------------------------------------------------
	void calcColorDistanceImage(
					/// The destination image. Must be a unsigned 8 bit grayscale image with 1 band and the same size as the source image.
					Image& dest,
					/// The source image. Must be a unsigned 8 bit color image with 3 bands.
					const Image& source,
					/// The first RGB color 
					const RGBColor& color1,
					/// The second RGB color 
					const RGBColor& color2
					);




    /**
	* \brief Returns a copy of the plane with the best gradient in a specified region.
	*
	* This function returns a copy of the plane with the best gradient inside a specified region.
	* It is also possible to pass a number of colorspace conversions, in this case the function
	* returns the best plane of all given color spaces. Parameter \c line together with parameter
	* \c lineWidth defines a rectangle inside the image that is used to determine the best plane.
	* The borders of the rectangle must not exceed the image borders. The \c edgeWidth parameter
	* is the maximal width of an edge, i.e it defines the width of the operator used to determine
	* the best plane.
	* 
	* \note The width of the line must be an odd number!
	* \note The edgeWidth must be an odd number >= 3 and must be smaller or equal than the length
	* \note of the given line!
	*/
	//-----------------------------------------------------------------------------
	Image getMaxContrastPlane(	
			/// Specifies the source image
			const Image& source,
			/// vector with the desired color conversions
			const std::vector<ColorSpaceConversion> colorConversions,
			/// Line that defines the direction of the rectangle that contains the edge
			const Line line,              
			/// Width of the line, must be an odd number > 0
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
			/// </mvParamRange>
			/// \endxmlonly    			  						
			int lineWidth,        
			/// Maximal width of an edge, must be a odd number >= 3 and must be smaller 
			/// or equal as the length of line
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="3"/></mvParamMin>
			/// </mvParamRange>
			/// \endxmlonly    			  						
			int edgeWidth         
			);


    /**
	* \brief Returns a copy of the plane with the best gradient in a specified region.
	*
	* This function returns a copy of the plane with the best gradient inside a specified region.
	* It is also possible to pass a number of colorspace conversions, in this case the function
	* returns the best plane of all given color spaces. Parameter \c line together with parameter
	* \c lineWidth defines a rectangle inside the image that is used to determine the best plane.
	* The borders of the rectangle must not exceed the image borders. The \c edgeWidth parameter
	* is the maximal width of an edge, i.e it defines the width of the operator used to determine
	* the best plane.
	* 
	* \note The width of the line must be an odd number!
	* \note The edgeWidth must be an odd number >= 3 and must be smaller or equal than the length
	* \note of the given line!
	*/
	//-----------------------------------------------------------------------------
	void getMaxContrastPlane(	
			/// The destination image. Must have 1 band, 1 slice, 1 frame.
			Image& dest,
			/// Specifies the source image
			const Image& source,
			/// vector with the desired color conversions
			const std::vector<ColorSpaceConversion> colorConversions,
			/// Line that defines the direction of the rectangle that contains the edge
			const Line line,              
			/// Width of the line, must be an odd number
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
			/// </mvParamRange>
			/// \endxmlonly    			  						
			int lineWidth,        
			/// Maximal width of an edge, must be a odd number >= 3 and must be smaller 
			/// or equal as the length of line
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="3"/></mvParamMin>
			/// </mvParamRange>
			/// \endxmlonly    			  						
			int edgeWidth         
			);
	

	} // namespace color
} // namespace mvIMPACT


#include "mvIMPACT_color_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::color;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_COLOR_H_
