#ifndef MVIMPACT_COLOR_IMPL_H_
#define MVIMPACT_COLOR_IMPL_H_


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace color {


inline Image colorThreshold(
	const Image& S,
	ColorSpace source_space,
	ColorSpace threshold_space,
	int channel_0_lower,
	int channel_0_upper,
	int channel_1_lower,
	int channel_1_upper,
	int channel_2_lower,
	int channel_2_upper,
	ColorThresholdMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_Threshold(
			S.getHandle(), &img,
			mvIMPACT_C::COL_SPACE(source_space),
			mvIMPACT_C::COL_SPACE(threshold_space),
			channel_0_lower, channel_0_upper,
			channel_1_lower, channel_1_upper,
			channel_2_lower, channel_2_upper,
			mvIMPACT_C::COL_THRESHOLD_MODE(mode)))
		throwImpactError();
	return Image::imageFactory(img);
}

inline void colorThreshold(
	Image& D,
	const Image& S,
	ColorSpace source_space,
	ColorSpace threshold_space,
	int channel_0_lower,
	int channel_0_upper,
	int channel_1_lower,
	int channel_1_upper,
	int channel_2_lower,
	int channel_2_upper,
	ColorThresholdMode mode)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::COL_Threshold(
			S.getHandle(), &hD,
			mvIMPACT_C::COL_SPACE(source_space),
			mvIMPACT_C::COL_SPACE(threshold_space),
			channel_0_lower, channel_0_upper,
			channel_1_lower, channel_1_upper,
			channel_2_lower, channel_2_upper,
			mvIMPACT_C::COL_THRESHOLD_MODE(mode)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline Image colorTwist(
	const Image& S,
	const MatrixDouble& matrix)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_ColorTwist(
			S.getHandle(), &img, matrix.data()))
		throwImpactError();
	return Image::imageFactory(img);
}

inline void colorTwist(
	Image& D,
	const Image& S,
	const MatrixDouble& matrix)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::COL_ColorTwist(
			S.getHandle(), &hD, matrix.data()))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline std::string colorModelFromID(
	ColorSpace id)
//-----------------------------------------------------------------------------
{
	int model_string_size = 1;
	std::string model;
	model.resize(model_string_size);
	while (mvIMPACT_C::COL_GetModelFromID(
			mvIMPACT_C::COL_SPACE(id),
			const_cast<char*>(model.data()),
			model_string_size) == IPL_FALSE)
	{
		model_string_size += 1;
		model.resize(model_string_size);
	}
	model.resize(strlen(model.c_str()));

	return model;
}





inline ColorSpace colorspaceID(
	const std::string& model)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::COL_SPACE Id;
	if (!mvIMPACT_C::COL_GetColorSpaceID(
			model.c_str(),
			&Id))
		throwImpactError();
	return ColorSpace(Id);
}





inline Image convertColorSpace(
	const Image& S,
	ColorSpaceConversion conversion)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_ConvertColorSpace(
			S.getHandle(), &img,
			mvIMPACT_C::COL_CONVERSION(conversion)))
		throwImpactError();
	return Image::imageFactory(img);
}

inline void convertColorSpace(
	Image& D,
	const Image& S,
	ColorSpaceConversion conversion)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::COL_ConvertColorSpace(
			S.getHandle(), &hD,
			mvIMPACT_C::COL_CONVERSION(conversion)))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline Image convertToColor(
	const Image& S,
	double red,
	double green,
	double blue)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_ConvertToColor(
			S.getHandle(), &img, red, green, blue))
		throwImpactError();
	return Image::imageFactory(img);
}

inline void convertToColor(
	Image& D,
	const Image& S,
	double red,
	double green,
	double blue)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::COL_ConvertToColor(
			S.getHandle(), &hD, red, green, blue))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline Image convertToGrayscale(
	const Image& S,
	double red ,
	double green ,
	double blue )
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_ConvertToGrayscale(
			S.getHandle(), &img, red, green, blue))
		throwImpactError();
	return Image::imageFactory(img);
}

inline void convertToGrayscale(
	Image& D,
	const Image& S,
	double red ,
	double green ,
	double blue )
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
	if (!mvIMPACT_C::COL_ConvertToGrayscale(
			S.getHandle(), &hD, red, green, blue))
		throwImpactError();
	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}





inline std::string extractModel(
	const std::string& model,
	int band)
//-----------------------------------------------------------------------------
{
	DynArray<mvIMPACT_C::IPL_CHAR> iplExtract(256);
	
	if (!mvIMPACT_C::COL_ExtractModel(
			model.c_str(), band,
			iplExtract, 256))
		throwImpactError();

	
	return std::string(iplExtract);
}





inline std::string modelToLong(
	const std::string& model)
//-----------------------------------------------------------------------------
{
	DynArray<mvIMPACT_C::IPL_CHAR> iplLng(256);

	if (!mvIMPACT_C::COL_ModelToLong(
			model.c_str(),
			iplLng, 256))
		throwImpactError();

	return std::string(iplLng);
}





inline std::string modelToShort(
	const std::string& model)
//-----------------------------------------------------------------------------
{
	DynArray<mvIMPACT_C::IPL_CHAR> iplShrt(256);

	if (!mvIMPACT_C::COL_ModelToShort(
			model.c_str(),
			iplShrt, 256))
		throwImpactError();

	return std::string(iplShrt);
}





inline std::vector<double> spectralToColor(
	int wavelength,
	ColorSpace space)
//-----------------------------------------------------------------------------
{
	std::vector<double> col;
	col.resize(3);
	if (!mvIMPACT_C::COL_SpectralToColor(
			wavelength,
			mvIMPACT_C::COL_SPACE(space),
			&col[0], &col[1], &col[2]))
		throwImpactError();
	return col;
}





inline Palette tintingPalette(
	const std::string& model)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE pal = IPL_DONT_CARE;
	if (!mvIMPACT_C::COL_ModelTinting(
			model.c_str(), &pal))
		throwImpactError();
	return Palette::paletteFactory(pal);
}

inline void tintingPalette(
	const std::string& model,
	Palette& pal)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hPal = pal.getHandle();
	if (!mvIMPACT_C::COL_ModelTinting(
			model.c_str(), &hPal))
		throwImpactError();
}


inline bool checkLicenseColor()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::COL_CheckLic());
}

inline std::string getVersionColor()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::COL_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

inline Image calcColorDistanceImage(
					const Image& source,
					const RGBColor& color1,
					const RGBColor& color2) 
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	mvIMPACT_C::IPL_COLOR c1 = PALETTERGB( color1.getRed(),color1.getGreen(),color1.getBlue());
	mvIMPACT_C::IPL_COLOR c2 = PALETTERGB( color2.getRed(),color2.getGreen(),color2.getBlue());
	if (!mvIMPACT_C::COL_CalcColorDistanceImage(
			source.getHandle(), &img, c1, c2))
		throwImpactError();
	return Image::imageFactory(img);
}




inline void calcColorDistanceImage(
					Image& dest,
					const Image& source,
					const RGBColor& color1,
					const RGBColor& color2)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_COLOR c1 = PALETTERGB( color1.getRed(),color1.getGreen(),color1.getBlue());
	mvIMPACT_C::IPL_COLOR c2 = PALETTERGB( color2.getRed(),color2.getGreen(),color2.getBlue());
	mvIMPACT_C::IPL_BUFHANDLE hD = dest.getHandle();
	if (!mvIMPACT_C::COL_CalcColorDistanceImage(
			source.getHandle(), &hD, c1, c2))
		throwImpactError();
	if(hD != dest.getHandle()) {
		dest = Image::imageFactory(hD);
	}
}



inline Image getMaxContrastPlane(	
			const Image& source,
			const std::vector<ColorSpaceConversion> colorConversions,
			const Line line,              
			int lineWidth,        
			int edgeWidth         
			)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE img = IPL_DONT_CARE;
	mvIMPACT_C::IPL_LINE l;
	l.x0 = line.getP0().getX();
	l.y0 = line.getP0().getY();
	l.x1 = line.getP1().getX();
	l.y1 = line.getP1().getY();

	mvIMPACT_C::COL_CONVERSION* cConv = new mvIMPACT_C::COL_CONVERSION[colorConversions.size()];
	for (size_t counter = 0; counter < colorConversions.size(); counter++) {
		cConv[counter] = mvIMPACT_C::COL_CONVERSION(colorConversions[counter]);
	}

	mvIMPACT_C::IPL_FRESULT result = mvIMPACT_C::COL_GetMaxContrastPlane(source.getHandle(), &img, cConv, colorConversions.size(), l, lineWidth, edgeWidth);
	free(cConv);
	if (! result) throwImpactError();
	return Image::imageFactory(img);
}


inline void getMaxContrastPlane(	
			Image& dest,
			const Image& source,
			const std::vector<ColorSpaceConversion> colorConversions,
			const Line line,              
			int lineWidth,        
			int edgeWidth         
			)
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE hD = dest.getHandle();
	mvIMPACT_C::IPL_LINE l;
	l.x0 = line.getP0().getX();
	l.y0 = line.getP0().getY();
	l.x1 = line.getP1().getX();
	l.y1 = line.getP1().getY();


	mvIMPACT_C::COL_CONVERSION* cConv = new mvIMPACT_C::COL_CONVERSION[colorConversions.size()];
	for (size_t counter = 0; counter < colorConversions.size(); counter++) {
		cConv[counter] = mvIMPACT_C::COL_CONVERSION(colorConversions[counter]);
	}

	mvIMPACT_C::IPL_FRESULT result = mvIMPACT_C::COL_GetMaxContrastPlane(source.getHandle(), &hD, cConv, colorConversions.size(), l, lineWidth, edgeWidth);
	free(cConv);
	if (! result) throwImpactError();
	if(hD != dest.getHandle()) {
		dest = Image::imageFactory(hD);
	}
}





	} // namespace color
} // namespace mvIMPACT
#endif // #ifndef WRAP_ANY
#endif// DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_COLOR_IMPL_H_
