#ifndef MVIMPACT_FOCUS_IMPL_H_
#define MVIMPACT_FOCUS_IMPL_H_

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
namespace mvIMPACT {
	namespace focus {
		


//-----------------------------------------------------------------------------
inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE FocusFinder::adjustFocusCallback
(
    mvIMPACT_C::AF_Z_DRIVE_MODE Status,
    mvIMPACT_C::IPL_DOUBLE      Position,
    mvIMPACT_C::IPL_CONST_VOID_PTR pCookie	
)
//-----------------------------------------------------------------------------
{
	FocusFinder* pFinder = ((CustomFocusCookie*)pCookie)->finder;

	mvIMPACT_C::IPL_FRESULT result;
	
	try {
		result = pFinder->onAdjustFocus((ZDriveMode)Status,Position);
	} catch(...) {
		 std::cerr << "caught exception in callback " << __FUNCTION__ << std::endl;
		result = IPL_FALSE;
	}

	return result;
}




inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE FocusFinder::grabImageCallback
(
	mvIMPACT_C::IPL_BUFHANDLE*  BufHandle, 
	mvIMPACT_C::IPL_CONST_VOID_PTR pCookie 
)
{
	FocusFinder* pFinder = ((CustomFocusCookie*)pCookie)->finder;

	mvIMPACT_C::IPL_FRESULT result;
	
	Image* pDestImage = ((CustomFocusCookie*)pCookie)->image;

	assert(pDestImage->getHandle() == *BufHandle);
		
	try {
		result = pFinder->onGrabImage(*pDestImage);
	} catch(...) {
		 std::cerr << "caught exception in callback " << __FUNCTION__ << std::endl;
		result = IPL_FALSE;
	}
	
	return result;
}

inline mvIMPACT_C::IPL_FRESULT IPL_C_CALLTYPE FocusFinder::customFocusCallback
(
	mvIMPACT_C::IPL_BUFHANDLE   /* BufHandle */, 
	mvIMPACT_C::IPL_DOUBLE*     FocusParam, 
	mvIMPACT_C::IPL_CONST_VOID_PTR pCookie  
)
//-----------------------------------------------------------------------------
{
	FocusFinder* pFinder = ((CustomFocusCookie*)pCookie)->finder;

	mvIMPACT_C::IPL_FRESULT result;
	
	Image* pDestImage = ((CustomFocusCookie*)pCookie)->image;
	
	try {
		result = pFinder->onCustomFocus(*pDestImage, *FocusParam);
	} catch(...) {
		 std::cerr << "caught exception in callback " << __FUNCTION__ << std::endl;
		result = IPL_FALSE;
	}
	
	return result;
}


inline FocusFinder::FocusFinder()
	{
		m_focusPosition  = 0.0;
		
		m_samplesAfterMax = 2;
		m_stepDecrement = 50;
		m_filterType = ftSobel;
		m_iterations = 1;
		m_filterStepWidth = 1;
		m_samplesDirection = 2;
		m_minPosition = -100000000;
		m_maxPosition = 100000000;
		m_startPosition = 0;
		m_stepIncrementRough = 10;
		m_stepIncrement = 1;
		
		m_useOnCustomFocus = false;
	}

inline void FocusFinder::useOnCustomFocus() 
{
	m_useOnCustomFocus = true;
}

inline void FocusFinder::findFocus
	(
        Image&            D,
        const Image&       FocusRegion,
        AutoFocusMode     mode
	)
	{
		transferConfiguration();

		mvIMPACT_C::IPL_BUFHANDLE hD = D.getHandle();
		if(IPL_DONT_CARE == hD) { hD = AF_DONT_CARE; }
		CustomFocusCookie cookie(this, &D);
		if (!mvIMPACT_C::AF_GetFocusImage(
				hD,
				(IPL_DONT_CARE == FocusRegion.getHandle()) ? AF_DONT_CARE : FocusRegion.getHandle(),
				(mvIMPACT_C::AF_FOCUS_MODE)mode,
				(mvIMPACT_C::IPL_VOID_PTR)grabImageCallback,
				(mvIMPACT_C::IPL_VOID_PTR)adjustFocusCallback,
				m_useOnCustomFocus ? (mvIMPACT_C::IPL_VOID_PTR)customFocusCallback : NULL,
				&m_focusPosition,
				&cookie))
			throwImpactError();
		
		assert(D.getHandle() == hD);
	}


inline void FocusFinder::inquireConfiguration()
{
	mvIMPACT_C::IPL_DOUBLE v;
	
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_SAMPLES_AFTER_MAX, &v)) { throwImpactError(); }
	m_samplesAfterMax  = (int)v;

	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_STEP_DECREMENT, &v)) { throwImpactError(); }
	m_stepDecrement  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_FILTER_TYPE, &v)) { throwImpactError(); }
	mvIMPACT_C::IPL_LONG iplFt  = (mvIMPACT_C::IPL_LONG)v;
	m_filterType = FilterType(iplFt);
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_ITERATIONS, &v)) { throwImpactError(); }
	m_iterations  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_FILTER_STEP_WIDTH, &v)) { throwImpactError(); }
	m_filterStepWidth  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_SAMPLES_DIRECTION, &v)) { throwImpactError(); }
	m_samplesDirection  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_MIN_POSITION, &v)) { throwImpactError(); }
	m_minPosition  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_MAX_POSITION, &v)) { throwImpactError(); }
	m_maxPosition  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_START_POSITION, &v)) { throwImpactError(); }
	m_startPosition  = (int)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_STEP_INCREMENT_ROUGH, &v)) { throwImpactError(); }
	m_stepIncrementRough  = (double)v;
		
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_STEP_INCREMENT, &v)) { throwImpactError(); }
	m_stepIncrement  = (double)v;
		
}


inline void FocusFinder::transferConfiguration() const
{
		mvIMPACT_C::IPL_DOUBLE v;
		
		v = (int) m_samplesAfterMax;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_SAMPLES_AFTER_MAX, v)) { throwImpactError(); }
		
		v = (int) m_stepDecrement;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_STEP_DECREMENT, v)) { throwImpactError(); }
		
		mvIMPACT_C::IPL_LONG iplFt(m_filterType);
		v = (mvIMPACT_C::IPL_LONG) iplFt;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_TYPE, v)) { throwImpactError(); }
		
		v = (int) m_iterations;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_ITERATIONS, v)) { throwImpactError(); }
		
		v = (int) m_filterStepWidth;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_STEP_WIDTH, v)) { throwImpactError(); }
		
		v = (int) m_samplesDirection;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_SAMPLES_DIRECTION, v)) { throwImpactError(); }
		
		v = (double) m_minPosition;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_MIN_POSITION, v)) { throwImpactError(); }
		
		v = (double) m_maxPosition;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_MAX_POSITION, v)) { throwImpactError(); }
		
		v = (int) m_startPosition;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_START_POSITION, v)) { throwImpactError(); }
		
		v = (double) m_stepIncrementRough;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_STEP_INCREMENT_ROUGH, v)) { throwImpactError(); }
		
		v = (double) m_stepIncrement;
		if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_STEP_INCREMENT, v)) { throwImpactError(); }
		
}

inline double FocusFinder::getFocusPosition() const
	{
		return m_focusPosition;
	}


inline FocusFinder& FocusFinder::setSamplesAfterMax ( int v )
	{
		m_samplesAfterMax = v;
		return *this;
	}

inline int FocusFinder::getSamplesAfterMax () const
	{
		return m_samplesAfterMax;
	}

inline FocusFinder& FocusFinder::setStepDecrement ( int v )
	{
		m_stepDecrement = v;
		return *this;
	}

inline int FocusFinder::getStepDecrement () const
	{
		return m_stepDecrement;
	}

inline FocusFinder& FocusFinder::setFilterType ( FilterType ft )
	{
		m_filterType = ft;
		return *this;
	}

inline FilterType FocusFinder::getFilterType () const
	{
		return m_filterType;
	}

inline FocusFinder& FocusFinder::setIterations ( int v )
	{
		m_iterations = v;
		return *this;
	}

inline int FocusFinder::getIterations() const
	{
		return m_iterations;
	}

inline FocusFinder& FocusFinder::setFilterStepWidth ( int v )
	{
		m_filterStepWidth = v;
		return *this;
	}

inline int FocusFinder::getFilterStepWidth () const
	{
		return m_filterStepWidth;
	}

inline FocusFinder& FocusFinder::setSamplesDirection ( int v )
	{
		m_samplesDirection = v;
		return *this;
	}

inline int FocusFinder::getSamplesDirection () const
	{
		return m_samplesDirection;
	}

inline FocusFinder& FocusFinder::setMinPosition ( int v )
	{
		m_minPosition = v;
		return *this;
	}

inline int FocusFinder::getMinPosition () const
	{
		return m_minPosition;
	}

inline FocusFinder& FocusFinder::setMaxPosition ( int v )
	{
		m_maxPosition = v;
		return *this;
	}

inline int FocusFinder::getMaxPosition () const
	{
		return m_maxPosition;
	}

inline FocusFinder& FocusFinder::setStartPosition ( int v )
	{
		m_startPosition = v;
		return *this;
	}

inline int FocusFinder::getStartPosition () const
	{
		return m_startPosition;
	}

inline FocusFinder& FocusFinder::setStepIncrementRough ( double v )
	{
		m_stepIncrementRough = v;
		return *this;
	}

inline double FocusFinder::getStepIncrementRough () const
	{
		return m_stepIncrementRough;
	}

inline FocusFinder& FocusFinder::setStepIncrement ( double v )
	{
		m_stepIncrement = v;
		return *this;
	}

inline double FocusFinder::getStepIncrement () const
	{
		return m_stepIncrement;
	}




inline FocusFinder::~FocusFinder()
{
}

inline bool FocusFinder::onCustomFocus(
				   const Image& /* frame */, 
				   double& /* score */ )
{
	return true;
}


inline CustomFocusCookie::CustomFocusCookie(FocusFinder* finder, Image* image)
		: finder(finder), image(image)
	{ }


inline void FocusFinder::setRegPath 
(
	mvIMPACT_C::IPL_HKEY key ,
	const std::string& subkey
)
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::AF_SetRegPath (key, subkey.c_str()))
		throwImpactError();
}





inline void FocusFinder::writeInitializationValues ()
//-----------------------------------------------------------------------------
{
	transferConfiguration();
	if (!mvIMPACT_C::AF_WriteInitializationValues())
		throwImpactError();
}





inline void FocusFinder::readInitializationValues ()
//-----------------------------------------------------------------------------
{
	if (!mvIMPACT_C::AF_ReadInitializationValues())
		throwImpactError();
	inquireConfiguration();
}





inline Image computeMultiFocusImage
(
	const std::vector<Image>& Images,
	MultiFocusMode mode 
)
//-----------------------------------------------------------------------------
{
	if (Images.size() < 2)
		throwImpactException("More than one image must be passed");

	// Copy Image vector to vector containing the handles.
	std::vector<mvIMPACT_C::IPL_BUFHANDLE> ids;
	std::transform(Images.begin(), Images.end(), std::back_inserter(ids), bufferToHandle);

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> iplIds(ids);
	mvIMPACT_C::IPL_BUFHANDLE  hD = -1;
	if (!mvIMPACT_C::AF_GetMultiFocusImage(
			iplIds.getPointer(),
			(mvIMPACT_C::IPL_ULONG)iplIds.getSize(),
			&hD,
			mvIMPACT_C::AF_MULTIFOCUS_MODE(mode)))
		throwImpactError();

	return Image::imageFactory(hD);
}

inline void computeMultiFocusImage
(
	Image& D,
	const std::vector<Image>& Images,
	MultiFocusMode mode 
)
//-----------------------------------------------------------------------------
{
	if (Images.size() < 2)
	{ throwImpactException("More than one image must be passed"); }

	DynArray<mvIMPACT_C::IPL_BUFHANDLE> handles((int)Images.size());
	for(int i = 0; i < (int)Images.size(); i++) {
		handles.getPointer()[i] = Images.at(i).getHandle();
	}

	mvIMPACT_C::IPL_BUFHANDLE  hD = D.getHandle();
	if (!mvIMPACT_C::AF_GetMultiFocusImage(
			handles,
			(mvIMPACT_C::IPL_ULONG)handles.getSize(),
			&hD,
			mvIMPACT_C::AF_MULTIFOCUS_MODE(mode)))
		throwImpactError();

	if(hD != D.getHandle()) {
		D = Image::imageFactory(hD);
	}
}

inline double findFocusValue
(
	Image&            S,
    const Image&      regionImage,
    AutoFocusMode     mode,
	FilterType type,
	int step
)	
//-----------------------------------------------------------------------------
{
	mvIMPACT_C::IPL_BUFHANDLE  hS = S.getHandle();
	mvIMPACT_C::IPL_BUFHANDLE  hR = regionImage.getHandle();
	mvIMPACT_C::IPL_DOUBLE value = 0;

	mvIMPACT_C::IPL_DOUBLE v = 0;
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_FILTER_TYPE, &v)) { throwImpactError(); }

	mvIMPACT_C::IPL_DOUBLE sw = 0;
	if(!mvIMPACT_C::AF_Inquire(mvIMPACT_C::AF_FILTER_STEP_WIDTH, &sw)) { throwImpactError(); }


	mvIMPACT_C::IPL_DOUBLE nv = static_cast<mvIMPACT_C::IPL_DOUBLE>(type);
	if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_TYPE, nv)) { throwImpactError(); }
	nv = step;
	if(!mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_STEP_WIDTH, nv)) { throwImpactError(); }

	if (!mvIMPACT_C::AF_GetFocusValue(
		hS,
		hR,
		mvIMPACT_C::AF_FOCUS_MODE(mode),
		&value))
	{
		mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_TYPE, v);
		mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_STEP_WIDTH, sw);
		throwImpactError();
	}

	mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_TYPE, v);
	mvIMPACT_C::AF_Control(mvIMPACT_C::AF_FILTER_STEP_WIDTH, sw);

	return value;
}


inline bool checkLicenseFocus()
//-----------------------------------------------------------------------------
{
	return (IPL_TRUE == mvIMPACT_C::AF_CheckLic());
}

inline std::string getVersionFocus()
{
	int size = 12;
	std::string sv;
	sv.resize(size);
	while (mvIMPACT_C::AF_GetVersion(sv.c_str(), size) == IPL_FALSE)
	{
		size += 1;
		sv.resize(size);
	}
	sv.resize(strlen(sv.c_str()));

	return sv;
}

	} // namespace mvIMPACT
} // namespace focus
#endif // #ifndef WRAP_ANY
#endif // DOXYGEN_SHOULD_SKIP_THIS


#endif // MVIMPACT_FOCUS_IMPL_H_
