#include <vl_base.h>
#include <vl_file.h>
#include <vl_display.h>
#include <samples.h>

struct UserData
{
	IPL_BUFHANDLE parent;
	IPL_BUFHANDLE child;
	IPL_HWND hwndChild;
};

IPL_FRESULT IPL_C_CALLTYPE OnTracked(DISP_OOISETHANDLE set, DISP_OOIHANDLE ooi, IPL_CONST_VOID_PTR UserPtr)
{
	struct UserData* pData = (struct UserData*)UserPtr;

	// move the child along with the aoi
	IPL_LONG aoileft = 0;
	CHECKSDKCALL(DISP_InquireOoi(set, ooi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+0), &aoileft));
	IPL_LONG aoiright = 0;
	CHECKSDKCALL(DISP_InquireOoi(set, ooi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+1), &aoiright));
	IPL_LONG aoitop = 0;
	CHECKSDKCALL(DISP_InquireOoi(set, ooi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+2), &aoitop));
	IPL_LONG aoibottom = 0;
	CHECKSDKCALL(DISP_InquireOoi(set, ooi, (DISP_OOICONTROLTYPE)(DISP_OOI_POSINDEX+3), &aoibottom));

	// normalize the aoi rectangle
	IPL_LONG left = min(aoileft, aoiright);
	IPL_LONG right = max(aoileft, aoiright);
	IPL_LONG top = min(aoitop, aoibottom);
	IPL_LONG bottom = max(aoitop, aoibottom);

	// find the parent size, to be able to confine child movement to parent bounds
	IPL_ULONG width = 0;
	CHECKSDKCALL(IPL_BufInquire(pData->parent, IPL_XSIZE, &width));
	IPL_ULONG height = 0;
	CHECKSDKCALL(IPL_BufInquire(pData->parent, IPL_YSIZE, &height));

	IPL_BufTranslateChild(pData->child,
		max(0l, left), max(0l, top),
		min((IPL_LONG)(width-left), (IPL_LONG)(right-left)), min((IPL_LONG)(height-top), (IPL_LONG)(bottom-top)));

	CHECKSDKCALL(DISP_UpdateWindow(pData->hwndChild, DISP_REDRAW));

	return TRUE;
}

int main(int /*argc*/, char** /*argv[]*/ )
{
	// load any supported file format, into image
	IPL_BUFHANDLE image = IPL_DONT_CARE;
	CHECKSDKCALL(IPL_FlImport(&image, NULL, FL_DONT_CARE, "screw.bmp"));

	// install callbacks
	CHECKSDKCALL(DISP_Control(DISP_ON_OOITRACKED, (IPL_VOID_PTR)OnTracked));

	// create a child with the initial position of the aoi
	IPL_BUFHANDLE child = IPL_DONT_CARE;
	CHECKSDKCALL(IPL_BufAllocChild(&child, image, 200, 200, 0, 0, 0, 100, 100, 1, 1, 1));

	// open window to child image
	IPL_HWND hwndChild = 0;
	CHECKSDKCALL(DISP_OpenWindow(child, &hwndChild, 0, NULL));

	// open window to parent image
	IPL_HWND hwnd = 0;
	CHECKSDKCALL(DISP_OpenWindow(image, &hwnd, 0, NULL));

	UserData Data;
	Data.parent = image;
	Data.child = child;
	Data.hwndChild = hwndChild;

	// create an ooiset
	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiSetAlloc(&oois));

	// create an aoi in the set
	IPL_POINT aoipoints[5] =
	{
		{ 200, 200 },
		{ 300, 200 },
		{ 300, 300 },
		{ 200, 300 },
		{ 200, 200 },
	};
	DISP_OOIHANDLE aoi = DISP_DONT_CARE;
	CHECKSDKCALL(DISP_OoiAlloc(oois, &aoi, DISP_AOI_TOOL, aoipoints, sizeof(aoipoints)/sizeof(aoipoints[0]), &Data));

	// set some ooi properties
	COLORREF color = PALETTERGB(0, 255, 0);
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_COLOR, &color));
	IPL_CHAR* text = "Drag me";
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT, text));
	CHECKSDKCALL(DISP_ControlOoi(oois, aoi, DISP_OOI_TEXT_COLOR, &color));

	// associate the ooiset with the thread (and all windows created thereafter)
	CHECKSDKCALL(DISP_ControlWindow(DISP_OOISET, hwnd, &oois));

	// keep sample alive until console window is closed or space bar is pressed
	CHECKSDKCALL(DISP_KeepRunning("Click the space bar to end program.\n"));

	// free allocated buffers
	CHECKSDKCALL(IPL_BufFreeAll());

	return 0;
}
