// cr1d.cpp : Defines the entry point for the console application.
//

#include <vl_base.h>
#include <vl_file.h>
#include <vl_ipl.h>
#include <vl_display.h>
#include <vl_cr1d.h>
#include <string>
#include <iostream>
#include <samples.h>
#define _InImgFile 		"BarCode16.bmp"

int main(int argc, char* argv[])
{
	bool bUI = true;
	char inputFileName[1024], *pinputFileName = NULL;
	memset( inputFileName, 0, 1024 );
	if( argc > 1 )
	{
		_skiparg( );
		_strarg( pinputFileName );

		sprintf( inputFileName, "%s", pinputFileName );

		if ( argc > 1 )
			bUI = false;

	}
	else
		sprintf( inputFileName, "%s", _InImgFile );

	IPL_BUFHANDLE lhSrc = IPL_DONT_CARE;

	//load sample file
	IPL_FlImport(&lhSrc, NULL, FL_DONT_CARE, inputFileName);

	//open window and draw binary image
	IPL_HWND hWnd = 0;
	if (bUI)
		DISP_OpenWindow(lhSrc, &hWnd, 0, 0);

	CR1D_BUFHANDLE config = IPL_DONT_CARE;
	CR1D_BufAlloc(&config);
	CR1D_Control(config, CR1D_CFG_CONTRAST, 10);
	CR1D_Control(config, CR1D_CFG_MIN_GAPS, 10);
	CR1D_Control(config, CR1D_CFG_NOISE_THRESHOLD, 40);
	CR1D_BUFHANDLE results = IPL_DONT_CARE;
	CR1D_Decode(lhSrc, &config, &results);

	std::cout << "File: " << inputFileName << std::endl;
	
	IPL_ULONG candidates=0;
	CR1D_Inquire(results, CR1D_GET_NUMBER_OF_CANDIDATES, 0, 0, &candidates);

	if (candidates)
	{
		IPL_ULONG total_time=0;
		CR1D_Inquire(results, CR1D_GET_TOTAL_DECODING_TIME, 0, 0, &total_time);

		std::cout << " Total decoding time: " << total_time << std::endl;
		
		for(unsigned int i=0; i<candidates; ++i)
		{
			std::cout << "  Candidate: " << i << std::endl;

			IPL_ULONG codes=0;
			CR1D_Inquire(results, CR1D_GET_NUMBER_OF_CODES, 0, 0, &codes);

			for(unsigned int j=0; j<codes; ++j)
			{
				std::cout << "   Code: " << j << " ";

				std::string decoder;
				int decoder_length = 0;
				CR1D_Inquire(results, CR1D_GET_CODETYPE_LENGTH, i, j, &decoder_length);
				decoder.resize(decoder_length);
				CR1D_Inquire(results, CR1D_GET_CODETYPE, i, j, &decoder[0]);

				std::cout << "Decoder: " << decoder << " ";

				std::string code;
				int code_length = 0;
				CR1D_Inquire(results, CR1D_GET_CODEDATA_LENGTH, i, j, &code_length);
				code.resize(code_length);
				CR1D_Inquire(results, CR1D_GET_CODEDATA, i, j, &code[0]);

				std::cout << "Code: " << code << std::endl;

				std::string checksum;
				int checksum_length = 0;
				CR1D_Inquire(results, CR1D_GET_CHECKSUM_LENGTH, i, j, &checksum_length);
				checksum.resize(checksum_length);
				CR1D_Inquire(results, CR1D_GET_CHECKSUM, i, j, &checksum[0]);

				std::cout << "   Checksum: " << checksum << std::endl;

				IPL_POINT location[4];
				memset(&location, 0, sizeof(IPL_POINT)*4);
				CR1D_Inquire(results, CR1D_GET_LOCATION, i, j, &location);

				std::cout << "   Location: " 
					<< location[0].xpos << ", " << location[0].ypos << ", " 
					<< location[1].xpos << ", " << location[1].ypos << ", " 
					<< location[2].xpos << ", " << location[2].ypos << ", " 
					<< location[3].xpos << ", " << location[3].ypos << "\n";

				IPL_DOUBLE overprint = 0.0;
				CR1D_Inquire(results, CR1D_GET_OVERPRINT, i, j, &overprint);

				std::cout << "   Overprint: " << overprint << std::endl;

				IPL_ULONG time = 0;
				CR1D_Inquire(results, CR1D_GET_DECODING_TIME, i, j, &time);

				std::cout << "   Decoding time: " << time << std::endl;
			}
		}
	}
	else
	{
		std::cout << " Decoding failed" << std::endl;
	}

	CR1D_BufFree(config);
	CR1D_BufFree(results);

	//wait until space bar is pressed
	if (bUI)
		DISP_KeepRunning("Press space bar to continue.\n");

	//free buffers
	IPL_BufFree(lhSrc);

	return 0;
}

