//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>cr1d_video<<
// Don Apr 29 17:03:00 CEST 2004
//----------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
#include <stdio.h>

#include <vl_file.h>
#include <vl_ipl.h>
#include <vl_base.h>

#include <vl_display.h>
#include <vl_device.h>

#include <vl_cr1d.h>

#ifdef MALLOC_TRACE
	#include <mcheck.h>
#endif	// MALLOC_TRACE

#include <samples.h>

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
#define TOOL_PREFIX			"cr1d_video"
#define MAX_OBJ_CNT			10

#define TRIG_SW				1
#define TRIG_FALLING		4
#define TRIG_RISING			5

//------------------------------------------------------------------------------------------
COLORREF colTab[MAX_OBJ_CNT] =
{
	PALETTERGB( 255,   0,   0 ),
	PALETTERGB(   0, 255,   0 ),
	PALETTERGB(   0,   0, 255 ),
	PALETTERGB( 255, 255,   0 ),
	PALETTERGB( 255,   0, 255 ),
	PALETTERGB(   0, 255, 255 ),
	PALETTERGB( 127, 255,   0 ),
	PALETTERGB(   0, 127, 255 ),
	PALETTERGB( 255, 127,   0 ),
	PALETTERGB( 127,   0, 255 )
};

//------------------------------------------------------------------------------------------
int main( int argc, char *argv[] )
{
#ifdef MALLOC_TRACE
	mtrace();
#endif	// MALLOC_TRACE

	int expose = -1;

	if( argc > 0 )
	{
		_skiparg( );
		_intarg( expose );
	}

	IPL_ULONG candidates = 0;
	IPL_ULONG numCodes = 0;
	IPL_BUFHANDLE imageHandle = IPL_DONT_CARE;
	CR1D_BUFHANDLE configHandle = IPL_DONT_CARE;
	CR1D_BUFHANDLE resultsHandle = IPL_DONT_CARE;
	IPL_UINT imageWidth, imageHeight;
	DEV_HANDLE deviceHandle = IPL_DONT_CARE;
	IPL_HWND hwndGrab = 0;

	DISP_OOISETHANDLE oois = DISP_DONT_CARE;
	DISP_OOIHANDLE aoi[MAX_OBJ_CNT];
	DISP_OOIHANDLE poi[MAX_OBJ_CNT];

	IPL_POINT loipoints[2];
	IPL_POINT txtpoint = { 0, 0 };

	memset( loipoints, 0, 2 * sizeof( IPL_POINT ));

	CR1D_BufAlloc( &configHandle );
	CR1D_Control( configHandle, CR1D_CFG_MIN_GAPS, 11 );

	DEV_Open( &deviceHandle, "matrixvision.ddc" );
	DEV_Control( deviceHandle, DEV_SINGLE, 0 );

	DEV_Grab( deviceHandle, &imageHandle );
	DISP_OpenWindow( imageHandle, &hwndGrab, 0, NULL );
	DISP_UpdateWindow( hwndGrab, DISP_INVALIDATE );

	IPL_BufInquire( imageHandle, IPL_XSIZE, &imageWidth );
	IPL_BufInquire( imageHandle, IPL_YSIZE, &imageHeight );
	static unsigned int dcnt = 0;

	DISP_OoiSetAlloc( &oois );

	while( !_kbhit( ))
	{
		if( !DEV_Grab( deviceHandle, &imageHandle ))
		{
			fprintf( stderr, "*** " TOOL_PREFIX ": Grab ERROR: %s (Code: %ld)!!\n", IPL_GetErrorString(IPL_GetErrorCode2()), IPL_GetErrorCode2( ));
			break;
		}

		if (!CR1D_Decode( imageHandle, &configHandle, &resultsHandle ))
		{
			fprintf( stderr,"*** " TOOL_PREFIX ": ERROR: %s (Code: %ld)!!\n", IPL_GetErrorString( IPL_GetErrorCode2( )), IPL_GetErrorCode2( ));
			fprintf( stderr,"*** " TOOL_PREFIX ": NOTE: Currently only 8-bit images are supported!!\n" );
			break;
		}
		else
		{
			CR1D_Inquire( resultsHandle, CR1D_GET_NUMBER_OF_CANDIDATES, 0, 0, &candidates );

			dprintf((" == " TOOL_PREFIX ": candidates: %ld\n", candidates ));
			if( candidates ) // if candidates were  found
			{
				IPL_ULONG total_time = 0;
				CR1D_Inquire( resultsHandle, CR1D_GET_TOTAL_DECODING_TIME, 0, 0, &total_time );

				for( IPL_ULONG i = 0; i < candidates; i++ )
				{
					CR1D_Inquire( resultsHandle, CR1D_GET_NUMBER_OF_CODES, 0, 0, &numCodes );
					dprintf(( " == " TOOL_PREFIX ": numCodes: %ld\n", numCodes ));

					for( IPL_ULONG j = 0; j < numCodes; j++ )
					{
						char resultData[2048];
						memset( resultData, 0, 2048 );
						CR1D_Inquire( resultsHandle, CR1D_GET_CODEDATA, i, j, resultData );

						if( strlen( resultData ))
						{
							IPL_POINT location[4];
							memset(&location, 0, sizeof(IPL_POINT)*4);
							CR1D_Inquire( resultsHandle, CR1D_GET_LOCATION, i, j, &location );

							aoi[dcnt] = DISP_DONT_CARE;
							DISP_OoiAlloc( oois, &aoi[dcnt], DISP_AOI_TOOL, location, sizeof(location)/sizeof(location[0]), NULL );

							COLORREF color = colTab[dcnt];
							DISP_ControlOoi( oois, aoi[dcnt], DISP_OOI_COLOR, &color );

							txtpoint.xpos = imageWidth/2;
							txtpoint.ypos = 10 * i + 25;

							poi[dcnt] = DISP_DONT_CARE;
							DISP_OoiAlloc( oois, &poi[dcnt], DISP_POI_TOOL, &txtpoint, 1, NULL );

							color = PALETTERGB( 0, 0, 0 );
							DISP_ControlOoi( oois, poi[dcnt], DISP_OOI_COLOR, &color );
							unsigned int vis = 0;
							DISP_ControlOoi( oois, poi[dcnt], DISP_OOI_LINES_VISIBLE, &vis );

							COLORREF textcolor = colTab[dcnt];
							DISP_ControlOoi( oois, poi[dcnt], DISP_OOI_TEXT, resultData );
							DISP_ControlOoi( oois, poi[dcnt], DISP_OOI_TEXT_COLOR, &textcolor );
							dcnt++;

							dprintf((" == " TOOL_PREFIX ": Object %ld/%ld: (%ld,%ld) (%ld,%ld)\n",
								i, j, loipoints[0].xpos, loipoints[0].ypos, loipoints[1].xpos, loipoints[1].ypos ));

							IPL_ULONG time = 0;
							CR1D_Inquire( resultsHandle, CR1D_GET_DECODING_TIME, i, j, &time );
							printf( " == " TOOL_PREFIX ": DATA: >>>>%s<<<<\ttime: total=%ld dec=%ld [ms]\n", resultData, total_time, time );
						}
					}
					DISP_ControlWindow( DISP_OOISET, hwndGrab, &oois );
				}
			} // if candidates
		}

		DISP_UpdateWindow( hwndGrab, DISP_INVALIDATE );
		DISP_KeepRunning( NULL );

		for( unsigned int i = 0; i < dcnt; i++ )
		{
			if( aoi[i] != IPL_DONT_CARE )
			{
				DISP_OoiFree( oois, aoi[i] );
				aoi[i] = IPL_DONT_CARE;
			}

			if( poi[i] != IPL_DONT_CARE )
			{
				DISP_OoiFree( oois, poi[i] );
				poi[i] = IPL_DONT_CARE;
			}
		}
		dcnt = 0;

	} // while main image capture loop

	DEV_Close( deviceHandle );

	DISP_OoiSetFree( oois );
	CR1D_BufFree( configHandle );
	CR1D_BufFree( resultsHandle );
	IPL_BufFree( imageHandle );

	return 0;
}

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------


