/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page aoi_page AOI example (CPP)

\section aoi_section_purpose Purpose
The AOI example shows, how different types of objects of interests can be used.

\section aoi_section_intro What it does
This sample shows how to create areas of interest and lines of interest and
shows how to change some properties. 

\section aoi_section_description How it works
First of all an image is loaded into memory and a window is created to show
the image.

\code 
		// create new ImageBuffer
		Image image = loadImage("screw.bmp");
		Window window = Window(image);

\endcode

Then an AreaOfInterest object is created, which is associated to \c window.
The corresponding area of interest is then shown in this window.

\code
		//create first area of interest and set some properties
		AreaOfInterest aoi1 = AreaOfInterest(window,Rectangle(10,10,110,110));
\endcode

Afterwards some properties for the created AreaOfInterest object are set.

\code
		aoi1.setColor(RGBColor(0, 255, 0));
		aoi1.setText("Rectangle 1");
		aoi1.setTextColor(RGBColor(255, 0, 0));
\endcode

Then another AreaOfInterest object is created, similar to the first one.
\code
		//create second area of interest and set some properties
		AreaOfInterest aoi2 = AreaOfInterest(window,Rectangle(210,210,310,310));
		aoi2.setColor(RGBColor(255, 0, 0));
		aoi2.setText("Rectangle 2");
		aoi2.setTextColor(RGBColor(255, 0, 0));
\endcode

Afterwards, a LineOfInterest object is created.

\code
		//create line of interest and set some properties
		LineOfInterest loi1 = LineOfInterest(window,Line(Point(150,50),Point(150,150)));
		loi1.setColor(RGBColor(0, 0, 255)); 
		loi1.setText("Line 1");
		loi1.setTextColor(RGBColor(255, 0, 0));
\endcode


Then another LineOfInterest object is created.
\code
		//create another line of interest
		LineOfInterest loi2 = LineOfInterest(window,Line(Point(350,250),Point(350,350)));
		loi2.setColor(RGBColor(255, 255, 255)); 
		loi2.setText("Line 2");
		loi2.setTextColor(RGBColor(255, 0, 0));
\endcode

Finally, the message loop of the windowing system is called. Now you can
interactively change the position and size (or length respectively) of the
generated objects of interest. Finally, press space to return from
the message loop function.

\code
		runMessageLoop();
\endcode

\section aoi_section_source Source file

<matrixdoc>
*/



#include "mvIMPACT.h"

int main(int , char*[] )
{
	try
	{
		// create new ImageBuffer
		Image image = loadImage("screw.bmp");
		Window window = Window(image);

		//create first area of interest and set some properties
		AreaOfInterest aoi1 = AreaOfInterest(window, mvIMPACT::Rectangle(10,10,110,110));
		aoi1.setColor(RGBColor(0, 255, 0));
		aoi1.setText("Rectangle 1");
		aoi1.setTextColor(RGBColor(255, 0, 0));

		//create second area of interest and set some properties
		AreaOfInterest aoi2 = AreaOfInterest(window, mvIMPACT::Rectangle(210,210,310,310));
		aoi2.setColor(RGBColor(255, 0, 0));
		aoi2.setText("Rectangle 2");
		aoi2.setTextColor(RGBColor(255, 0, 0));

		//create line of interest and set some properties
		LineOfInterest loi1 = LineOfInterest(window,Line(Point(150,50),Point(150,150)));
		loi1.setColor(RGBColor(0, 0, 255)); 
		loi1.setText("Line 1");
		loi1.setTextColor(RGBColor(255, 0, 0));

		//create another line of interest
		LineOfInterest loi2 = LineOfInterest(window,Line(Point(350,250),Point(350,350)));
		loi2.setColor(RGBColor(255, 255, 255)); 
		loi2.setText("Line 2");
		loi2.setTextColor(RGBColor(255, 0, 0));

		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
	return 0;

}

//</matrixdoc>
