/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page device_page Old device interface example (CPP)

\section device_section_purpose Purpose
This example shows how to use the device interface of the C-API. 
This is only relevant until the new acquire interface will be available for
all supported devices. Note that then this example gets deprecated.(AcquireControl installation required!)

\section device_section_intro What it does
A single imais captured and a window is opened to display the image.

\section device_section_description How it works

First of all the namespace where the C-API functions are defined is imported with
the using directive.

\code
		  using namespace mvIMPACT_C;
\endcode

Then the default device is opened and configured to capture a single image. Note that
the error codes are converted to exceptions here by using the (undocumented) 
function throwImpactError() if a single call fails.
\code
		  DEV_HANDLE hDevice = IPL_DONT_CARE;
		  if(!DEV_Open(&hDevice, "matrixvision.ddc")) { throwImpactError(); }

		  if(!DEV_Control (hDevice, DEV_SINGLE, NULL)) { throwImpactError(); }
\endcode

Afterwards a variable holding an image handle is defined and the grabbing is done.
\code
		  IPL_BUFHANDLE hImage = IPL_DONT_CARE;
		  if(!DEV_Grab (hDevice , &hImage)) { throwImpactError(); }
\endcode

Now the image handle is converted into an Image object which can be used with all functions
of the C++ wrapper afterwards. Then the device is closed again

\code
		  // now construct image object
		  Image image = Image::imageDeviceFactory(hImage);

		  if(!DEV_Close(hDevice))  { throwImpactError(); }
\endcode

Afterwards a window is opened and the message loop is run.
\code
		  // open window to show image
		  Window window(image);
	
		  runMessageLoop();
\endcode

Finally, no cleanup is done (!), i.e. the image handle must not be freed with the
function IPL_BufFree from the C-API, because it is "owned" by the corresponding Image object
now.
\code
		  // do _not_ call IPL_BufFree() here, because the Image object owns the
		  // handle now. And never call IPL_BufFreeAll() when 
		  // using the C++ wrapper !
\endcode

\section device_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

int main(int /*argc*/, char* /*argv*/[])
{

	 try
	 {
		  using namespace mvIMPACT_C;

		  DEV_HANDLE hDevice = IPL_DONT_CARE;
		  if(!DEV_Open(&hDevice, "matrixvision.ddc")) { throwImpactError(); }

		  if(!DEV_Control (hDevice, DEV_SINGLE, NULL)) { throwImpactError(); }

		  IPL_BUFHANDLE hImage = IPL_DONT_CARE;
		  if(!DEV_Grab (hDevice , &hImage)) { throwImpactError(); }
 
 
		  // now construct image object
		  Image image = Image::imageDeviceFactory(hImage);
 
		  // close device
		  if(!DEV_Close(hDevice))  { throwImpactError(); }

		  // open window to show image
		  Window window(image);
	
		  runMessageLoop();

		  // do _not_ call IPL_BufFree() here, because the Image object owns the
		  // handle now. And never call IPL_BufFreeAll() when 
		  // using the C++ wrapper !

	 }	
	 catch(ImpactException& e)	
	 {		
		  std::cerr << e.getErrorString() << std::endl;	
		  getChar();
	 }	
	
	 return 0;
}

//</matrixdoc>
