#include "mvIMPACT.h"
#include <iostream>

// this class handles the window events
class MyWindow : public Window 
{
public:
	MyWindow() 
		: Window()
	{ }

	MyWindow(const Image &image) 
		: Window(image)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	// event implementations
	virtual void onOoiCreate(const ObjectOfInterest*)
	{
		std::cout << "window event: onOoiCreate(ooi=?)" << std::endl;
	}

	virtual void onOoiDestroy(const ObjectOfInterest*)
	{
		std::cout << "window event: onOoiDestroy(ooi=?)" << std::endl;
	}

	virtual bool onOoiSelect(const ObjectOfInterest*, bool selected)
	{
		std::cout << "window event: onOoiSelect(ooi=?, selected=" << selected << ")" << std::endl;
		return true;
	}

	virtual bool onOoiStartTrack(const ObjectOfInterest *, int handle)
	{
		std::cout << "window event: onOoiStartTrack(ooi=?, handle=" << handle << ")" << std::endl;
		return true;
	}

	virtual void onOoiStopTrack(const ObjectOfInterest *)
	{
		std::cout << "window event: onOoiStopTrack(ooi=?)" << std::endl;
	}

	bool onOoiTrackTo(const ObjectOfInterest *, int handle, const Point * point)
	{
		std::cout << "window event: onOoiTrackTo(ooi=?, handle=" << handle << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual void onOoiTracked(const ObjectOfInterest *)
	{
		std::cout << "window event: onOoiTracked(ooi=?)" << std::endl;
	}

	virtual void onDraw(DeviceContext *, const mvIMPACT::Rectangle * source, const mvIMPACT::Rectangle * dest)
	{
		std::cout << "window event: onDraw(context=?, sourceRect={" << source->getLeft() << ", " << source->getTop() << ", " << source->getRight() << ", " << source->getBottom() << "}, destRect={" << dest->getLeft() << ", " << dest->getTop() << ", " << dest->getRight() << ", " << dest->getBottom() << "})" << std::endl;
	}

	virtual bool onBeforeDraw(DeviceContext *, const mvIMPACT::Rectangle * source, const mvIMPACT::Rectangle * dest)
	{
		std::cout << "window event: onBeforeDraw(context=?, sourceRect={" << source->getLeft() << ", " << source->getTop() << ", " << source->getRight() << ", " << source->getBottom() << "}, destRect={" << dest->getLeft() << ", " << dest->getTop() << ", " << dest->getRight() << ", " << dest->getBottom() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseButtonDownWindow(MouseButton button, const Point * point)
	{
		std::cout << "window event: onMouseButtonDownWindow(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseButtonUpWindow(MouseButton button, const Point * point)
	{
		std::cout << "window event: onMouseButtonUpWindow(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseDoubleClickWindow(MouseButton button, const Point * point)
	{
		std::cout << "window event: onMouseDoubleClickWindow(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseMoveWindow(MouseButton button, const Point * point)
	{
		std::cout << "window event: onMouseMoveWindow(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseButtonDown(MouseButton button, const PointDouble * point)
	{
		std::cout << "window event: onMouseButtonDown(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseButtonUp(MouseButton button, const PointDouble * point)
	{
		std::cout << "window event: onMouseButtonUp(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseDoubleClick(MouseButton button, const PointDouble * point)
	{
		std::cout << "window event: onMouseDoubleClick(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onMouseMove(MouseButton button, const PointDouble * point)
	{
		std::cout << "window event: onMouseMove(button=" << button << ", point={" << point->getX() << ", " << point->getY() << "})" << std::endl;
		return true;
	}

	virtual bool onKeyDown(int character)
	{
		std::cout << "window event: onKeyDown(character=" << character << ")" << std::endl;
		return true;
	}

	virtual void onScrollPosChange(int scrollPosX, int scrollPosY)
	{
		std::cout << "window event: onScrollPosChange(scrollPosX=" << scrollPosX << ", scrollPosY=" << scrollPosY << ")" << std::endl;
	}

	virtual void onZoomFactorsChange(double zoomX, double zoomY)
	{
		std::cout << "window event: onZoomFactorsChange(zoomX=" << zoomX << ", zoomY=" << zoomY << ")" << std::endl;
	}
};

int main(int , char*[] )
{
	try
	{
		// create new ImageBuffer
		Image image = loadImage("screw.bmp");

		// create a new Window
		MyWindow window = MyWindow(image);

		// create a new area of interest
		AreaOfInterest aoi = AreaOfInterest(window, mvIMPACT::Rectangle(10, 10, 110, 110));
		aoi.setColor(RGBColor(0, 255, 0));

		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}
	return 0;
}
