/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page shuffle_page Shuffle example (CPP)

\section shuffle_section_purpose Purpose
This example shows how to use palette shuffling.

\section shuffle_section_intro What it does
This example shows how to use palette shuffling in conjunction with multiple thresholds.

\section shuffle_section_description How it works

First the input image is loaded and displayed.

\code
		//load and display image
		Image image = loadImage("phone.bmp");
		Window window1 = Window(image);
\endcode

Then a threshold mapping is set up using two thresholds. These are the same thresholds as in the
multithreshold example \ref multithreshold_page. Then the threholds are applied to the image.

\code
		//threshold image
		std::vector<Threshold> thresholds;
		thresholds.push_back(Threshold(50,pmLess));
		thresholds.push_back(Threshold(180,pmGreater));
		Palette pal = generateThresholdPalette(thresholds,image.getDatatype());
		image = mapLut(image,pal);
\endcode

To make the values of the thresholded image, which lie between 0 and 2, better visible, the
bits 0 and 1 are shuffled to the bits 6 and 7 respectively. This is done by building a list of
instances of class Shuffle with the corresponding fromIndex and toIndex values.

\code
		//setup shuffle struct
		std::vector<Shuffle> shuffle;
		Shuffle s;
		s.setFromIndex(0).setToIndex(6);
		shuffle.push_back(s);
		s.setFromIndex(1).setToIndex(7);
		shuffle.push_back(s);
\endcode

Afterwards the shuffle palette is created and the image is mapped using this palette, resulting
in a second image.

\code
		//calculate new palette
		Palette pal1 = generateShufflePalette(shuffle);

		//map the image over the new palette
		Image image2 = mapLut(image,pal1);
\endcode

Finally a second window is opened to display the resulting image. 
Afterwards, the program waits for space key and exits.

\code
		//display new image
		Window window2 = Window(image2);

		//keep sample alive
		runMessageLoop();
\endcode


\section shuffle_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"


int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		//load and display image
		Image image = loadImage("phone.bmp");
		Window window1 = Window(image);

		//threshold image
		std::vector<Threshold> thresholds;
		thresholds.push_back(Threshold(50,pmLess));
		thresholds.push_back(Threshold(180,pmGreater));
		Palette pal = generateThresholdPalette(thresholds,image.getDatatype());
		image = mapLut(image,pal);

		//setup shuffle struct
		std::vector<Shuffle> shuffle;
		Shuffle s;
		s.setFromIndex(0).setToIndex(6);
		shuffle.push_back(s);
		s.setFromIndex(1).setToIndex(7);
		shuffle.push_back(s);


		//calculate new palette
		Palette pal1 = generateShufflePalette(shuffle);

		//map the image over the new palette
		Image image2 = mapLut(image,pal1);

		//display new image
		Window window2 = Window(image2);

		//keep sample alive
		runMessageLoop();

	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
