//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>blob-afs<<
// Mit Aug 13 11:31:13 CEST 2003
//----------------------------------------------------------------------
#ifndef __blbproc_h__
#define __blbproc_h__
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_ipl.h>
#include <vl_blob.h>

#include <pgutils.h>
#include <samples.h>

#include "blob-afs.h"
#include "afsapi.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define REF_CLASS								0
#define MIN_BLB_AREA							50
#define MAX_BLB_CNT								20

#define CL_CNT_STR								"CLASS_CNT:"
#define CL_NAME_STR								"CLASS_NAME:"
#define FAET_NB_STR								"FEAT_NB:"
#define FAET_NAME_STR							"FEAT_NAME:"

#define ID_STR									"id:"
#define IPL_FEAT_STR							"feat:"
#define ITEMS_STR								"item:"
#define RESULT_STR								"res:"
#define CTRL_VAL_STR							"ctrl:"
#define DELTA_STR								"D:"
#define TIME_MS_STR								"tms:"
#define RANGE_MIN_STR							"rmin:"
#define RANGE_MAX_STR							"rmax:"
#define SUM_CNT_STR								"sum_cnt:"
#define SUM_STR									"sum:"
#define SUM_2_STR								"sum_2:"

#define DEFAULT_FEAT_LIST_CNT					5
#define MAX_FEAT_LIST_CNT						50

#define MIN_TIME_MS								5
#define DEFAULT_TIME_MS							200
#define MAX_TIME_MS								5000

#define INIT_STR_PROC							"PROCESS"
#define INIT_STR_MAXFCNT						"MAX_FEAT_CNT"
#define INIT_STR_POL							"POLARITY"
#define INIT_STR_AUTO_TH						"AUTO_TH"
#define INIT_STR_TH								"THRESHOLD"


#define FILL_BLOB_LIST( feat, ctrl )				\
			{ feat, BL_AR_##feat, #feat, 0, NULL, ctrl, 0.0,0.0,{ 0.0,0.0 }, 0, 0.0,0.0 }

//----------------------------------------------------------------------
//----------------------------------------------------------------------
enum BlbFeature
{
	AREA = 0,
	PERIMETER,
	ROUNDNESS,
	NUMBER_HOLES,
	MAJOR_AXIS_LENGTH,
	MINOR_AXIS_LENGTH,
	FERET_MAX,
	FERET_MIN,
	FERET_RATIO,
	ROUGHNESS,
	AREA_HOLES,
	AREA_PERFORATION,
	FIBER_LENGTH,
	FIBER_WIDTH,
	MAX,
	SUM,
	SQUARED_SUM,
	MEAN,
	STDDEV,
	SKEWNESS,
	KURTOSIS,
	SKEWNESS_FAC,
	KURTOSIS_FAC,
	ENTROPY,
	OPTINT,
	ENERGY,
	CONTRAST,
	ENTROPY_CCMTX,
	CORRELATION,
	POLAR_DIST_MAX,
	POLAR_DIST_MIN,
	/*EXTREME_BOX,*/
	END_LIST
};

typedef enum BlbFeature BLB_FEARURE;

struct SInterval
{
	double x1, x2;
};

typedef struct SInterval INTERVAL;

struct SBlobs
{
	BLB_FEARURE ID;
	BL_FEATURE feat;
	char *name;
	unsigned int items;
	double *results;
	double ctrlVal;
	/*unsigned int ctrlVal;*/
	double delta;
	double time_ms;

	INTERVAL range;

	unsigned long sum_cnt;
	double sum;
	double sum_2;
};

typedef struct SBlobs BLOBS;

struct SImage
{
	unsigned long xpos;
	unsigned long ypos;
	unsigned long width;
	unsigned long height;
	unsigned long pitch;
	unsigned long bpp;
	BYTE *data;
};

typedef struct SImage BLBIMG;

struct SBlobClass
{
	char name[MAX_BUF_SIZE];
	int list_size;
	BLOBS *blb_list;

	unsigned long img_cnt;
	BLBIMG *img;
};

typedef struct SBlobClass BLOBCLASS;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int init_blobs( );
void free_blobs( );
void clear_res_bufs( );
int get_blb_res( );
int set_all_features( int index );
int rm_all_features( int index );

int analyse( IPL_BUFHANDLE greyimgHandle, IPL_BUFHANDLE *binimgHandle );
int get_bounding_box( IPL_BUFHANDLE greyimgHandle, IPL_BUFHANDLE *binimgHandle,
								double **res_buf, unsigned int *ilist );

void init_tmp_buf( );
void clear_tmp_buf( );
int get_tmp_res( );
int set_all_tmp_features( );

int mk_new_blb_class( char *class_name );
void rm_blb_class( char *class_name );
int get_class_cnt( );
int get_class_index( char *class_name );
int get_blb_class_name( unsigned int index, char *name );

void teach_all_results( int index, int obj_index, BYTE *img_data, unsigned long width, unsigned long height );
void delete_class( int index );

int check_feat( );

int set_feat4classify( );
int get_res4classify( );
int rm_feat4classify( );
void del_res4classify( );

double get_calc_time_ms( );
long get_obj_cnt( );
int get_feat_cnt( );
int get_max_feat_nb( );
long get_max_time_ms( );
int get_max_feat( );

void set_max_feat_nb( int val );
void set_max_time_ms( long val );
double get_frame_ps( );
int get_proc_flag( );
void w_on_close( );

int get_blob_init_flag( );
void init_blb_globals( DEV_HANDLE devHandle, IPL_BUFHANDLE *imgHandle );
int check_init_file( );

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __blbproc_h__
