//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#ifndef __cr2d_h__
#define __cr2d_h__

#include <vl_base.h>
#include <vl_ipl.h>
#include <vl_cr2d.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
struct SCR2D_OBJCNT
{
	unsigned int candidate_cnt;
	unsigned int *code_cnt;
};
typedef struct SCR2D_OBJCNT CR2D_OBJCNT;

struct SCR2D_RESULTS
{
	char data[MAX_CHAR_SIZE];
	char ct[MAX_CHAR_SIZE];
	IPL_POINT pts[4];
	//double quality;
};
typedef struct SCR2D_RESULTS CR2D_RESULTS;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
class CCR2D
{
public:
	/*----------------------------------------------------------------*/
	CCR2D( )
	{
		m_cr2d_configH  = IPL_DONT_CARE;
		m_cr2d_resultsH = IPL_DONT_CARE;

		m_cr2d_spacing     = -1;
		m_cr2d_contrast    = -1;
		m_cr2d_gap_size    = -1;

		m_cr2d_res     = NULL;
		m_cr2d_obj_cnt = 0;

		cr2d_init_obj( );

		if( !CR2D_BufAlloc( &m_cr2d_configH ))
			exit_demo( "alloc 2D coder reader buffer failed", IPL_GetErrorCode( ));
	}

	/*----------------------------------------------------------------*/
	~CCR2D( )
	{
		if( m_cr2d_res ) delete m_cr2d_res;

		cr2d_free_obj( );

		if( m_cr2d_configH != IPL_DONT_CARE )
		{
			CR2D_BufFree( m_cr2d_configH );
			m_cr2d_configH = IPL_DONT_CARE;
		}

		if( m_cr2d_resultsH != IPL_DONT_CARE )
		{
			CR2D_BufFree( m_cr2d_resultsH );
			m_cr2d_resultsH = IPL_DONT_CARE;
		}
	}

	/*----------------------------------------------------------------*/
	int cr2d_decode( const IPL_BUFHANDLE imageH )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s (imageH: %ld, configH: %ld, resultsH: %ld)\n",
							__FUNCTION__, imageH, m_cr2d_configH, m_cr2d_resultsH ));
		int ret = 0;
		CRPGASSERT_IPL( CR2D_Decode( imageH, &m_cr2d_configH, &m_cr2d_resultsH ));
		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int m_cr2d_spacing;
	int m_cr2d_contrast;
	int m_cr2d_gap_size;

	/*----------------------------------------------------------------*/
	void cr2d_config( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		if( m_cr2d_spacing > 0 )
		{
			if( !CR2D_Control( m_cr2d_configH, CR2D_CFG_SPACING, m_cr2d_spacing ))
				exit_demo( "set spacing failed", IPL_GetErrorCode( ));
		}

		if( m_cr2d_contrast > 0 )
		{
			if( !CR2D_Control( m_cr2d_configH, CR2D_CFG_CONTRAST, m_cr2d_contrast ))
				exit_demo( "set locator contrast failed", IPL_GetErrorCode( ));
		}

// 		if( m_cr2d_gap_size > 0 )
// 		{
// 			if( !CR2D_Control( m_cr2d_configH, CR2D_CFG_GAP_SIZE, m_cr2d_gap_size ))
// 				exit_demo( "set gap size failed", IPL_GetErrorCode( ));
// 		}

		dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	}

	/*----------------------------------------------------------------*/
	CR2D_OBJCNT   m_cr2d_obj;
	CR2D_RESULTS *m_cr2d_res;
	unsigned int  m_cr2d_obj_cnt;

	/*----------------------------------------------------------------*/
	void cr2d_free_obj( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		if( m_cr2d_obj.code_cnt )
		{
			delete m_cr2d_obj.code_cnt;
			m_cr2d_obj.code_cnt = NULL;
		}
		m_cr2d_obj.candidate_cnt = 0;
		dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	}

	/*----------------------------------------------------------------*/
	void cr2d_init_obj( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

		m_cr2d_obj.code_cnt = NULL;
		m_cr2d_obj.candidate_cnt = 0;

		dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_candidats_and_codes( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = -1;
		cr2d_free_obj( );
		m_cr2d_obj_cnt = 0;

		m_cr2d_obj.candidate_cnt = cr2d_get_candidates_nb( );
		dprintf((" == " TOOL_PREFIX ": %s - candid: %d\n", __FUNCTION__, m_cr2d_obj.candidate_cnt ));
		if( m_cr2d_obj.candidate_cnt > 0 )
		{
			m_cr2d_obj.code_cnt = new unsigned int[m_cr2d_obj.candidate_cnt];
			memset( m_cr2d_obj.code_cnt, 0, sizeof(unsigned int) * m_cr2d_obj.candidate_cnt );

			for( unsigned int i = 0; i < m_cr2d_obj.candidate_cnt; i++ )
			{
				m_cr2d_obj.code_cnt[i] = cr2d_get_codes_nb( );
				dprintf((" == " TOOL_PREFIX ": %s - cand: %d codenb: %d\n", __FUNCTION__, i, m_cr2d_obj.code_cnt[i] ));
				m_cr2d_obj_cnt += m_cr2d_obj.code_cnt[i];
			}

			ret = 0;
		}
		else
		{
			//fprintf( stderr, "*** %s - have no candidats!!\n", __FUNCTION__ );
			set_err_str( "WARNING: Have no 2D candidats!!", MVCOLOR_BLACK );
		}

		dprintf((" -- " TOOL_PREFIX ": %s (%d) (cr2d_obj_cnt: %d)\n", __FUNCTION__, ret, m_cr2d_obj_cnt ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_results( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = -1;

		CRASSERT( m_cr2d_obj.candidate_cnt > 0 , ret );
		CRASSERT( m_cr2d_obj_cnt > 0, ret );

		if( m_cr2d_res ) delete m_cr2d_res;
		m_cr2d_res = new CR2D_RESULTS[m_cr2d_obj_cnt];
		memset( m_cr2d_res, 0, sizeof(CR2D_RESULTS) * m_cr2d_obj_cnt );

		CR_PROP tmp_prop = prop_get_prop( CR_MAX_OBJ );
		unsigned int lcnt = 0, max_cnt = tmp_prop.default_val;
		for( unsigned int i = 0; i < m_cr2d_obj.candidate_cnt; i++ )
		{
			if( m_cr2d_obj.code_cnt[i] > 0 )
			{
				unsigned int j = 0;

				memset( m_cr2d_res[lcnt].data, 0, sizeof(char) * MAX_CHAR_SIZE );
				cr2d_get_code_data( i, j, m_cr2d_res[lcnt].data, MAX_CHAR_SIZE );

				memset( m_cr2d_res[lcnt].ct, 0, sizeof(char) * MAX_CHAR_SIZE );
				cr2d_get_code_type( i, j, m_cr2d_res[lcnt].ct, MAX_CHAR_SIZE );

				cr2d_get_locpts( i, j, m_cr2d_res[lcnt].pts );

				dprintf((" == " TOOL_PREFIX ": %s - data: >>%s<< code type: >>%s<<\n",
							__FUNCTION__, m_cr2d_res[lcnt].data, m_cr2d_res[lcnt].ct ));

				lcnt++;
			}

			if( max_cnt == lcnt )
				break;
		}

		m_cr2d_obj_cnt = lcnt;
		ret = 0;

		dprintf((" -- " TOOL_PREFIX ": %s (%d) cr2d_obj_cnt: %d\n", __FUNCTION__, ret, m_cr2d_obj_cnt ));
		return ret;
	}

	/*----------------------------------------------------------------*/
private:
	CR2D_BUFHANDLE m_cr2d_configH;
	CR2D_BUFHANDLE m_cr2d_resultsH;

	/*----------------------------------------------------------------*/
	int cr2d_get_infos( const unsigned int candidate, const unsigned int code_nb,
						char *buf, const unsigned int buf_size,
						CR2D_CONFIG_INQUIRE ilen, CR2D_CONFIG_INQUIRE idata )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s - candidate: %d code_nb: %d ilen: %d idata: %d\n",
							__FUNCTION__, candidate, code_nb, ilen, idata ));

		int ret = -1;

		CRPGASSERT( buf != NULL, ret );

		IPL_ULONG len = 0;
		CRPGASSERT_IPL( CR2D_Inquire( m_cr2d_resultsH, ilen, candidate, code_nb, &len ));
		dprintf((" == " TOOL_PREFIX ": %s - candidate: %d code_nb: %d len: %ld\n",
											__FUNCTION__, candidate, code_nb, len ));
		CRASSERT( len > 0, -2 );

		CRPGASSERT( len <= buf_size, ret );

		CRPGASSERT_IPL( CR2D_Inquire( m_cr2d_resultsH, idata, candidate, code_nb, buf ));

		dprintf((" == " TOOL_PREFIX ": %s - candidate: %d code_nb: %d data: %s\n",
										__FUNCTION__, candidate, code_nb, buf ));
		ret = 0;

		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_candidates_nb( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = 0;
		CRPGASSERT_IPL( CR2D_Inquire( m_cr2d_resultsH, CR2D_GET_NUMBER_OF_CANDIDATES, 0, 0, &ret ));
		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_codes_nb( )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

		int ret = 0;
		CRPGASSERT_IPL( CR2D_Inquire( m_cr2d_resultsH, CR2D_GET_NUMBER_OF_CODES, 0, 0, &ret ));
		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_code_type( const unsigned int candidate, const unsigned int code_nb,
							char *buf, const unsigned int buf_size )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = 0;
		if(( ret =  cr2d_get_infos( candidate, code_nb, buf, buf_size,
							CR2D_GET_CODETYPE_LENGTH, CR2D_GET_CODETYPE )) == -2 )
			set_err_str( "ERR: get code type length is zero!!", RET_ERROR );

		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_code_data( const unsigned int candidate, const unsigned int code_nb,
							char *buf, const unsigned int buf_size )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = 0;
		if(( ret =  cr2d_get_infos( candidate, code_nb, buf, buf_size,
							CR2D_GET_CODEDATA_LENGTH, CR2D_GET_CODEDATA )) == -2 )
			set_err_str( "ERR: get code data length is zero!!", RET_ERROR );

		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
	int cr2d_get_locpts( const unsigned int candidate, const unsigned int code_nb, IPL_POINT *pts )
	{
		dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
		int ret = -1;

		CRPGASSERT( pts != NULL, ret )
		CRPGASSERT_IPL( CR2D_Inquire( m_cr2d_resultsH, CR2D_GET_LOCATION, candidate, code_nb, pts ))
		ret = 0;

		dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
		return ret;
	}

	/*----------------------------------------------------------------*/
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __cr2d_h__

