//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>
#include <time.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_color.h>
#include <vl_ipl.h>
#include <vl_file.h>

#include <pgutils.h>
#include <samples.h>
#include <mvslib.h>
#include <mvsensor.h>

#include "crtool.h"
#include "prop.h"
#include "gui.h"
#include "preproc.h"
#include "ftools.h"
#include "pgtools.h"
#include "cr1d.h"
#include "cr2d.h"

//#define DISPLAY_GRAY_IMAGE

#ifdef MALLOC_TRACE
	#include <mcheck.h>
#endif	// MALLOC_TRACE

//-------------------------------------------------------------------
//-------------------------------------------------------------------
static DEV_HANDLE    deviceHandle = IPL_DONT_CARE;
static IPL_BUFHANDLE imageHandle = IPL_DONT_CARE;
static IPL_BUFHANDLE origHandle = IPL_DONT_CARE;
static IPL_BUFHANDLE grayHandle = IPL_DONT_CARE;
static IPL_BUFHANDLE destinationHandle = IPL_DONT_CARE;

static bool colorSensor = BASE_CheckModule( "Color" ) == IPL_TRUE;

static int width = 0, height = 0, pitch = 0, bpp = 0;
static BYTE *shm_buffer = NULL;
static BYTE *img_buffer = NULL;

static char read_txt[MAX_OBJ_CNT];
static char display_txt[ MAX_DISP_CNT];

int run_flag = 0;
static unsigned int dec_flag = 0, pp_flag = 0, dec_type = CT_TYPE_UNKNOWN;

static int displ_img_mode = -1;
static int displ_res_mode = DISP_ONE_RESULT;
static int bc_obj_index = -1, dm_obj_index = -1;
static long dec_time_ms = 0, preproc_time_ms = 0;

static CCR1D *cr1d = NULL;
static CCR2D *cr2d = NULL;

//-------------------------------------------------------------------
//-------------------------------------------------------------------
int get_stat_th( );

//-------------------------------------------------------------------
//-------------------------------------------------------------------
/*! \brief Free all buffers.*/
void close_all( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( img_buffer != NULL )
		IPL_ReleasePtr( imageHandle, IPL_LOCK_READ, 0, 0, 0, img_buffer );

	if( deviceHandle != IPL_DONT_CARE )
    	DEV_Close( deviceHandle );

	if( grayHandle != IPL_DONT_CARE )
		IPL_BufFree( grayHandle );
	if( origHandle != IPL_DONT_CARE )
		IPL_BufFree( origHandle );

	if( destinationHandle != IPL_DONT_CARE )
		IPL_BufFree( destinationHandle );

	if( cr1d ) delete cr1d;
	if( cr2d ) delete cr2d;

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Binarize the imageHandle.*/
void binarize_img( int th )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, th ));

	CR_PROP tmp_prop = prop_get_prop( CR_POLARITY );
	int pmode = tmp_prop.default_val;

	if( pmode != POL_UNKNOWN )
	{
		IPL_POINT_MODE bdirection = IPL_GREATER_CST;
		if( pmode == POL_W2B )
			bdirection = IPL_LESS_CST;

		if( destinationHandle == IPL_DONT_CARE )
		{
			if( !IPL_PtToPoint( imageHandle, th, &destinationHandle, bdirection ))
				fprintf( stderr, "*** " TOOL_PREFIX ": %s - binarize the image failed -> err: %ld!!\n",
								__FUNCTION__, IPL_GetErrorCode( ));
		}
		else
		{
			if( pp_flag )
			{
				if( !IPL_PtToPoint( destinationHandle, th, &destinationHandle, bdirection ))
					fprintf( stderr, "*** " TOOL_PREFIX ": %s - binarize the image failed -> err: %ld!!\n",
								__FUNCTION__, IPL_GetErrorCode( ));
			}
			else
			{
				if( !IPL_PtToPoint( imageHandle, th, &destinationHandle, bdirection ))
					fprintf( stderr, "*** " TOOL_PREFIX ": %s - binarize the image failed -> err: %ld!!\n",
								__FUNCTION__, IPL_GetErrorCode( ));
			}
		}
	}
	else
	{
		note_or_err_msg( "*** Polarity Error ***", "Please set the symbole polarity!!\nYou have to specify the symbole polarity\nfor the binary image!\nProbably it is set to 'Unknown'", 0 );
		evtLiveOFF( NULL );
		displ_update( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Make a binary image.*/
void mk_bin_img( )
{
	CR_PROP tmp_prop_thm = prop_get_prop( CR_THRESHOLD_MODE );
	CR_PROP tmp_prop_th = prop_get_prop( CR_THRESHOLD );
	CR_PROP tmp_prop_ct = prop_get_prop( CR_CODE_TYPE );

	dprintf((" ++ " TOOL_PREFIX ": %s - th-mod: %d, th: %d, code type: %d\n", __FUNCTION__,
			tmp_prop_thm.default_val, tmp_prop_th.default_val, tmp_prop_ct.default_val ));

	static int bin_flag = 1;

	switch( tmp_prop_thm.default_val )
	{
	case TH_MODE_AUTO:
		binarize_img( get_stat_th( ));
		break;
	case TH_MODE_MANUAL:
		binarize_img( tmp_prop_th.default_val );
		break;
	default:
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - wrong threshold mode!!\n", __FUNCTION__ );
		bin_flag = 0;
		break;
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
/*! \brief Copies the image to the display buffer.*/
int ipl_cp2displ( const IPL_BUFHANDLE imgBufH )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = IPL_FALSE;
	IPL_LONG ipitch = 0;
	BYTE *ibuff = NULL;

	if( imgBufH != IPL_DONT_CARE )
	{
		if(( ret = IPL_GetPtr( imgBufH, IPL_LOCK_READ, 0, 0, 0, &ipitch, &ibuff )))
		{
			cp_img( ibuff, shm_buffer, width, height, ipitch, pitch );
			IPL_ReleasePtr( imgBufH, IPL_LOCK_READ, 0, 0, 0, ibuff );
		}
		else
			fprintf( stderr, "*** cp_bin_img - no binary image found!\n");
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Saves the image to a file.*/
void save_img( )
{
	dprintf((" ++ "  TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	static char *pgm_file_name = PGM_FILE_NAME;
	BYTE *pgm_img = NULL;
	int img_size = 0;

	if( shm_buffer != NULL )
	{
		if(( img_size = mk_pgm( shm_buffer, &pgm_img, width, height, pitch, bpp )) > 0 )
		{
			if( pgm_img != NULL )
				save_file( pgm_img, pgm_file_name, ".pgm", img_size );
			else
				note_or_err_msg( SIMG_ERROR, "No PGM image in the buffer", 0 );
		}
		else
			note_or_err_msg( SIMG_ERROR, "Make a PGM image failed", 0 );
	}
	else
		note_or_err_msg( SIMG_ERROR, "No image in the SHM buffer", 0 );

	dprintf((" -- "  TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
// =SET= fct. block
//-------------------------------------------------------------------
/*! \brief Set the display mode.*/
int set_disp_img_mode( const int image_mode )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = RET_SUCCESS;

	displ_img_mode = image_mode;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__,ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Set the result display mode.*/
int set_disp_result_mode( const int result_mode )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, result_mode ));
	int ret = RET_SUCCESS;

	displ_res_mode = result_mode;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Activates the decoder.*/
void set_decoder( const unsigned int flag )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	dec_flag = flag;
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void set_decoder_type( const unsigned int type )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	dec_type = type;
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
// =GET= fct. block
//-------------------------------------------------------------------
/*! \brief Gets the number of found codes.*/
int get_cr1d_obj_cnt( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = 0;

	if( cr1d )
		ret = cr1d->m_cr1d_obj_cnt;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;;
}

//-------------------------------------------------------------------
/*! \brief Gets the number of found codes.*/
int get_cr2d_obj_cnt( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = 0;

	if( cr2d )
		ret = cr2d->m_cr2d_obj_cnt;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;;
}

//-------------------------------------------------------------------
/*! \brief Gets the static best threshold from the image.*/
int get_stat_th( )
{
	CR_PROP tmp_prop_thm = prop_get_prop( CR_THRESHOLD_MODE );
	CR_PROP tmp_prop_th = prop_get_prop( CR_THRESHOLD );

	dprintf((" ++ " TOOL_PREFIX ": %s - th-mode: %d, th: %d, pp_flag: %d\n",
				__FUNCTION__, tmp_prop_thm.default_val, tmp_prop_th.default_val, pp_flag ));

	long ret = tmp_prop_th.default_val;
	if( tmp_prop_thm.default_val != TH_MODE_MANUAL )
	{
		if( imageHandle != IPL_DONT_CARE  )
		{
			if( destinationHandle == IPL_DONT_CARE )
			{
				if( !IPL_StatBestThreshold( imageHandle, &ret ))
				{
					fprintf( stderr, "*** " TOOL_PREFIX ": %s - get best threshold failed - err: %ld!!\n", __FUNCTION__, IPL_GetErrorCode( ));
					ret = RET_ERROR;
				}
			}
			else
			{
				if( pp_flag )
				{
					if( !IPL_StatBestThreshold( destinationHandle, &ret ))
					{
						fprintf( stderr, "*** " TOOL_PREFIX ": %s - get best threshold failed - err: %ld!!\n",
										__FUNCTION__, IPL_GetErrorCode( ));
						ret = RET_ERROR;
					}
				}
				else
				{
					IPL_BufFree( destinationHandle );
					destinationHandle = IPL_DONT_CARE;

					if( !IPL_StatBestThreshold( imageHandle, &ret ))
					{
						fprintf( stderr, "*** " TOOL_PREFIX ": %s - get best threshold failed - err: %ld!!\n",
													__FUNCTION__, IPL_GetErrorCode( ));
						ret = RET_ERROR;
					}
				}
			}
		}

		if( ret >= 0 )
			prop_set_prop( CR_THRESHOLD, ret );
		else
			ret = tmp_prop_th.default_val;
	}
	else
		ret = tmp_prop_th.default_val;

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, ret ));

	return ret;
}

//-------------------------------------------------------------------
/*! \brief Gets the decoded data.*/
void get_cr1d_data( const int index, char *data )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - index: %d\n", __FUNCTION__, index ));

	if( cr1d )
	{
		if( cr1d->m_cr1d_obj_cnt > 0 )
		{
			if( index < 0 )
				sprintf( data, NO_ONJ_SEL_STR );
			else
			{
				if( strlen( cr1d->m_cr1d_res[index].data ))
					sprintf( data, "%s", cr1d->m_cr1d_res[index].data );
				else
					sprintf( data, "??????????" );
			}
		}
		else
			sprintf( data, "??????????" );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s)\n", __FUNCTION__, data ));
}

//-------------------------------------------------------------------
/*! \brief Gets the decoded data.*/
void get_cr2d_data( const int index, char *data )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - index: %d\n", __FUNCTION__, index ));

	if( cr2d )
	{
		if( cr2d->m_cr2d_obj_cnt > 0 )
		{
			if( index < 0 )
				sprintf( data, NO_ONJ_SEL_STR );
			else
			{
				if( strlen( cr2d->m_cr2d_res[index].data ))
					sprintf( data, "%s", cr2d->m_cr2d_res[index].data );
				else
					sprintf( data, "??????????" );
			}
		}
		else
			sprintf( data, "??????????" );
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s)\n", __FUNCTION__, data ));
}

//-------------------------------------------------------------------
/*! \brief Gets the code index.*/
int get_cr_index( const IPL_POINT click )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = RET_ERROR;
	IPL_RECT rect;
	if( cr1d )
	{
		if( cr1d->m_cr1d_obj_cnt > 0 )
		{
// 			unsigned int line_offset = 30;
			bc_obj_index = -1;
			for( unsigned int i = 0; i < cr1d->m_cr1d_obj_cnt; i++ )
			{
// 				rect.left   = cr1d->m_cr1d_res[i].line.x0 - line_offset;
// 				rect.top    = cr1d->m_cr1d_res[i].line.y0 - line_offset;
// 				rect.right  = cr1d->m_cr1d_res[i].line.x1 + line_offset;
// 				rect.bottom = cr1d->m_cr1d_res[i].line.y1 + line_offset;

				if( pt_in_rect( rect, click ) == RET_SUCCESS )
				{
					ret = i;
					bc_obj_index = i;
					break;
				}
			}
		}
	}

	if( cr2d && bc_obj_index < 0 )
	{
		if( cr2d->m_cr2d_obj_cnt > 0 )
		{
			dm_obj_index = -1;
			for( unsigned int i = 0; i < cr2d->m_cr2d_obj_cnt; i++ )
			{
				rect.left   = cr2d->m_cr2d_res[i].pts[0].xpos;
				rect.top    = cr2d->m_cr2d_res[i].pts[0].ypos;
				rect.right  = cr2d->m_cr2d_res[i].pts[2].ypos;
				rect.bottom = cr2d->m_cr2d_res[i].pts[2].ypos;

				if( pt_in_rect( rect, click ) == RET_SUCCESS )
				{
					ret = cr1d->m_cr1d_obj_cnt + i;
					dm_obj_index = i;
					break;
				}
			}
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief Gets the time needed by the decoder.*/
long get_dec_time( )
{
	dprintf((" ==> " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, dec_time_ms ));
	return dec_time_ms;
}

//-------------------------------------------------------------------
/*! \brief Gets the time needed by the preprocessing functions.*/
long get_preproc_time( )
{
	dprintf((" ==> " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, preproc_time_ms ));
	return preproc_time_ms;
}

//-------------------------------------------------------------------
// Image PreProcessing
//-------------------------------------------------------------------
#define PP_IMG_CLEAN( mode, src, dst )		\
	img_clean_##mode( src, dst, tmp_prop1.default_val )

#define PP_CELL_IMPR( mode, src, dst )		\
	cell_impr_##mode( src, dst, tmp_prop1.default_val, polarity_mode )

#define PP_SYMB_LOC( mode, src, dst )		\
	symb_loc_##mode( src, dst, tmp_prop1.default_val, polarity_mode )

//-------------------------------------------------------------------
/*! \brief Checks the settings for the preprocessing functions and
 *  calles them if needed.*/
int img_preproc( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int ret = IPL_TRUE;
	CR_PROP tmp_prop = prop_get_prop( CR_POLARITY ), tmp_prop1;
	int polarity_mode = tmp_prop.default_val;
	dprintf((" == " TOOL_PREFIX ": %s - polarity_mode: %d\n", __FUNCTION__, polarity_mode ));

	if( destinationHandle != IPL_DONT_CARE )
	{
		IPL_BufFree( destinationHandle );
		destinationHandle = IPL_DONT_CARE;
	}

	pp_flag = 0;

	if( run_flag )
	{
		tmp_prop = prop_get_prop( CR_IMG_CLEANING_MODE );
		tmp_prop1 = prop_get_prop( CR_FILTER_STRENGHT );
		switch( tmp_prop.default_val )
		{
		case IMG_CLEAN_BEST:
			dprintf((" == " TOOL_PREFIX ": %s - image clean (best)\n", __FUNCTION__ ));
			ret = PP_IMG_CLEAN( best, imageHandle, &destinationHandle );
			pp_flag = 1;
			break;

		case IMG_CLEAN_FAST:
			dprintf((" == " TOOL_PREFIX ": %s - image clean (fast)\n", __FUNCTION__ ));
			ret = PP_IMG_CLEAN( fast, imageHandle, &destinationHandle );
			pp_flag = 1;
			break;

		case IMG_CLEAN_SALT:
			dprintf((" == " TOOL_PREFIX ": %s - image clean (salt)\n", __FUNCTION__ ));
			ret = PP_IMG_CLEAN( salt_noise, imageHandle, &destinationHandle );
			break;

		case IMG_CLEAN_PEPR:
			dprintf((" == " TOOL_PREFIX ": %s - image clean (pepper)\n", __FUNCTION__ ));
			ret = PP_IMG_CLEAN( pepper_noise, imageHandle, &destinationHandle );
			pp_flag = 1;
			break;

		default:
			break;
		}
	}

	if( ret != IPL_FALSE && run_flag )
	{
		tmp_prop = prop_get_prop( CR_CELL_IMPR_MODE );
		tmp_prop1 = prop_get_prop( CR_DEFECT_SIZE );
		switch( tmp_prop.default_val )
		{
		case CELL_IMPR_2THIN:
			dprintf((" == " TOOL_PREFIX ": %s - cell improvement (too thin)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_CELL_IMPR( 2thin, imageHandle, &destinationHandle );
			else
				ret = PP_CELL_IMPR( 2thin, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		case CELL_IMPR_2THICK:
			dprintf((" == " TOOL_PREFIX ": %s - cell improvement (too thick)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_CELL_IMPR( 2thick, imageHandle, &destinationHandle );
			else
				ret = PP_CELL_IMPR( 2thick, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		case CELL_IMPR_NU:
			dprintf((" == " TOOL_PREFIX ": %s - cell improvement (too not uniform)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_CELL_IMPR( nu, imageHandle, &destinationHandle );
			else
				ret = PP_CELL_IMPR( nu, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		default:
			break;
		}
	}

	if( ret != IPL_FALSE && run_flag  )
	{
		tmp_prop = prop_get_prop( CR_SL_SHADING_CORR );
		tmp_prop1 = prop_get_prop( CR_CELL_SIZE );
		switch( tmp_prop.default_val )
		{
		case SYMB_LOC_SC_BEST:
			dprintf((" == " TOOL_PREFIX ": %s - Symbol location * shading correction (best)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_SYMB_LOC( sc_best, imageHandle, &destinationHandle );
			else
				ret = PP_SYMB_LOC( sc_best, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		case SYMB_LOC_SC_FAST:
			dprintf((" == " TOOL_PREFIX ": %s - Symbol location * shading correction (fast)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_SYMB_LOC( sc_fast, imageHandle, &destinationHandle );
			else
				ret = PP_SYMB_LOC( sc_fast, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		default:
			break;
		}
	}

	if( ret != IPL_FALSE && run_flag  )
	{
		tmp_prop = prop_get_prop( CR_SL_REG_SELECTION );
		switch( tmp_prop.default_val )
		{
		case SYMB_LOC_RS_BEST:
			dprintf((" == " TOOL_PREFIX ": %s - Symbol location * region selection (best)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_SYMB_LOC( rs_best, imageHandle, &destinationHandle );
			else
				ret = PP_SYMB_LOC( rs_best, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		case SYMB_LOC_RS_FAST:
			dprintf((" == " TOOL_PREFIX ": %s - Symbol location * region selection (fast)\n", __FUNCTION__ ));

			if( destinationHandle == IPL_DONT_CARE )
				ret = PP_SYMB_LOC( rs_fast, imageHandle, &destinationHandle );
			else
				ret = PP_SYMB_LOC( rs_fast, destinationHandle, &destinationHandle );

			pp_flag = 1;
			break;

		default:
			break;
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( ret == IPL_FALSE )
		return RET_ERROR;
	else
		return RET_SUCCESS;
}

//----------------------------------------------------------------------
int get_threshold( )
{
	CR_PROP tmp_prop_th = prop_get_prop( CR_THRESHOLD );
	dprintf(( " -> %s - threshold: %d\n", __FUNCTION__, tmp_prop_th.default_val ));
	return tmp_prop_th.default_val;
}

//----------------------------------------------------------------------
int get_cr1d_quality( const int index )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = 0;
	if( cr1d )
	{
		if( cr1d->m_cr1d_obj_cnt > 0 )
		{
			if( index >= 0 && index < (int)cr1d->m_cr1d_obj_cnt )
				ret = (int)( cr1d->m_cr1d_res[index].quality * 100 );
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
int get_cr2d_quality( const int /*index*/ )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = 0;

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
int cr1d_decode( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = -1;
	if( cr1d )
	{
		struct timeval start_tv, diff_tv;

		StartTimer ( &start_tv );
			if( destinationHandle == IPL_DONT_CARE )
				ret = cr1d->cr1d_decode( imageHandle );
			else
				ret = cr1d->cr1d_decode( destinationHandle );
		ElapsedTime ( &start_tv, &diff_tv, 30 );

		dec_time_ms += ch_tv2ms( diff_tv );
		dprintf((" == " TOOL_PREFIX ": %s - dec_time: %ld\n", __FUNCTION__, dec_time_ms ));
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
int cr2d_decode( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%s: %d)\n", __FUNCTION__, __FILE__, __LINE__ ));
	int ret = 0;

	if( cr2d )
	{
		struct timeval start_tv, diff_tv;

		StartTimer ( &start_tv );
			if( destinationHandle == IPL_DONT_CARE )
				ret = cr2d->cr2d_decode( imageHandle );
			else
				ret = cr2d->cr2d_decode( destinationHandle );
		ElapsedTime ( &start_tv, &diff_tv, 30 );

		dec_time_ms += ch_tv2ms( diff_tv );
		dprintf((" == " TOOL_PREFIX ": %s - dec_time: %ld\n", __FUNCTION__, dec_time_ms ));
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%s: %d) (%d)\n", __FUNCTION__, __FILE__, __LINE__, ret ));
	return ret;
}

//-------------------------------------------------------------------
static void config_cr( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_BARCODE )
	{
		if( cr1d )
			cr1d->cr1d_config( );
	}

	if( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_DATAMATRIX )
	{
		if( cr2d )
			cr2d->cr2d_config( );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
int set_dec_prop( const CR_PROPS_ID prop )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret = 0;

	CR_PROP tmp_prop = prop_get_prop( prop );
	switch( prop )
	{
	case CR_SPACING:
		cr1d->m_cr1d_spacing = tmp_prop.default_val;
		cr2d->m_cr2d_spacing = tmp_prop.default_val;
		break;
	case CR_CONTRAST:
		cr1d->m_cr1d_contrast = tmp_prop.default_val;
		cr2d->m_cr2d_contrast = tmp_prop.default_val;
		break;
	case CR_GAP_SIZE:
		cr1d->m_cr1d_gap_size = tmp_prop.default_val;
		cr2d->m_cr2d_gap_size = tmp_prop.default_val;
		break;
	case CR_MIN_GAPS:
		cr1d->m_cr1d_min_gaps = tmp_prop.default_val;
		break;
	case CR_NOISE_TH:
		cr1d->m_cr1d_noise_th = tmp_prop.default_val;
		break;
	default:
		ret = -1;
		break;
	}

	config_cr( );
	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
/*! \brief As the code reader accepts only gray or binary images, bayer images
 *         (what DEV_Grab yields on a color LYNX) need to be converted. */
static void possiblyConvertToGray( )
{
	if( colorSensor )
	{
		IPL_LONG grayColorSpace;
		if( COL_ConvertColorSpace( origHandle, &grayHandle, COL_BM2GRAY ) == IPL_FALSE ||
		    IPL_BufInquire( grayHandle, IPL_COLOR_SPACE, &grayColorSpace ) == IPL_FALSE ||
		    grayColorSpace != COL_GRAY )
		{
			exit_demo( "demosaicing failed", IPL_GetErrorCode2( ));
		}
#ifndef DISPLAY_GRAY_IMAGE
		imageHandle = grayHandle;
#endif
	}
#ifndef DISPLAY_GRAY_IMAGE
	else imageHandle = origHandle;
#endif
}

//-------------------------------------------------------------------
// Image snap, preprocessing and decode loop.
//-------------------------------------------------------------------
/*! \brief Image snap, preprocessing and decode.*/
void snap_img( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );
	static unsigned int pol_flag = 0;
	static int dec1d_ret = -1, dec2d_ret = -1;
	CR_PROP tmp_prop;
	struct timeval start_tv, diff_tv;

	pp_flag = 0;
	config_cr( );

	while( run_flag )
	{
		pgEventPoll( );

		dec_time_ms = 0;
		if( run_flag )
		{
			if( !DEV_Grab( deviceHandle, &origHandle ))
				exit_demo( "grab image failed", IPL_GetErrorCode( ));
			possiblyConvertToGray( );
		}

		if( run_flag )
		{
			if(( !dec_flag && displ_img_mode == PP_IMAGE ) || dec_flag || ( !dec_flag && displ_img_mode == BINARY_IMAGE ))
			{
				StartTimer ( &start_tv );

					if( img_preproc( ) < 0 )
					{
						evtLiveOFF( NULL );
						displ_update( );
					}

				ElapsedTime ( &start_tv, &diff_tv, 30 );
				preproc_time_ms = ch_tv2ms( diff_tv );
			}
		}

		if( run_flag )
		{
			if(( !dec_flag && displ_img_mode == BINARY_IMAGE ) || dec_flag )
			{
				tmp_prop = prop_get_prop( CR_POLARITY );
				if( tmp_prop.default_val != POL_UNKNOWN )
					mk_bin_img( );
				else
					set_err_str( "WARNING: Have no polarity!!", MVCOLOR_BLACK );
			}
		}


		if( dec_flag && run_flag )
		{
			if( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_BARCODE )
			{
				dec1d_ret = cr1d_decode( );
				dprintf((" == %s - cr1d_decode -> dec_ret: %d\n", __FUNCTION__, dec1d_ret ));
				displ_update( );
				dprintf((" == %s - cr1d_decode -> decoded in: %ld msec\n", __FUNCTION__, dec_time_ms ));
			}

			if( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_DATAMATRIX )
			{
				dec2d_ret = cr2d_decode( );
				dprintf((" == %s - cr2d_decode -> dec_ret: %d\n", __FUNCTION__, dec2d_ret ));
				displ_update( );
				dprintf((" == %s - cr2d_decode -> decoded in: %ld msec\n", __FUNCTION__, dec_time_ms ));
			}
		}

		if( displ_img_mode == BINARY_IMAGE )
			ipl_cp2displ( destinationHandle );
		else if( displ_img_mode == PP_IMAGE )
			ipl_cp2displ( destinationHandle );
		else
			cp_img( img_buffer, shm_buffer, width, height, pitch, pitch );
		displ_img( );

		if( dec_flag && run_flag )
		{
			if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_BARCODE ) && dec1d_ret >= 0 )
			{
				if(( dec1d_ret = cr1d->cr1d_get_candidats_and_codes( )) >= 0 )
					if(( dec1d_ret = cr1d->cr1d_get_results( )) >= 0 )
						bc_obj_index = 0;
			}

			if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_DATAMATRIX ) && dec2d_ret >= 0 )
			{
				if(( dec2d_ret = cr2d->cr2d_get_candidats_and_codes( )) >= 0 )
					if(( dec2d_ret = cr2d->cr2d_get_results( )) >= 0 )
						dm_obj_index = 0;
			}
		}

		if( dec_flag && run_flag )
		{
			if( displ_res_mode == DISP_ALL_RESULT )
			{
				if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_BARCODE ) && dec1d_ret >= 0 )
				{
					for( unsigned int i = 0; i < cr1d->m_cr1d_obj_cnt; i++ )
						display_pts_results( cr1d->m_cr1d_res[i].location, cr1d->m_cr1d_res[i].data, width/2, DISPLAY_TEXT_Y_OFFSET * i  );
				}

				if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_DATAMATRIX ) && dec2d_ret >= 0 )
				{
					for( unsigned int i = 0; i < cr2d->m_cr2d_obj_cnt; i++ )
						display_pts_results( cr2d->m_cr2d_res[i].pts, cr2d->m_cr2d_res[i].data, width/2, DISPLAY_TEXT_Y_OFFSET * i  );
				}
			}
			else if( displ_res_mode == DISP_ONE_RESULT )
			{
				if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_BARCODE ) && dec1d_ret >= 0 )
				{
					if( cr1d->m_cr1d_obj_cnt > 0 && bc_obj_index >= 0 )
						display_pts_results( cr1d->m_cr1d_res[bc_obj_index].location, cr1d->m_cr1d_res[bc_obj_index].data, width/2, 0  );
				}

				if(( dec_type == CT_TYPE_UNKNOWN || dec_type == CT_TYPE_DATAMATRIX ) && dec2d_ret >= 0 )
				{
					if( cr2d->m_cr2d_obj_cnt > 0 && dm_obj_index >= 0 )
						display_pts_results( cr2d->m_cr2d_res[dm_obj_index].pts, cr2d->m_cr2d_res[dm_obj_index].data, width/2, 0  );
				}
			}
		}

		if( run_flag )
			update_status_line( );

		displ_update( );
	}

	pol_flag = 0;
	displ_update( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
void set_cr_props( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s %d (%s)\n", __FUNCTION__, __LINE__, __FILE__ ));

	CR_PROP tmp_prop = prop_get_prop( CR_SPACING );
	cr1d->m_cr1d_spacing = tmp_prop.default_val;
	cr2d->m_cr2d_spacing = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_CONTRAST );
	cr1d->m_cr1d_contrast = tmp_prop.default_val;
	cr2d->m_cr2d_contrast = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_GAP_SIZE );
	cr1d->m_cr1d_gap_size = tmp_prop.default_val;
	cr2d->m_cr2d_gap_size = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_MIN_GAPS );
	cr1d->m_cr1d_min_gaps = tmp_prop.default_val;

	tmp_prop = prop_get_prop( CR_NOISE_TH );
	cr1d->m_cr1d_noise_th = tmp_prop.default_val;

	config_cr( );

	dprintf((" -- " TOOL_PREFIX ": %s %d (%s)\n", __FUNCTION__, __LINE__, __FILE__ ));
}

//----------------------------------------------------------------------
static int init_cr( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s %d (%s)\n", __FUNCTION__, __LINE__, __FILE__ ));
	int ret = 0;
	CR_PROP tmp_prop = prop_get_prop( CR_USE_INI_FILE );

	if( tmp_prop.default_val == 1 )
		set_cr_props( );

	dprintf((" -- " TOOL_PREFIX ": %s %d (%s) (%d)\n", __FUNCTION__, __LINE__, __FILE__, ret ));
	return ret;
}

//----------------------------------------------------------------------
static void init_device( )
{
	dprintf((" ++ " TOOL_PREFIX ": %s %d (%s)\n", __FUNCTION__, __LINE__, __FILE__ ));

	if( !DEV_Open( &deviceHandle, "matrixvision.ddc" ))
		exit_demo( "open device failed", IPL_GetErrorCode( ));

	if( !DEV_Control( deviceHandle, DEV_SINGLE, 0 ))
		exit_demo( "set control single failed", IPL_GetErrorCode( ));

	int trig_mode = -1;
	if( !DEV_InquireCaps( deviceHandle, DEV_TRIGGER, &trig_mode ))
		exit_demo( "get trigger mode failed", IPL_GetErrorCode( ));

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		int sw_trigmode = TRIG_SW;
		if( !DEV_ControlCaps( deviceHandle, DEV_TRIGGER, (IPL_VOID_PTR)&sw_trigmode ))
			exit_demo( "set SW trigger mode failed", IPL_GetErrorCode( ));
	}

	if( !DEV_Grab( deviceHandle, &origHandle ))
		exit_demo( "grab image failed", IPL_GetErrorCode( ));

	if( colorSensor )
	{
		long lBufAttr;
		IPL_BufInquire( origHandle, IPL_USAGETYPE, &lBufAttr );
		colorSensor = lBufAttr == IPL_BAYERMOSAICBUF; // we have a bayer image and a color license
		possiblyConvertToGray( );
	}

	// Whatever we assign to imageHandle here will determine what
	// image is displayed (original color or converted gray); the
	// actual decoding is always based on the converted gray image
	// regardless of what is displayed.
	// Note that in order to see a correct gray image, the pgserver
	// must be started with
	//   palette = mvlut_grey
	// in /etc/pgserver.conf, rather than
	//   palette = mvlut_bayer
	//
	imageHandle =
#ifdef DISPLAY_GRAY_IMAGE
		colorSensor ? grayHandle :
#endif
		origHandle;

	if( trig_mode != TRIG_SW && trig_mode != TRIG_AUTO )
	{
		if( !DEV_ControlCaps( deviceHandle, DEV_TRIGGER, (IPL_VOID_PTR)&trig_mode ))
			exit_demo( "reset the trigger mode failed", IPL_GetErrorCode( ));
	}

	if( !IPL_BufInquire( imageHandle, IPL_XSIZE, &width ))
		exit_demo( "inquier image width failed", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( imageHandle, IPL_YSIZE, &height ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));
	if( !IPL_BufInquire( imageHandle, IPL_NB_BANDS, &bpp ))
		exit_demo( "inquier image bit per pixel failed", IPL_GetErrorCode( ));

	dprintf((" -- " TOOL_PREFIX ": %s %d (%s)\n", __FUNCTION__, __LINE__, __FILE__ ));
}

//----------------------------------------------------------------------
// Main
//-------------------------------------------------------------------
/*! \brief Main.*/
int main( int argc, char *argv[] )
{
#ifdef MALLOC_TRACE
	mtrace();
#endif	// MALLOC_TRACE

	char msg[MSG_SIZE];

	memset( msg, 0, MSG_SIZE );
	sprintf( msg, "mvBlueLYNX Code Reader TOOL - " __DATE__ " / " __TIME__ );

	memset( read_txt, '?', MAX_OBJ_CNT );
	memset( display_txt, 0, MAX_DISP_CNT );

	read_ini_file( );

	/**/
	if(( cr1d = new CCR1D( )) == NULL )
	{
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - out of memory!!\n", __FUNCTION__ );
		return -1;
	}

	if(( cr2d = new CCR2D( )) == NULL )
	{
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - out of memory!!\n", __FUNCTION__ );
		return -1;
	}
	/**/

	init_cr( );

	pgInit( argc, argv );
	wApp = pgRegisterApp( PG_APP_NORMAL, msg, 0 );
	pgBind( PGDEFAULT, PG_WE_CLOSE, &evtClose, NULL );

	wBox = mk_buttons( wApp );
	wStatLine = mk_statline( wBox, PG_DERIVE_AFTER, 20 );

	get_s_len( );
	mk_statLineLabel( wStatLine );

	init_device( );

	if( !IPL_GetPtr( imageHandle, IPL_LOCK_READ, 0, 0, 0, (IPL_LONG*)&pitch, &img_buffer ))
		exit_demo( "inquier image height failed", IPL_GetErrorCode( ));

	get_img4displ( &shm_buffer, width, height );

	cp_img( img_buffer, shm_buffer, width, height, pitch, pitch );
	mk_disp( wStatLine, PG_DERIVE_AFTER );

	pghandle wTL = pgGetWidget( wApp, PG_WP_PANELBAR_LABEL );
	wCTRL = mk_btn( wTL, "HIDE CONTROL", PG_DERIVE_BEFORE, 80, &evtCTRL );

	pgUpdate( );

	CR_PROP tmp_prop = prop_get_prop( CR_PROCESS );

	if( tmp_prop.default_val == 1 )
		evtLiveON( NULL );
	if( tmp_prop.default_val == 2 )
		evtDecode( NULL );

	pgEventLoop( );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

