//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include <picogui.h>
#include <system.h>
#include <pgutils.h>

#include "crtool.h"
#include "ftools.h"

//-------------------------------------------------------------------
//-------------------------------------------------------------------
void widget_delete( pghandle *widget )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, *widget ));

	if( *widget != 0 )
	{
		pgDelete( *widget );
		*widget = 0;
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, *widget ));
}

//-------------------------------------------------------------------
static int bcMenuFromArray( const pghandle *items, const int numitems )
{
	dprintf((" ++ " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, numitems ));
	int i, ret;
	pghandle returnHandle;
	returnHandle = pgNewPopupAt( PG_POPUP_ATEVENT, PG_POPUP_ATEVENT, 0, 0 );

	for( i = 0; i < numitems; i++ )
	{
		pgNewWidget( PG_WIDGET_MENUITEM, 0, 0 );
		pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, items[i],
				0 );

		pgSetPayload( PGDEFAULT, i+1 );
	}

	pghandle evtFrom = pgGetEvent()->from;

	while( evtFrom == wMainCanvas )
	{
		dprintf((" ==** " TOOL_PREFIX ": %s - event from canvas: %ld\n", __FUNCTION__, evtFrom ));
		evtFrom = pgGetEvent()->from;
	}

	dprintf((" == " TOOL_PREFIX ": %s - event from: %ld\n", __FUNCTION__, evtFrom ));

	/* Return event */
	ret = pgGetPayload( evtFrom );

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));

	return ret;
}

//-------------------------------------------------------------------
int mk_popup_menu( char *menu_entrys, const unsigned int entry_cnt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - menu entrys: >%s<, cnt: %d\n", __FUNCTION__,
						menu_entrys, entry_cnt ));

	char *p, menu[MSG_SIZE];
	int ret;

	if( !menu_entrys || !*menu_entrys || entry_cnt <= 0 ) return 0;

	pghandle *items = (pghandle*) malloc( entry_cnt * sizeof( pghandle ));

	for( unsigned int i = 0; i < entry_cnt; i++ )
	{
		if( !( p = strchr( menu_entrys,'|' )))
			p = menu_entrys + strlen( menu_entrys );

		memset( menu, 0, MSG_SIZE );
		strncpy( menu, menu_entrys, strlen( menu_entrys ) - strlen( p ));

		pgFlushRequests( );
		menu_entrys = p + 1;
		items[i] = pgNewString( menu );
	}

	ret = bcMenuFromArray( items, entry_cnt );

	if( items )
		free( items );

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));

	return ret;
}

//-------------------------------------------------------------------
void cp_img( const unsigned char *src, unsigned char *dst, const int width, const int height,
				const int spitch, const int dpitch )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if( src != NULL )
	{
		int y;
		for( y = 0; y < height; y++ )
			memcpy( dst + y * dpitch, src + y * spitch, width );
	}
	else
	{
		dprintf(( "*** " TOOL_PREFIX ": %s - no source image found!!\n", __FUNCTION__ ));
//		fprintf( stderr, "*** " TOOL_PREFIX ": %s - no source image found!!\n", __FUNCTION__ );
		err_msg( "Empty source image",  -1 );
	}

	dprintf((" -- " TOOL_PREFIX ": %s \n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void repl_txt_si( STATUS_INFO si )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - >>%s<<\n", __FUNCTION__, si.msg ));
	pghandle oldtext = 0;

	oldtext = pgGetWidget( si.wText ,PG_WP_TEXT );
	if( si.msg_len != -1 )
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.msg ),
					PG_WP_COLOR, si.msg_col,
					PG_WP_SIZE, si.msg_len,
					0 );
	else
		pgSetWidget( si.wText,
					PG_WP_TEXT, pgNewString( si.msg ),
					PG_WP_COLOR, si.msg_col,
					0 );

	pgSubUpdate( si.wBox );
	pgSubUpdate( si.wText );

	if( oldtext != 0 )
		pgDelete( oldtext );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void repl_txt( pghandle widget, const char *text )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - >>%s<<\n", __FUNCTION__, text ));

	pgReplaceText( widget, text );
	pgSubUpdate( widget );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void draw_aoi( const pgcontext gc, const int x, const int y, const int w, const int h, const pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - x: %d, y: %d, w: %d, h: %d, color: 0x%x\n", __FUNCTION__,
								x, y, w, h, color ));

	pgSetColor( gc, color );
	pgFrame( gc, x, y, w, h );
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void draw_txt( const pgcontext gc, const int x, const int y, const char *text, const pgcolor color )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - x: %d, y: %d, text: >>%s<<, color: 0x%x\n", __FUNCTION__,
						x, y, text, color ));

	pgSetColor( gc, color );
	pgText( gc, x, y, pgNewString( text ));
	pgContextUpdate( gc );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//----------------------------------------------------------------------
void draw_line( const pgcontext gc, const int x1, const int y1, const int x2, const int y2, const pgcolor color )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s - x/y1: %d/%d, x/y2: %d/%d\n", __FUNCTION__, x1, y1, x2, y2 ));

	pgSetColor( gc, color );
	pgLine( gc, x1, y1, x2, y2 );
	pgContextUpdate( gc );

	dprintf(( " -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void note_or_err_msg( const char *msg_type, const char *s, const long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	if( err_code == 0 )
		sprintf( msg, "%s !!\n", s );
	else
		sprintf( msg, "%s -> err: %ld !!\n", s, err_code );

	pgMessageDialog( msg_type, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void err_msg( const char *s, const long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	sprintf( msg, "*** An error hase occurred! ***\n"
					"Error message and code:\n"
					"\"%s -> err: %ld\"!!\n", s, err_code );
	pgMessageDialog( AN_ERROR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	dprintf(( "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code ));
//	fprintf( stderr, "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void exit_demo( const char *s, const long err_code )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );

	sprintf( msg, "*** An fatal error hase occurred! ***\n"
					"Error message and code:\n"
					"\"%s -> err: %ld\"\n"
					"Application will be terminated!!", s, err_code );
	pgMessageDialog( "FATAL ERROR!!", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	dprintf(( "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code ));
//	fprintf( stderr, "*** " TOOL_PREFIX ": %s - err: %ld!!\n", s, err_code );
	close_all( );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	exit( err_code );
}

//-------------------------------------------------------------------
pghandle mk_chb( const pghandle parent, const char *txt, pgevthandler evtChBox, const int pos, const int status )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pghandle wRet = pgNewWidget( PG_WIDGET_CHECKBOX, pos, parent );
	pgSetWidget( wRet,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_EXTDEVENTS, PG_EXEV_TOGGLE,
				PG_WP_ON, status,
				0 );

	if( evtChBox != NULL )
		pgBind( wRet, PG_WE_ACTIVATE, evtChBox, NULL );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wRet;
}

//-------------------------------------------------------------------
pghandle mk_btn( const pghandle parent, const char *txt, const int pos, const int size, pgevthandler evtBtn )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_SIZE, size,
				PG_WP_FONT, pgNewFont( NULL, 6, 0 ),
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtBtn, NULL );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_ctrl_btn( const pghandle parent, const char *txt, const int pos, pgevthandler evtBtn, const pghandle color, int fsize )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pghandle wBtn = pgNewWidget( PG_WIDGET_BUTTON, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, fsize, 0 ),
				PG_WP_COLOR, color,
				0 );

	if( evtBtn != NULL )
		pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtBtn, NULL );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wBtn ;
}

//-------------------------------------------------------------------
pghandle mk_label( const pghandle parent, const char *txt, const int pos, const pghandle txt_col )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pghandle wLabel = pgNewWidget( PG_WIDGET_LABEL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_ALIGN, PG_A_RIGHT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, pgNewFont( NULL, 8, 0 ),
				PG_WP_COLOR, txt_col,
				PG_WP_SIZE, 8,
				0 );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	return wLabel;
}

//-------------------------------------------------------------------
pghandle mk_scroller( const pghandle parent, const int pos, const int max_s, const int def_s, pgevthandler evtScroll )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - max_s: %d, def_s: %d\n", __FUNCTION__, max_s, def_s ));

	pghandle wRet = pgNewWidget( PG_WIDGET_SCROLL, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, max_s,
				PG_WP_VALUE, def_s,
				PG_WP_SCROLL_X, 0,
				PG_WP_SCROLL_Y, def_s,
				0 );
	pgBind( PGDEFAULT, PG_WE_ACTIVATE, evtScroll, NULL );

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, wRet ));
	return wRet;
}

//-------------------------------------------------------------------
pghandle mk_wBox( const pghandle parent, const int pat_pos, const int side, const int size )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pghandle wRet = pgNewWidget( PG_WIDGET_BOX, pat_pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, side,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, wRet ));
	return wRet;
}

//-------------------------------------------------------------------
pghandle mk_scroller_lable( const pghandle parent, const int pos, char *txt )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	int x,y;
	pghandle font = pgNewFont( NULL, 6, PG_FSTYLE_DEFAULT );
	get_str_size( &x, &y, font, txt );

	dprintf((" == " TOOL_PREFIX ": %s - txt: >>%s<< (x/y: %d/%d)\n", __FUNCTION__, txt, x, y ));

	pghandle wRet = mk_wBox( parent, pos, PG_S_LEFT, x + 5 );
	pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, PGDEFAULT );
	pgSetWidget( PGDEFAULT,
				PG_WP_TEXT, pgNewString( txt ),
				PG_WP_FONT, font,
				PG_WP_SIZE, x,
				PG_WP_SIDE, PG_S_LEFT,
				0 );

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, wRet ));
	return wRet;
}

//-------------------------------------------------------------------
pghandle mk_statline( const pghandle parent, const int pos, const int size )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	pghandle wRet = pgNewWidget( PG_WIDGET_TOOLBAR, pos, parent );
	pgSetWidget( PGDEFAULT,
				PG_WP_SIDE, PG_S_TOP,
				PG_WP_SIZE, size,
				0 );

	dprintf((" -- " TOOL_PREFIX ": %s (%ld)\n", __FUNCTION__, wRet ));
	return wRet;
}

//-------------------------------------------------------------------
void get_str_size( int *x, int *y, const pghandle font, const char *str )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	pgEnterContext( );
	pgSizeText( x, y, font, pgNewString( str ));
	pgLeaveContext( );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void mk_stll_si( const pghandle parent, const int pos, STATUS_INFO *si )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int x,y, offs = 5;

	if( si->msg != NULL )
	{
		x = si->msg_len;

		if( x < 0 )
		{
			pghandle font = pgNewFont( NULL, 8, 0 );
			get_str_size( &x, &y, font, si->msg );
		}

		si->wBox = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
		si->wText = mk_label( PGDEFAULT, si->msg, PG_DERIVE_INSIDE, si->msg_col );
	}
	else
	{
		dprintf(( "*** " TOOL_PREFIX ": %s - wrong msg (%s)!!\n", __FUNCTION__, si->msg ));
//		fprintf( stderr, "*** " TOOL_PREFIX ": %s - wrong msg (%s)!!\n", __FUNCTION__, si->msg );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
pghandle mk_stll( const pghandle parent, const int pos, pghandle *wText, const char *str, const int size )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int x,y, offs = 5;

	x = size;

	if( size < 0 )
	{
		pghandle font = pgNewFont( NULL, 8, 0 );
		get_str_size( &x, &y, font, str );
	}

	pghandle wB = mk_wBox( parent, pos, PG_S_LEFT, x + offs );
	*wText = mk_label( PGDEFAULT, str, PG_DERIVE_INSIDE, MVCOLOR_BLACK );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return wB;
}

//-------------------------------------------------------------------
void set_w_color( pghandle widget, const pghandle color )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	pgSetWidget( widget, PG_WP_COLOR, color, 0 );
	pgSubUpdate( widget );

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
int check_if_file_exist( const char *fname )
{
	FILE *f;
	char s[MAX_CHAR_SIZE];
	memset( s, 0, MAX_CHAR_SIZE );

	dprintf(( " ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	if(( f = fopen( fname, "r" )) == NULL )
	{
		sprintf( s, "No '%s' found!!", fname );
		pgMessageDialog( KB_ERROR, s, PG_MSGBTN_OK | PG_MSGICON_ERROR );
		return -1;
	}

	fclose( f );

	dprintf(( " -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return 0;
}

//-------------------------------------------------------------------
void get_s_size( const char *s, int *x, int *y )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s - s: >>%s<<, x: %d, y: %d\n", __FUNCTION__, s, *x, *y ));
	pghandle font = pgNewFont( NULL, 8, 0 );
	get_str_size( x, y, font, s );
	dprintf(( " -- " TOOL_PREFIX ": %s - x: %d, y: %d\n", __FUNCTION__, *x, *y ));
}

//-------------------------------------------------------------------
int get_x_size( const char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s - s: >>%s<<\n", __FUNCTION__, s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": %s - x: %d\n", __FUNCTION__, x ));
	return x;
}

//-------------------------------------------------------------------
int get_y_size( const char *s )
{
	dprintf(( " ++ " TOOL_PREFIX ": %s - s: >>%s<<\n", __FUNCTION__, s ));

	int x = 0, y = 0;
	get_s_size( s, &x, &y );

	dprintf(( " -- " TOOL_PREFIX ": %s - y: %d\n", __FUNCTION__, y ));
	return y;
}

//-------------------------------------------------------------------
void ni( )
{
	pgMessageDialog( NOTE_STR, "\nThis functionality is not implemented yet!\n\nBut it soon will be :o)\n", PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
}

//-------------------------------------------------------------------
void note4max_limit( const char *type, const int val )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "The maximal %s is limitated to %d!!\n"
					"The %s will be set to this value!\n", type, val, type );
	pgMessageDialog( NOTE_STR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void note4min_limit( const char *type, const int val )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	char msg[MAX_CHAR_SIZE];
	memset( msg, 0, MAX_CHAR_SIZE );
	sprintf( msg, "The entered value is %d!! This is an illegal value!\n"
					"The %s will be set to the default value!\n", val, type );
	pgMessageDialog( NOTE_STR, msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
pghandle get_stat_color( const int col_stat_mode )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	pghandle ret;

	if( col_stat_mode == 0 )
		ret = MVCOLOR_GREY;
	else if( col_stat_mode == RET_ERROR )
		ret = MVCOLOR_RED;
	else
		ret = MVCOLOR_BLACK;

	dprintf((" -- " TOOL_PREFIX ": %s (0x%6lx)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
void si_iupdate( STATUS_INFO *si, const char *msg_prefix, const char *msg_sufix,
			const int value, const int color_mode, const int msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	if( msg_prefix != NULL )
		sprintf( msg, "%s: %d", msg_prefix, value );
	if( msg_sufix != NULL )
		sprintf( msg, "%d %s", value, msg_sufix );
	if( msg_prefix != NULL && msg_sufix != NULL )
	{
		memset( msg, 0, MSG_SIZE );
		sprintf( msg, "%s: %d %s", msg_prefix, value, msg_sufix );
	}

	si->msg = msg;

	if( msg_len == -1 )
		si->msg_len = msg_len;
	else
		si->msg_len = get_x_size( si->msg );

	si->msg_col = get_stat_color( color_mode );

	repl_txt_si( *si );
	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
void si_cupdate( STATUS_INFO *si, const char *msg_prefix, const char *msg_sufix,
					const char *data, const int color_mode, const int msg_len )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - msg_prefix: >>%s<<, msg_sufix: >>%s<<, data: >>%s<<, color_mode: %d, msg_len: %d\n", __FUNCTION__,
							msg_prefix, msg_sufix, data, color_mode, msg_len ));
	char msg[MSG_SIZE];
	memset( msg, 0, MSG_SIZE );

	if( data != NULL )
	{
		if( msg_prefix != NULL && ( strncmp( PREFIX_4_MSG_DATA, msg_prefix, strlen( PREFIX_4_MSG_DATA )) == 0 ))
		{
			sprintf( msg, "%s", data );
		}
		else
		{
			if( msg_prefix != NULL && msg_sufix != NULL )
				sprintf( msg, "%s: >>%s<< %s", msg_prefix, data, msg_sufix );
			else if( msg_prefix == NULL && msg_sufix == NULL )
				sprintf( msg, ">>%s<<", data );
			else if( msg_prefix != NULL )
				sprintf( msg, "%s: >>%s<<", msg_prefix, data );
			else if( msg_sufix != NULL )
				sprintf( msg, ">>%s<< %s", data, msg_sufix );
		}

		si->msg = msg;

		if( msg_len == -1 )
			si->msg_len = msg_len;
		else
			si->msg_len = get_x_size( si->msg );

		si->msg_col = get_stat_color( color_mode );

		repl_txt_si( *si );
	}

	dprintf((" -- " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
}

//-------------------------------------------------------------------
static int save_pgm_file( const char *data_buf, FILE *f, const int fsize )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	int ret;
	char str[MSG_SIZE];

	if(( ret = fwrite (data_buf, 1, fsize, f )) != fsize )
	{
		memset( str, 0, MAX_CHAR_SIZE );
		sprintf( str,"%d BYTES written from %d", ret, fsize );
		note_or_err_msg( "Save error", str, 0 );
		ret = RET_ERROR;
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//-------------------------------------------------------------------
int save_file( const unsigned char *data_buf, const char *fname, const char *ftype, const int fsize )
{
	dprintf((" ++ " TOOL_PREFIX ": %s\n", __FUNCTION__ ));

	const char *file;
	char *str, msg[MAX_CHAR_SIZE];
	FILE *f = NULL;
	int /*i, */ret = RET_ERROR;

	memset( msg, 0, MAX_CHAR_SIZE );

	file = pgFilePicker( NULL, NULL, fname, PG_FILESAVE, "Save the file");
	if( file )
	{
		str = strstr( file, ftype );

		if( str == NULL )
		{
			sprintf( msg, "Wrong file name specified!!\n Please enter a *%s file name", ftype );
			note_or_err_msg( SAVE_ERROR, msg, 0 );
			return ret;
		}

		//i = pgMessageDialog("Saving the file...", file, PG_MSGBTN_YES | PG_MSGBTN_NO );

		//if( i == PG_MSGBTN_YES )
		{
			if( strncmp( ".ini", ftype, 4 ) == 0 )
			{
				f = fopen( file, "w" );
				ret = save_ini_file( f );
			}
			else if( strncmp( ".pgm", ftype, 4 ) == 0 )
			{
				f = fopen( file, "wb" );
				if( data_buf != NULL )
					ret = save_pgm_file( (char*)data_buf, f, fsize );
				else
					note_or_err_msg( SAVE_ERROR, "No image data found", 0 );
			}
			else
				note_or_err_msg( SAVE_ERROR, "Wrong file type specified", 0 );

			if( f != NULL )
			{
				fclose( f );
				f = NULL;
			}
		}
	}

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));

	return ret;
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
