//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>crtool<<
// Don Apr 29 14:28:38 CEST 2004
//----------------------------------------------------------------------
#include <stdio.h>

#include "crtool.h"
#include "prop.h"
#include "gui.h"
#include "preproc.h"
#include "ftools.h"

//----------------------------------------------------------------------
static CR_PROP prop_list[ ] =
{
			// name/ID				default 						min								max
	CR_FILL_PROP( NO_PROP,           -1,                          -1,                         -1 ),
	CR_FILL_PROP( USE_INI_FILE,      0,                            0,                          1 ),
	CR_FILL_PROP( PROCESS,           0,                            0,                          2 ),
	CR_FILL_PROP( CODE_TYPE,         CT_TYPE_UNKNOWN,             CT_TYPE_NSET,               CT_TYPE_UNKNOWN ),
	CR_FILL_PROP( POLARITY,          POL_UNKNOWN,                 POL_NSET,                   POL_UNKNOWN ),
	CR_FILL_PROP( BC_TYPE,           BC_TYPE_UNKNOWN,             BC_TYPE_NSET,               BC_TYPE_UNKNOWN ),
	CR_FILL_PROP( DM_TYPE,           DM_TYPE_UNKNOWN,             DM_TYPE_NSET,               DM_TYPE_UNKNOWN ),
	CR_FILL_PROP( THRESHOLD_MODE,    TH_MODE_AUTO,                TH_MODE_NSET,               TH_MODE_MANUAL ),
	CR_FILL_PROP( THRESHOLD,         0,                            0,                         MAX_THRESHOLD ),
	CR_FILL_PROP( MAX_OBJ,           DEFAULT_OBJ_CNT,              1,                         MAX_OBJ_CNT ),
	CR_FILL_PROP( SPACING,           DEFAULT_SPACING,              0,                         100 ),
	CR_FILL_PROP( CONTRAST,          DEFAULT_CONTRAST,             0,                         100 ),
	CR_FILL_PROP( GAP_SIZE,          DEFAULT_GAP_SIZE,             0,                         100 ),
	CR_FILL_PROP( MIN_GAPS,          DEFAULT_MIN_GAPS,             0,                         100 ),
	CR_FILL_PROP( NOISE_TH,          DEFAULT_NOISE_TH,             0,                         100 ),
	CR_FILL_PROP( IMG_CLEANING_MODE, IMG_CLEAN_NONE,              IMG_CLEAN_BEST,             IMG_CLEAN_NONE ),
	CR_FILL_PROP( FILTER_STRENGHT,   DEFAULT_FL_STRENGTH,         DEFAULT_FL_STRENGTH,        MAX_FL_STRENGTH ),
	CR_FILL_PROP( CELL_IMPR_MODE,    CELL_IMPR_NONE,              CELL_IMPR_2THIN,            CELL_IMPR_NONE ),
	CR_FILL_PROP( DEFECT_SIZE,       DEFAULT_DEFECT_SIZE,         DEFAULT_DEFECT_SIZE,        MAX_DEFECT_SIZE ),
	CR_FILL_PROP( SL_SHADING_CORR,   SYMB_LOC_SC_NONE,            SYMB_LOC_SC_BEST,           SYMB_LOC_SC_NONE ),
	CR_FILL_PROP( SL_REG_SELECTION,  SYMB_LOC_RS_NONE,            SYMB_LOC_RS_BEST,           SYMB_LOC_RS_NONE ),
	CR_FILL_PROP( CELL_SIZE,         DEFAULT_CELL_SIZE,            1,                         MAX_CELL_SIZE ),
};

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \brief Sets the default property value.*/
int prop_set_prop( const CR_PROPS_ID index, const int value )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - index: %d, value: %d\n", __FUNCTION__, index, value ));

	int ret = RET_ERROR;
	int list_size = sizeof( prop_list )/sizeof( prop_list[0] );

	if( index > CR_NO_PROP && index < list_size )
		prop_list[index].default_val = prop_check_val( prop_list[index], value );
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - unknown property (%d)!!\n", __FUNCTION__, index );

	dprintf((" -- " TOOL_PREFIX ": %s (%d)\n", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
/*! \brief Returns the inquired property.*/
CR_PROP prop_get_prop( const CR_PROPS_ID index )
{
	dprintf((" ++ " TOOL_PREFIX ": %s - index: %d\n", __FUNCTION__, index ));

	CR_PROP ret = prop_list[0];
	int list_size = sizeof( prop_list )/sizeof( prop_list[0] );

	if( index > CR_NO_PROP && index < list_size )
		ret = prop_list[index];
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - unknown property (%d)!!\n", __FUNCTION__, index );

	dprintf((" -- " TOOL_PREFIX ": %s (%s)\n", __FUNCTION__, ret.name ));
	return ret;
}

//----------------------------------------------------------------------
/*! \brief Returns the size of the property list.*/
int prop_get_prop_list_size( )
{
	dprintf((" -> " TOOL_PREFIX ": %s\n", __FUNCTION__ ));
	return ( sizeof( prop_list )/sizeof( prop_list[0] ));
}

//----------------------------------------------------------------------
/*! \brief Checks if the value matches with the property range.*/
int prop_check_val( const CR_PROP prop, const int val )
{
	dprintf((" ++ "  TOOL_PREFIX ": %s - val: %d, min/max: %d/%d\n", __FUNCTION__, val, prop.min_val, prop.max_val ));

	int ret;

	if( val >= prop.min_val && val <= prop.max_val )
		ret = val;
	else
		ret = prop.default_val;

	dprintf((" -- "  TOOL_PREFIX ": %s (%d)", __FUNCTION__, ret ));
	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
