//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>gmmtool<<
// Don Nov 13 11:40:07 CET 2003
//----------------------------------------------------------------------
#include <stdio.h>
#include <math.h>
#include <vector>
using namespace std;

#include <vl_base.h>
#include <vl_gmm.h>
#include <vl_device.h>

#include <mvslib.h>
#include <mvsensor.h>

#include <pgutils.h>
#include <samples.h>

#include "gmmtool.h"
#include "gmmipl.h"
#include "pgapi.h"
#include "pgdev.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define COPY_PTS2IMG			1

#define MAX_POINTS				(640*480/8)	//4096

//----------------------------------------------------------------------
//----------------------------------------------------------------------
struct SGMM_TRIG_RESULTS
{
	vector<IPL_LINE_FLOAT> vEdge1;
	vector<IPL_LINE_FLOAT> vEdge2;
	vector<float> vX;
	vector<float> vY;
	vector<float> vR;

	char msg[MAX_CHAR_SIZE];
};

typedef struct SGMM_TRIG_RESULTS SGMM_TRIG_RESULTS;

struct SGMM_MT_RESULTS
{
	vector<IPL_POINT_FLOAT> vMatchPoints;
	char msg[MAX_CHAR_SIZE];
};

typedef struct SGMM_MT_RESULTS SGMM_MT_RESULTS;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static int threshold = DEFAULT_THRESHOLD, quality = DEFAULT_QUALITY;
static int max_feat_nb = MAX_FEAT_NB, max_obj_nb = MAX_OBJ_NB;
static int teach_flag = 0, new_res_flag = 0;
static double gmm_time_ms = 0.0;
static IPL_ULONG obj_nb = 0;

static GMM_MODELHANDLE model = IPL_DONT_CARE;
static GMM_MATCHERHANDLE matcher = IPL_DONT_CARE;
static IPL_BUFHANDLE bin_image = IPL_DONT_CARE;

static vector<GMM_FEATUREHANDLE> vCornerHandles;
static vector<GMM_FEATUREHANDLE> vArcHandles;
static vector<GMM_CORNER>        vCorners;
static vector<GMM_ARC>           vArcs;

static SGMM_TRIG_RESULTS gmm_tres[MAX_OBJ_NB];
static SGMM_MT_RESULTS gmm_mtres[MAX_OBJ_NB];

static IPL_ULONG corner_cnt = 0, arc_cnt = 0;

BYTE *img_buffer = NULL;

static IPL_POINT_FLOAT ModelPoints[MAX_POINTS];
static IPL_ULONG points_cnt = 0;
static int draw_res_type = GMM_DISPL_MATCH_TABLE;
static char res_txt[MAX_OBJ_NB][MAX_CHAR_SIZE];

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static int fillEdgePoints( IPL_BUFHANDLE hBuffer, IPL_POINT_FLOAT *Points )
{
	dprintf((" ++ " TOOL_PREFIX ": fillEdgePoints\n"));
	int count = 0;
	IPL_LONG lPitch = 0;
	IPL_UCHAR *ptr = NULL ;
	IPL_ULONG lHeight = 0;

	IPL_BufInquire( hBuffer, IPL_YSIZE, &lHeight );
	IPL_GetPtr( hBuffer, IPL_LOCK_READ, 0, 0, 0, &lPitch, &ptr );

	for( IPL_ULONG y = 0; y < lHeight; y++ )
	{
		for( IPL_ULONG x = 0; x < (IPL_ULONG)lPitch; x++ )
		{
			if( ptr[ y * lPitch + x ] )
			{
				Points[count].x = x;
				Points[count].y = y;
				count++;
			}

			if( count >= MAX_POINTS )
			{
				count = -1;
				break;
			}
		}

		if( count < 0 )
			break;
	}

	if( count < 0 )
		fprintf( stderr, "*** " TOOL_PREFIX ": %s - max allowed point number overflow(MAX: %d)!!\n", __FUNCTION__, MAX_POINTS );

	IPL_ReleasePtr( hBuffer, IPL_LOCK_READ, 0, 0, 0, ptr );

	dprintf((" -- " TOOL_PREFIX ": fillEdgePoints (%d)\n", count ));
	return count;
}

//----------------------------------------------------------------------
static void rotatePointRad( float cx, float cy, float* pX, float* pY, float fAngle )
{
	dprintf((" ++ " TOOL_PREFIX ": rotatePointRad\n"));

	// transform center to 0,0
	*pX -= cx;
	*pY -= cy;

	// rotate x,y
	float fsinD = (float)sin(-fAngle); // - because of screen coordinates
	float fcosD = (float)cos(fAngle);

	float ox,oy;
	ox = *pX;
	oy = *pY;

	*pX = ox * fcosD - oy * fsinD;
	*pY = oy * fcosD + ox * fsinD;

	// back transformation
	*pX += cx;
	*pY += cy;

	dprintf((" -- " TOOL_PREFIX ": rotatePointRad\n"));
}

//----------------------------------------------------------------------
static void transformPointRad( float oldx, float oldy, const GMM_MATCH& imatches, float& newX, float& newY )
{
	dprintf((" ++ " TOOL_PREFIX ": transformPointRad\n"));

	rotatePointRad( 0, 0, &oldx, &oldy, imatches.r );

	newX = imatches.x + imatches.s * oldx;
	newY = imatches.y + imatches.s * oldy;

	dprintf((" -- " TOOL_PREFIX ": transformPointRad\n"));
}

//----------------------------------------------------------------------
static IPL_LINE_FLOAT transformLine( const IPL_LINE_FLOAT& edge, const GMM_MATCH& imatches )
{
	dprintf((" ++ " TOOL_PREFIX ": transformLine\n"));
	IPL_LINE_FLOAT result;

	transformPointRad( edge.x1, edge.y1, imatches, result.x1, result.y1 );
	transformPointRad( edge.x2, edge.y2, imatches, result.x2, result.y2 );

	dprintf((" -- " TOOL_PREFIX ": transformLine\n"));
	return result;
}

//----------------------------------------------------------------------
static void transformArc( const GMM_MATCH& imatches, float in_x, float in_y, float in_r, float *x, float *y, float *r )
{
	dprintf((" ++ " TOOL_PREFIX ": transformArc\n"));

	transformPointRad( in_x, in_y, imatches, *x, *y );
	*r = in_r * imatches.s;

	dprintf((" -- " TOOL_PREFIX ": transformArc\n"));
}

//----------------------------------------------------------------------
static void clear_tres_bufs( SGMM_TRIG_RESULTS *gmm_res )
{
	gmm_res->vEdge1.clear( );
	gmm_res->vEdge2.clear( );
	gmm_res->vX.clear( );
	gmm_res->vY.clear( );
	gmm_res->vR.clear( );
	memset( gmm_res->msg, 0, MAX_CHAR_SIZE );

}

//----------------------------------------------------------------------
static void get_trig_results( const GMM_MATCH& imatches, int index )
{
	dprintf((" ++ " TOOL_PREFIX ": get_trig_results\n"));
	new_res_flag = 0;

	clear_tres_bufs( &gmm_tres[index] );

	for( IPL_ULONG i = 0; i < corner_cnt; i++ )
	{
		gmm_tres[index].vEdge1.push_back( transformLine( vCorners[i].line1, imatches ));
		gmm_tres[index].vEdge2.push_back( transformLine( vCorners[i].line1, imatches ));

		dprintf((" == " TOOL_PREFIX ": get_trig_results - ci: %ld, e1(%.3f/%.3f/%.3f/%.3f), e2(%.3f/%.3f/%.3f/%.3f)\n",
					i, gmm_tres[index].vEdge1[i].x1, gmm_tres[index].vEdge1[i].y1,
					gmm_tres[index].vEdge1[i].x2, gmm_tres[index].vEdge1[i].y2,
					gmm_tres[index].vEdge2[i].x1, gmm_tres[index].vEdge2[i].y1,
					gmm_tres[index].vEdge2[i].x2, gmm_tres[index].vEdge2[i].y2 ));
	}

	float x, y, r;
	for( IPL_ULONG i = 0; i < arc_cnt; i++ )
	{
		transformArc( imatches, vArcs[i].x, vArcs[i].y, vArcs[i].radius, &x, &y, &r );
		dprintf((" == " TOOL_PREFIX ": get_trig_results - ai: %ld, x: %.3f, y: %.3f, r: %.3f\n", i, x, y, r ));

		gmm_tres[index].vX.push_back( x );
		gmm_tres[index].vY.push_back( y );
		gmm_tres[index].vR.push_back( r );
	}

	sprintf( gmm_tres[index].msg, "scaling: %3.2f, score: %3.2f", imatches.s, imatches.score );

	new_res_flag = 1;
	dprintf((" -- " TOOL_PREFIX ": get_trig_results\n"));
}

//----------------------------------------------------------------------
static void clear_mtres_bufs( SGMM_MT_RESULTS *gmm_res )
{
	gmm_res->vMatchPoints.clear( );
	memset( gmm_res->msg, 0, MAX_CHAR_SIZE );
}

//----------------------------------------------------------------------
static void get_match_table_results( const GMM_MATCH& imatches, int index )
{
	dprintf((" ++ " TOOL_PREFIX ": get_match_table_results\n"));

	clear_mtres_bufs( &gmm_mtres[index] );
	gmm_mtres[index].vMatchPoints.resize( points_cnt );

	for( IPL_ULONG p = 0; p < points_cnt; p++ )
		transformPointRad( ModelPoints[p].x, ModelPoints[p].y, imatches,
						gmm_mtres[index].vMatchPoints[p].x, gmm_mtres[index].vMatchPoints[p].y );

	sprintf( gmm_mtres[index].msg, "scaling: %3.2f, score: %3.2f", imatches.s, imatches.score );
	new_res_flag = 1;
	dprintf((" -- " TOOL_PREFIX ": get_match_table_results\n"));
}

//----------------------------------------------------------------------
static int get_edge_points( )
{
	dprintf((" ++ " TOOL_PREFIX ": get_edge_points\n" ));
	int ret = IPL_FALSE;

	if( !( ret = GMM_ModelGetEdgePoints( model, &bin_image )))
		fprintf( stderr, "*** " TOOL_PREFIX ": get_edge_points - GMM_ModelGetEdgePoints failed (%ld)!!\n", IPL_GetErrorCode( ));

	int cnt = 0;
	if(( cnt = fillEdgePoints( bin_image, ModelPoints )) > 0 )
	{
		points_cnt = cnt;
		clear_canvas( );
		if( cp_bin_img( bin_image ))
		{
			displ_img( );
			redraw_canvas( );
		}

		if( bin_image != IPL_DONT_CARE )
		{
			IPL_BufFree( bin_image );
			bin_image = IPL_DONT_CARE;
		}
	}
	else if( cnt < 0 )
		points_cnt = 0;
	else
		ret = IPL_FALSE;

	dprintf((" -- " TOOL_PREFIX ": get_edge_points (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int set_matcher_score( )
{
	dprintf((" ++ " TOOL_PREFIX ": set_matcher_score\n" ));
	int ret = IPL_FALSE;

	if( matcher != IPL_DONT_CARE )
	{
		IPL_DOUBLE f = (IPL_DOUBLE)quality/100.;
		if( !( ret = GMM_MatcherControl( matcher, MATCHER_MIN_SCORE, &f )))
			fprintf( stderr, "*** " TOOL_PREFIX ": set_matcher_score - failed (%ld)!!\n", IPL_GetErrorCode( ));
	}

	dprintf((" -- " TOOL_PREFIX ": set_matcher_score (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int set_matcher_nb_match( )
{
	dprintf((" ++ " TOOL_PREFIX ": set_matcher_nb_match\n" ));
	int ret = IPL_FALSE;

	if( matcher != IPL_DONT_CARE )
	{
		IPL_LONG  n = max_obj_nb;
		if( !( ret = GMM_MatcherControl( matcher, MATCHER_NUMBER_OF_MATCHES, &n )))
			fprintf( stderr, "*** " TOOL_PREFIX ": set_matcher_nb_match - failed (%ld)!!\n", IPL_GetErrorCode( ));
	}

	dprintf((" -- " TOOL_PREFIX ": set_matcher_nb_match (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int set_matcher_control( )
{
	dprintf((" ++ " TOOL_PREFIX ": set_matcher_control\n" ));
	int ret;

	ret = set_matcher_score( );
	ret = set_matcher_nb_match( );

	dprintf((" -- " TOOL_PREFIX ": set_matcher_control (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int get_corners( )
{
	dprintf((" ++ " TOOL_PREFIX ": get_corners\n" ));
	int ret = IPL_FALSE;

	vCornerHandles.clear( );
	vCorners.clear( );
	if( !( ret = GMM_ModelCountCorners( model, &corner_cnt )))
		fprintf( stderr, "*** " TOOL_PREFIX ": get_corners - GMM_ModelCountCorners failed (%ld)!!\n", IPL_GetErrorCode( ));

	if( corner_cnt > 0 && ret )
	{
		vCornerHandles.resize( corner_cnt );
		if(( ret = GMM_ModelGetCornerHandles( model, &(*vCornerHandles.begin( )))))
		{
			GMM_CORNER corner;
			for( IPL_ULONG i = 0; i < corner_cnt; i++ )
			{
				GMM_ModelGetCorner( model, vCornerHandles[i], &corner );
				vCorners.push_back( corner );
			}
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": get_corners - GMM_ModelGetCornerHandles failed (%ld)!!\n", IPL_GetErrorCode( ));
	}
	else
		ret = IPL_FALSE;

	dprintf((" -- " TOOL_PREFIX ": get_corners (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int get_arcs( )
{
	dprintf((" ++ " TOOL_PREFIX ": get_arcs\n"));
	int ret = IPL_FALSE;

	vArcHandles.clear( );
	vArcs.clear( ); // free aufrufen

	if( !( ret = GMM_ModelCountArcs( model, &arc_cnt )))
		fprintf( stderr, "*** " TOOL_PREFIX ": get_arcs - GMM_ModelCountArcs failed (%ld)!!\n", IPL_GetErrorCode( ));

	if( arc_cnt > 0 && ret )
	{
		vArcHandles.resize( arc_cnt );
		if(( ret = GMM_ModelGetArcHandles( model, &(*vArcHandles.begin( )))))
		{
			GMM_ARC arc;
			for( IPL_ULONG i = 0; i < arc_cnt; i++ )
			{
				GMM_ModelGetArc( model, vArcHandles[i], &arc );
				vArcs.push_back( arc );
			}
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": get_arcs - GMM_ModelGetArcHandles failed (%ld)!!\n", IPL_GetErrorCode( ));
	}
//	else
//		ret = IPL_FALSE;

	dprintf((" -- " TOOL_PREFIX ": get_arcs (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
static int teach_process( )
{
	dprintf((" ++ " TOOL_PREFIX ": teach_process\n"));
	int ret = IPL_FALSE;

	if(( ret = get_edge_points( )))
	{
		if(( ret = set_matcher_control( )))
		{
			if(( ret = get_corners( )))
			{
				if(( ret = get_arcs( )))
				{
					if(( ret = GMM_ModelSave( model, TOOL_PREFIX ".gmm" )))
					{
						dprintf(("++ teach_flag: %d\n", teach_flag ));
						teach_flag = 1;
						dprintf(("-- teach_flag: %d\n", teach_flag ));
					}
					else
						fprintf( stderr, "*** " TOOL_PREFIX ": teach_process - GMM_ModelSave failed (%ld)!!\n", IPL_GetErrorCode( ));
				}
				else
					fprintf( stderr, "*** " TOOL_PREFIX ": teach_process - get_arcs failed!!\n");
			}
			else
				fprintf( stderr, "*** " TOOL_PREFIX ": teach_process - get_corners failed!!\n");
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": teach_process - set_matcher_control failed!!\n");
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": teach_process - get_edge_points failed!!\n");

	dprintf((" -- " TOOL_PREFIX ": teach_process (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
void set_max_feat_nb( int nb ){	max_feat_nb = nb;}
int get_max_feat_nb( ){	return max_feat_nb;}

//----------------------------------------------------------------------
void set_max_obj_nb( int nb ){	max_obj_nb = nb; set_matcher_nb_match( ); }
int get_max_obj_nb( ){	return max_obj_nb;}
int get_obj_nb( ){	return obj_nb;}

//----------------------------------------------------------------------
void set_threshold( int th ){	threshold = th;}
int get_threshold( ){	return threshold; }

//----------------------------------------------------------------------
void set_quality( int q ){	quality = q; set_matcher_score( ); }
int get_quality( ){	return quality; }

//----------------------------------------------------------------------
int get_arc_cnt( ){	return arc_cnt; }
int get_corner_cnt( ){	return corner_cnt; }

//----------------------------------------------------------------------
double get_time_ms( ){	return gmm_time_ms; }

//----------------------------------------------------------------------
void set_displ_res_mode( int type ){	draw_res_type = type; }
int get_displ_res_mode( ){	return draw_res_type; }

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static GMM_MODELHANDLE lmodel = IPL_DONT_CARE;
void clear_label( )
{
	if( lmodel != IPL_DONT_CARE )
	{
		GMM_ModelFree( lmodel );
		lmodel = IPL_DONT_CARE;
	}
}

//----------------------------------------------------------------------
int set_label_img( )
{
	int ret = IPL_FALSE;

	if( lmodel == IPL_DONT_CARE )
	{
		if( !( ret = GMM_ModelAlloc( &lmodel )))
			fprintf( stderr, "*** " TOOL_PREFIX ": init_gmm - GMM_ModelAlloc failed (%ld)!!\n", IPL_GetErrorCode( ));

		if( !( ret = GMM_ModelCreateFromImage( lmodel, image, max_feat_nb, threshold )))
			fprintf( stderr, "*** " TOOL_PREFIX ": set_label_img - GMM_ModelCreateFromImage failed (%ld)!!\n", IPL_GetErrorCode( ));
	}
	else
	{
		if( !( ret = GMM_ModelSelectEdgePoints( lmodel, threshold )))
			fprintf( stderr, "*** " TOOL_PREFIX ": set_label_img - GMM_ModelCreateFromImage failed (%ld)!!\n", IPL_GetErrorCode( ));
	}

	if( !( ret = GMM_ModelGetEdgePoints( lmodel, &bin_image )))
		fprintf( stderr, "*** " TOOL_PREFIX ": set_label_img - GMM_ModelGetEdgePoints failed (%ld)!!\n", IPL_GetErrorCode( ));

	clear_canvas( );
	cp_bin_img( bin_image );
	displ_img( );
	redraw_canvas( );

	if( bin_image != IPL_DONT_CARE )
	{
		IPL_BufFree( bin_image );
		bin_image = IPL_DONT_CARE;
	}

	return ret;
}

//----------------------------------------------------------------------
#define TEXT_V_STEP						5
#define TEXT_H_STEP						15
void draw_results( )
{
	dprintf((" ++ " TOOL_PREFIX ": draw_results - new_res_flag: %d\n", new_res_flag ));
	pgEnterContext( );

	if( new_res_flag )
	{
		clear_canvas( );

		if( obj_nb > 0 )
		{
			if( draw_res_type == GMM_DISPL_MATCH_TABLE )
			{
#ifdef COPY_PTS2IMG
				cp2disp( 0 );
				for( unsigned int c = 0; c < obj_nb; c++ )
					cp_pts2img( gmm_mtres[c].vMatchPoints, c );

				displ_img( );

				for( unsigned int c = 0; c < obj_nb; c++ )
					gui_draw_text( TEXT_V_STEP, TEXT_H_STEP * c + 5, gmm_mtres[c].msg, c );
#else
				for( unsigned int c = 0; c < obj_nb; c++ )
				{
					for( unsigned long p = 0; p < points_cnt; p++ )
						gui_draw_point( (int)gmm_mtres[c].vMatchPoints[p].x, (int)gmm_mtres[c].vMatchPoints[p].y, c );

					gui_draw_text( TEXT_V_STEP, TEXT_H_STEP * c + 5, gmm_mtres[c].msg, c );
				}
#endif	// COPY_PTS2IMG
			}
			else if( draw_res_type == GMM_DISPL_TRIGS )
			{
				cp2disp( 0 );
				displ_img( );

				for( unsigned int c = 0; c < obj_nb; c++ )
				{
					for( IPL_ULONG i = 0; i < corner_cnt; i++ )
					{
						gui_draw_line( (int)gmm_tres[c].vEdge1[i].x1, (int)gmm_tres[c].vEdge1[i].y1,
										(int)gmm_tres[c].vEdge1[i].x2, (int)gmm_tres[c].vEdge1[i].y2, c );

						gui_draw_line( (int)gmm_tres[c].vEdge2[i].x1, (int)gmm_tres[c].vEdge2[i].y1,
										(int)gmm_tres[c].vEdge2[i].x2, (int)gmm_tres[c].vEdge2[i].y2, c );
					}

					for( IPL_ULONG i = 0; i < arc_cnt; i++ )
						gui_draw_circle( (int)gmm_tres[c].vX[i], (int)gmm_tres[c].vY[i], (int)gmm_tres[c].vR[i], c );

					gui_draw_text( TEXT_V_STEP, TEXT_H_STEP * c + 5, gmm_tres[c].msg, c );
				}
			}
			else
			{
				for( unsigned int c = 0; c < obj_nb; c++ )
					gui_draw_text( TEXT_V_STEP, TEXT_H_STEP * c + 5, res_txt[c], c );
			}
		}

		redraw_canvas( );
	}
	new_res_flag = 0;
	obj_nb = 0;
	clear_canvas( );

	pgLeaveContext( );

	dprintf((" -- " TOOL_PREFIX ": draw_results\n" ));
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int init_gmm( )
{
	dprintf((" ++ " TOOL_PREFIX ": init_gmm\n"));
	int ret;
	if( !( ret = GMM_MatcherAlloc( &matcher )))
		fprintf( stderr, "*** " TOOL_PREFIX ": init_gmm - GMM_MatcherAlloc failed (%ld)!!\n", IPL_GetErrorCode( ));
	if( !( ret = GMM_ModelAlloc( &model )))
		fprintf( stderr, "*** " TOOL_PREFIX ": init_gmm - GMM_ModelAlloc failed (%ld)!!\n", IPL_GetErrorCode( ));

	dprintf((" -- " TOOL_PREFIX ": init_gmm (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int teach_gmm( )
{
	dprintf((" ++ " TOOL_PREFIX ": teach_gmm - max_feat_nb: %d, threshold: %d, quality: %d, max_obj_nb: %d\n",
							max_feat_nb, threshold, quality, max_obj_nb ));

	int ret = IPL_FALSE;

	if( model != IPL_DONT_CARE && matcher != IPL_DONT_CARE )
	{
		if(( ret = GMM_ModelCreateFromImage( model, image, max_feat_nb, threshold )))
			ret = teach_process( );
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": teach_gmm - GMM_ModelCreateFromImage failed (%ld)!!\n", IPL_GetErrorCode( ));

	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": teach_gmm - empty handels failed!!\n");

	dprintf((" -- " TOOL_PREFIX ": teach_gmm (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
int find_gmm( )
{
	dprintf((" ++ " TOOL_PREFIX ": find_gmm\n"));
	static struct timeval start_tv, diff_tv;
	int ret = IPL_FALSE;

	if( teach_flag )
	{
		if( model != IPL_DONT_CARE && matcher != IPL_DONT_CARE )
		{
			StartTimer( &start_tv );

			if( !( ret = GMM_MatcherFindModel( matcher, model, image )))
				fprintf( stderr, "*** " TOOL_PREFIX ": find_gmm - GMM_MatcherFindModel failed (%ld)!!\n", IPL_GetErrorCode( ));

			ElapsedTime( &start_tv, &diff_tv, 30 );
			gmm_time_ms = (double)(diff_tv.tv_sec * 1000 + diff_tv.tv_usec/1000. );

			GMM_MATCH* pMatches = NULL;
			obj_nb = 0;

			if( !( ret = GMM_MatcherGetMatches( matcher, &pMatches, &obj_nb )))
				fprintf( stderr, "*** " TOOL_PREFIX ": find_gmm - GMM_MatcherGetMatches failed (%ld)!!\n", IPL_GetErrorCode( ));

			dprintf(("\nNumber of matches: %ld\n", obj_nb ));
			if( obj_nb > 0 )
			{
				if( draw_res_type == GMM_DISPL_MATCH_TABLE )
				{
					for( unsigned long i = 0; i < obj_nb; i++ )
					{
						get_match_table_results( pMatches[i], i );
						dprintf((" == x: %.3f, y: %.3f, r: %.3f, s: %.3f\n",
							pMatches[i].x, pMatches[i].y, pMatches[i].r, pMatches[i].s ));
					}
				}
				else if( draw_res_type == GMM_DISPL_TRIGS )
				{
					for( IPL_ULONG i = 0; i < obj_nb; i++ )
						get_trig_results( pMatches[i], i );
				}
				else
				{
					for( IPL_ULONG i = 0; i < obj_nb; i++ )
						sprintf( res_txt[i], "scaling: %3.2f, score: %3.2f", pMatches[i].s, pMatches[i].score );

					new_res_flag = 1;
				}
			}

			if( pMatches != NULL )
			{
				GMM_MatcherFreeMatches( matcher, pMatches );
				pMatches = NULL;
			}
		}
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": find_gmm - empty handels failed!!\n");
	}
	else
	{
		fprintf( stderr, "*** " TOOL_PREFIX ": find_gmm - nothing taugth yet!!\n");
		pgMessageDialog( "Find error...", "\nNothing taught yet!!\n", PG_MSGBTN_OK | PG_MSGICON_ERROR );
	}

	dprintf((" -- " TOOL_PREFIX ": find_gmm (%d)\n", ret ));
	return ret;
}

//----------------------------------------------------------------------
void free_gmm( )
{
	dprintf((" ++ " TOOL_PREFIX ": free_gmm\n"));

	clear_label( );

	for( IPL_ULONG i = 0; i < arc_cnt; i++ )
		GMM_ModelFreeArc( model, &vArcs[i] );

	if( model != IPL_DONT_CARE )
		GMM_ModelFree( model );
	if( matcher != IPL_DONT_CARE )
		GMM_MatcherFree( matcher );

	dprintf((" -- " TOOL_PREFIX ": free_gmm\n"));
}

//----------------------------------------------------------------------
int load_gmm_fl( )
{
	dprintf((" ++ " TOOL_PREFIX ": load_gmm_fl\n"));
	int ret = IPL_FALSE;

	if( model != IPL_DONT_CARE )
	{
		if(( ret = GMM_ModelCreateFromFile( model, TOOL_PREFIX ".gmm" )))
			ret = teach_process( );
		else
			fprintf( stderr, "*** " TOOL_PREFIX ": load_gmm_fl - load file '" TOOL_PREFIX ".gmm' failed (%ld)!!\n", IPL_GetErrorCode());
	}

	dprintf((" -- " TOOL_PREFIX ": load_gmm_fl (%d)\n", ret ));

	return ret;
}
//----------------------------------------------------------------------
//----------------------------------------------------------------------
