//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>ocrtool<<
// Tue May 20 14:35:47 CEST 2003
//----------------------------------------------------------------------

#ifndef __ocrtool_h__
#define __ocrtool_h__

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <vl_ocr.h>
#include <vl_display.h>


//---------------------------------------------------------------------
// minimal version of OCR library required
#define MIN_OCR_VERSION "6.4.0.4610"



//----------------------------------------------------------------------
#define TOOL_PREFIX							"ocrtool"
#define DEFAULT_OCR_FILE					"pattern.ocr"
#define DUMMY_BMP_FILE						"docr.bmp"
#define OCRTOOL_INIT						"ocrtool.ini"

#define READ_MODE_STR						"OCR read mode"
#define TEACH_MODE_STR						"OCR teach mode"
#define DELETE_MODE_STR						"OCR delete mode"
#define NO_MODE_STR							"NO OCR MODE SELECTED"
#define START_LIVE_STR						"live mode"
#define STOP_LIVE_STR						"live mode stopped"
#define B2W_STR								"black/white"
#define W2B_STR								"white/black"

#define NONE_STR							"none"
#define MANUAL_STR							"manual"

#define STATBEST_TH_STR						"stat. best."
#define DYNLINE_TH_STR						"dyn. line"

#define VERT_RANK_STR						"vert."
#define HOR_RANK_STR						"horiz."
#define VH_RANK_STR							"vert. & horiz."

#define SPLIT_SEG_STR						"split"
#define MERGE_SEG_STR						"merge"
#define SM_SEG_STR							"spl. & mrg."
#define MS_SEG_STR							"mrg. & spl."

#define OPEN_MORPH_STR						"open"
#define CLOSE_MORPH_STR						"close"

#define TEACH_MODE							0
#define READ_MODE							1
#define DELETE_MODE							2

#define BGRD_WHITE							0
#define BGRD_BLACK							1

#define MAX_OBJ_CNT							256
#define MAX_DISP_CNT						( 4 * MAX_OBJ_CNT )

#define CHAR_BTN_SIZE						30
#define UNKNOWN_CHAR						255
#define DEFAULT_BLANK_WIDTH					20

#define MIN_W								3
#define MIN_H								3

#define DEFAULT_WMIN						MIN_W//10
#define DEFAULT_WMAX						80
#define DEFAULT_HMIN						MIN_H//20
#define DEFAULT_HMAX						80

#define DEFAULT_WMINC						MIN_W
#define DEFAULT_WMAXC						30
#define DEFAULT_HMINC						MIN_H
#define DEFAULT_HMAXC						40

#define DEFAULT_QMIN						50

#define RET_SUCCESS							0
#define RET_ERROR							-1
#define RET_CLOSE							-2
#define RET_SKIPP							-3

/* GLOBALS */
#define INIT_STR_PROC						"PROCESS"

/* OCRSETTINGS */
#define INIT_STR_WMIN						"WMIN"
#define INIT_STR_WMAX						"WMAX"
#define INIT_STR_HMIN						"HMIN"
#define INIT_STR_HMAX						"HMAX"
#define INIT_STR_SEGTYPE					"SEGMANTATIONTYPE"
#define INIT_STR_CLSM						"CLASSIFMODE"
#define INIT_STR_POL						"POLARITY"
#define INIT_STR_DISPQ						"QUALITY"
#define INIT_STR_QMIN						"QMIN"
#define INIT_STR_WMINC						"CHAR_WMIN"
#define INIT_STR_WMAXC						"CHAR_WMAX"
#define INIT_STR_HMINC						"CHAR_HMIN"
#define INIT_STR_HMAXC						"CHAR_HMAX"

/* PREPPROCESSING */
#define INIT_STR_TH							"THRESHOLD"
#define INIT_STR_THTYPE						"THRESHOLDTYPE"
#define INIT_STR_RANK						"RANKTYPE"
#define INIT_STR_MORPH						"MORPHTYPE"
#define INIT_STR_OPLEN						"OPLEN"
#define INIT_STR_MINDIST					"MINDIST"

/* ROI */
#define INIT_STR_ROI						"ROI"
#define INIT_STR_ROI_LEFT					"ROI_LEFT"
#define INIT_STR_ROI_TOP					"ROI_TOP"
#define INIT_STR_ROI_RIGHT					"ROI_RIGHT"
#define INIT_STR_ROI_BOTTOM					"ROI_BOTTOM"

/* SENDDATA */
#define INIT_STR_SENDDATA					"SENDDATA"
#define INIT_STR_BAUDRATE 					"BAUDRATE"
#define INIT_STR_DEVNO 						"DEVNO"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define DMO_CHECK_ERR( a, e )														\
	{																				\
		if( !(a) )																	\
		{																			\
			return dmoCheckError( stderr, __FUNCTION__, __FILE__, __LINE__, #a, e );	\
		}																			\
	}

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#ifdef __cplusplus
extern "C"{
#endif

//----------------------------------------------------------------------
//----------------------------------------------------------------------
int evtRead   ( struct pgEvent* pEvt );
int evtTeach  ( struct pgEvent* pEvt );
int evtLiveOff( struct pgEvent* pEvt );
int evtLiveOn ( struct pgEvent* pEvt );
int evtLoad   ( struct pgEvent* pEvt );
int evtClose  ( struct pgEvent* pEvt );
int evtOPLen  ( struct pgEvent* pEvt );
int evtMinDist( struct pgEvent* pEvt );
int evtThresholdScroller( struct pgEvent* pEvt );

int get_chars( int *c, unsigned char *chimg_buffer, int width, int height, OCR_RESULT *result );
int get2delchar( int *c_list );


  /* needed for version check */
//-------------------------------------------------------------------------------------------
struct _SVersionNumber
//-------------------------------------------------------------------------------------------
{
	_SVersionNumber( int iMaj, int iMin, int iBld, int iBld2 )
		: iMajor(iMaj), iMinor(iMin), iBuild(iBld), iBuild2(iBld2)
	{
	}

	int iMajor;
	int iMinor;
	int iBuild;
	int iBuild2;

};

typedef struct _SVersionNumber SVersionNumber;





//----------------------------------------------------------------------
//----------------------------------------------------------------------
static int dmoCheckError( FILE* pF, const char* pFunction, const char* pFileName, const int line,
				const char* pMsg, const long errCode )
{
	char msg[512];
	memset( msg, 0, 512 );

	if( errCode == - 1 )
		sprintf( msg, "*** %s (%s: %d) - %s ?!\n",
							pFunction, pFileName, line, pMsg );
	else
		sprintf( msg, "*** %s (%s: %d) - %s ?!\n\t-> %s [%ld]\n",
							pFunction, pFileName, line, pMsg, IPL_GetErrorString( errCode ), errCode );

	fprintf( pF, "%s", msg );
	pgMessageDialog( "Error!!", msg, PG_MSGBTN_OK | PG_MSGICON_MESSAGE );

	return IPL_FALSE;
}


//----------------------------------------------------------------------
inline bool negate( const bool bValue ){ return bValue ? false : true; }

inline int getStatusC( const bool bValue ) { return bValue ? '*' : ' '; }

//----------------------------------------------------------------------
#ifdef __cplusplus
}
#endif

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#endif	// __ocrtool_h__
