//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>pgCalibration<<
// Di Nov  8 11:21:23 CET 2005
//----------------------------------------------------------------------
/*! \file pgCalibration.cpp
 *  \brief This sample shows how the \b calibration module can be used.
 *  \author MATRIX VISION GmbH (CP)
 */
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#include <stdio.h>
#include <samples.h>
#include <pgtools.h>

#include "mvIMPACT.h"
#include "mvIMPACT_calibration.h"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define TOOL_PREFIX							"pgCalibration"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
 using namespace mvIMPACT_C;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static IPL_BUFHANDLE hImage = IPL_DONT_CARE;					// mvIMPACT image handle.
static Image image;												// mvIMPACT image (C++).
static DEV_HANDLE hDevice = IPL_DONT_CARE;						// mvIMPACT device handle.
static IPL_HWND hWnd = 0;										// mvIMPACT windows handle.
static Window window;											// mvIMPACT display window (C++).
static pghandle wLive = 0, wStop = 0, wExit = 0, wCalib = 0, wChCalib = 0;	// PicoGUI widget handles.
static bool bRun = false;										// Flag for live snap.
static CPGTools* pPGT = 0;										// Class for PicoGUI objects.
static CameraCalibration ImageCalibration;						// mvIMPACT calibration (C++).
static bool bTrCalibration = false;								// Flag for calibration transformation.
static bool bTeach = false;										// Flag for calibration transformation.

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//! \brief Grab a image and display it.
static void SnapLoop( )
//----------------------------------------------------------------------
{
	while( bRun )
	{
		// Check if there are GUI events.
		pgEventPoll( );

		// Grab and dispay the image.
		if( bRun )
			CHECKSDKCALL( DEV_Grab( hDevice, &hImage ));

		if( bTrCalibration && bRun && bTeach )
			ImageCalibration.transformImage( image, image );

		if( bRun )
			window.redraw();
	}
}

//----------------------------------------------------------------------
//! \brief Event funtion to set the transformation flag for the calibration.
int evtTrCalibration( struct pgEvent* /*pEvt*/ )
//----------------------------------------------------------------------
{
	bTrCalibration = pgGetWidget( wChCalib, PG_WP_ON );
	return 0;
}

//----------------------------------------------------------------------
//! \brief Event funtion to teach the calibration data and transform the input image.
int evtCalibration( struct pgEvent* /*pEvt*/ )
//----------------------------------------------------------------------
{
	ImageCalibration.setGridPointDistanceX( 4.0 );
	ImageCalibration.setGridPointDistanceY( 4.0 );

	dprintf(( " == " TOOL_PREFIX ": Configured calibration and start the teach process\n" ));

	ImageCalibration.teachByGrid( image );

	unsigned int uiIterCount = ImageCalibration.getIterationCount();
	printf( " == " TOOL_PREFIX ": Iteration Count: %d\n", uiIterCount );

	CalibrationStatistics stat = ImageCalibration.getStatistics( );

	printf( " == " TOOL_PREFIX ": meanXDifference: %.4lf\tmeanStddevXDifference: %.4lf\n", stat.getMeanXDifference(), stat.getStddevXDifference());
	printf( " == " TOOL_PREFIX ": meanYDifference: %.4lf\tmeanStddevYDifference: %.4lf\n", stat.getMeanYDifference(), stat.getStddevYDifference());
	printf( " == " TOOL_PREFIX ": meanEuclideanDistance: %.4lf\tstddevEuclideanDistance: %.4lf\n", stat.getMeanEuclideanDistance(), stat.getStddevEuclideanDistance());

	dprintf(( " == " TOOL_PREFIX ": Transform/Calibrate the image\n"));
	// Transform/calibrate the image.
	ImageCalibration.transformImage( image, image );

	window.redraw();
	bTeach = true;

	return 0;
}

//----------------------------------------------------------------------
//! \brief Event funtion to start the live snap.
int evtLiveOn( struct pgEvent* /*pEvt*/ )
//----------------------------------------------------------------------
{
	bRun = true;
	SnapLoop( );
	return 0;
}

//----------------------------------------------------------------------
//! \brief Event funtion to stop the live snap.
int evtLiveOff( struct pgEvent* /*pEvt*/ )
//----------------------------------------------------------------------
{
	bRun = false;
	return 0;
}

//----------------------------------------------------------------------
//! \brief Event funtion to close and exit the sample.
int evtClose( struct pgEvent* pEvt )
//----------------------------------------------------------------------
{
	evtLiveOff( pEvt );

	// Leave the GUI event loop.
	pgExitEventLoop( );

	// Free the class resources.
	if( pPGT )	delete pPGT;

	// Close the opened devic.
	if( hDevice != IPL_DONT_CARE )
		DEV_Close( hDevice );

	// Free the image buffer.
	IPL_BufFree( hImage );

	return 0;
}

//----------------------------------------------------------------------
int main( int /*argc*/, char * /*argv[]*/ )
//----------------------------------------------------------------------
{
	// Open a device.
	CHECKSDKCALL( DEV_Open( &hDevice, "matrixvision.ddc" ));
	CHECKSDKCALL( DEV_Control( hDevice, DEV_SINGLE, 0 ));

	// Grab a image and make a C++ image.
	CHECKSDKCALL( DEV_Grab( hDevice, &hImage ));
	image = Image::imageFactory( hImage );

	// Open a dispay window.
	window = Window( image );

	if(( pPGT = new CPGTools( )) != 0 )
	{
		// Make 3 buttons.
		pghandle wBox = pgNewWidget( PG_WIDGET_BOX, PG_DERIVE_BEFORE, hWnd );
		pgSetWidget( PGDEFAULT,
					PG_WP_SIZE, 40,
					0 );

		wLive  = pPGT->PGT_mkButton(      wBox, PG_DERIVE_INSIDE, "Live",       &evtLiveOn,      PGT_COLOR_BLACK, 16 );
		wStop  = pPGT->PGT_mkButton( PGDEFAULT, PG_DERIVE_AFTER, "Stop",        &evtLiveOff,     PGT_COLOR_BLACK, 16 );
		wExit  = pPGT->PGT_mkButton( PGDEFAULT, PG_DERIVE_AFTER, "Exit",        &evtClose,       PGT_COLOR_RED,   16 );
		wCalib = pPGT->PGT_mkButton( PGDEFAULT, PG_DERIVE_AFTER, "Calibration", &evtCalibration, PGT_COLOR_BLACK, 16 );

		wChCalib = pPGT->PGT_mkCheckBox( PGDEFAULT, PG_DERIVE_AFTER, "Transform snapped image", evtTrCalibration, 0 );
		pgSetWidget( wChCalib,
					PG_WP_FONT, pgNewFont( NULL, 16, 0 ),
					0 );

		// Update the display.
		window.redraw( );

		// Start the event loop.
		pgEventLoop( );
	}
	else
		fprintf( stderr, "*** " TOOL_PREFIX ": out of memory!!\n" );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------

